@@ -1,4 +1,5 @@
 # Copyright 2012 OpenStack Foundation
+# Copyright 2013 IBM Corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,6 +14,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
+
 from glance.common import exception
 import glance.domain.proxy
 
@@ -53,6 +56,31 @@ def proxy_member(context, member):
         return ImmutableMemberProxy(member)
 
 
+def is_task_mutable(context, task):
+    """Return True if the task is mutable in this context."""
+    if context.is_admin:
+        return True
+
+    if context.owner is None:
+        return False
+
+    return task.owner == context.owner
+
+
+def proxy_task(context, task):
+    if is_task_mutable(context, task):
+        return task
+    else:
+        return ImmutableTaskProxy(task)
+
+
+def proxy_task_details(context, task, task_details):
+    if is_task_mutable(context, task):
+        return task_details
+    else:
+        return ImmutableTaskDetailsProxy(task_details)
+
+
 class ImageRepoProxy(glance.domain.proxy.Repo):
 
     def __init__(self, image_repo, context):
@@ -82,8 +110,8 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
 
     def get(self, member_id):
         if (self.context.is_admin or
-            self.context.owner == self.image.owner or
-            self.context.owner == member_id):
+                self.context.owner == self.image.owner or
+                self.context.owner == member_id):
             member = self.member_repo.get(member_id)
             return proxy_member(self.context, member)
         else:
@@ -93,7 +121,7 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
     def list(self, *args, **kwargs):
         members = self.member_repo.list(*args, **kwargs)
         if (self.context.is_admin or
-            self.context.owner == self.image.owner):
+                self.context.owner == self.image.owner):
             return [proxy_member(self.context, m) for m in members]
         for member in members:
             if member.member_id == self.context.owner:
@@ -103,7 +131,7 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
 
     def remove(self, image_member):
         if (self.image.owner == self.context.owner or
-            self.context.is_admin):
+                self.context.is_admin):
             self.member_repo.remove(image_member)
         else:
             message = _("You cannot delete image member for %s")
@@ -112,8 +140,8 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
 
     def add(self, image_member):
         if (self.image.owner == self.context.owner or
-            self.context.is_admin):
-            return self.member_repo.add(image_member)
+                self.context.is_admin):
+            self.member_repo.add(image_member)
         else:
             message = _("You cannot add image member for %s")
             raise exception.Forbidden(message
@@ -121,9 +149,8 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
 
     def save(self, image_member):
         if (self.context.is_admin or
-            self.context.owner == image_member.member_id):
-            updated_member = self.member_repo.save(image_member)
-            return proxy_member(self.context, updated_member)
+                self.context.owner == image_member.member_id):
+            self.member_repo.save(image_member)
         else:
             message = _("You cannot update image member %s")
             raise exception.Forbidden(message % image_member.member_id)
@@ -182,8 +209,11 @@ def _immutable_attr(target, attr, proxy=None):
         return value
 
     def forbidden(self, *args, **kwargs):
-        message = _("You are not permitted to modify '%s' on this image.")
-        raise exception.Forbidden(message % attr)
+        resource = getattr(self, 'resource_name', 'resource')
+        message = _("You are not permitted to modify '%(attr)s' on this "
+                    "%(resource)s.")
+        raise exception.Forbidden(message % {'attr': attr,
+                                             'resource': resource})
 
     return property(get_attr, forbidden, forbidden)
 
@@ -194,6 +224,9 @@ class ImmutableLocations(list):
                     "for this image.")
         raise exception.Forbidden(message)
 
+    def __deepcopy__(self, memo):
+        return ImmutableLocations(copy.deepcopy(list(self), memo))
+
     append = forbidden
     extend = forbidden
     insert = forbidden
@@ -245,6 +278,7 @@ class ImmutableImageProxy(object):
     def __init__(self, base, context):
         self.base = base
         self.context = context
+        self.resource_name = 'image'
 
     name = _immutable_attr('base', 'name')
     image_id = _immutable_attr('base', 'image_id')
@@ -262,6 +296,7 @@ class ImmutableImageProxy(object):
     disk_format = _immutable_attr('base', 'disk_format')
     container_format = _immutable_attr('base', 'container_format')
     size = _immutable_attr('base', 'size')
+    virtual_size = _immutable_attr('base', 'virtual_size')
     extra_properties = _immutable_attr('base', 'extra_properties',
                                        proxy=ImmutableProperties)
     tags = _immutable_attr('base', 'tags', proxy=ImmutableTags)
@@ -285,6 +320,7 @@ class ImmutableImageProxy(object):
 class ImmutableMemberProxy(object):
     def __init__(self, base):
         self.base = base
+        self.resource_name = 'image member'
 
     id = _immutable_attr('base', 'id')
     image_id = _immutable_attr('base', 'image_id')
@@ -294,6 +330,44 @@ class ImmutableMemberProxy(object):
     updated_at = _immutable_attr('base', 'updated_at')
 
 
+class ImmutableTaskProxy(object):
+    def __init__(self, base):
+        self.base = base
+        self.resource_name = 'task'
+
+    task_id = _immutable_attr('base', 'task_id')
+    type = _immutable_attr('base', 'type')
+    status = _immutable_attr('base', 'status')
+    owner = _immutable_attr('base', 'owner')
+    expires_at = _immutable_attr('base', 'expires_at')
+    created_at = _immutable_attr('base', 'created_at')
+    updated_at = _immutable_attr('base', 'updated_at')
+
+    def run(self, executor):
+        self.base.run(executor)
+
+    def begin_processing(self):
+        message = _("You are not permitted to set status on this task.")
+        raise exception.Forbidden(message)
+
+    def succeed(self, result):
+        message = _("You are not permitted to set status on this task.")
+        raise exception.Forbidden(message)
+
+    def fail(self, message):
+        message = _("You are not permitted to set status on this task.")
+        raise exception.Forbidden(message)
+
+
+class ImmutableTaskDetailsProxy(object):
+    def __init__(self, base):
+        self.base = base
+
+    input = _immutable_attr('base', 'input')
+    message = _immutable_attr('base', 'message')
+    result = _immutable_attr('base', 'result')
+
+
 class ImageProxy(glance.domain.proxy.Image):
 
     def __init__(self, image, context):
@@ -308,3 +382,59 @@ class ImageProxy(glance.domain.proxy.Image):
         else:
             member_repo = self.image.get_member_repo(**kwargs)
             return ImageMemberRepoProxy(member_repo, self, self.context)
+
+
+class TaskProxy(glance.domain.proxy.Task):
+
+    def __init__(self, task):
+        self.task = task
+        super(TaskProxy, self).__init__(task)
+
+
+class TaskDetailsProxy(glance.domain.proxy.TaskDetails):
+
+    def __init__(self, task_details):
+        self.task_details = task_details
+        super(TaskDetailsProxy, self).__init__(task_details)
+
+
+class TaskFactoryProxy(glance.domain.proxy.TaskFactory):
+
+    def __init__(self, task_factory, context):
+        self.task_factory = task_factory
+        self.context = context
+        super(TaskFactoryProxy, self).__init__(
+            task_factory,
+            task_proxy_class=TaskProxy,
+            task_details_proxy_class=TaskDetailsProxy)
+
+    def new_task(self, **kwargs):
+        owner = kwargs.get('owner', self.context.owner)
+
+        #NOTE(nikhil): Unlike Images, Tasks are expected to have owner.
+        # We currently do not allow even admins to set the owner to None.
+        if owner is not None and (owner == self.context.owner
+                                  or self.context.is_admin):
+            return super(TaskFactoryProxy, self).new_task(**kwargs)
+        else:
+            message = _("You are not permitted to create this task with "
+                        "owner as: %s")
+            raise exception.Forbidden(message % owner)
+
+
+class TaskRepoProxy(glance.domain.proxy.TaskRepo):
+
+    def __init__(self, task_repo, context):
+        self.task_repo = task_repo
+        self.context = context
+        super(TaskRepoProxy, self).__init__(task_repo)
+
+    def get_task_and_details(self, task_id):
+        task, task_details = self.task_repo.get_task_and_details(task_id)
+        return proxy_task(self.context, task), proxy_task_details(self.context,
+                                                                  task,
+                                                                  task_details)
+
+    def list_tasks(self, *args, **kwargs):
+        tasks = self.task_repo.list_tasks(*args, **kwargs)
+        return [proxy_task(self.context, t) for t in tasks]
