@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -36,8 +34,8 @@ from glance.common import utils
 from glance.common import wsgi
 import glance.db
 from glance import image_cache
-import glance.openstack.common.log as logging
 from glance import notifier
+import glance.openstack.common.log as logging
 import glance.registry.client.v1.api as registry
 
 LOG = logging.getLogger(__name__)
@@ -210,7 +208,8 @@ class CacheFilter(wsgi.Middleware):
         images Resource, removing image file from the cache
         if necessary
         """
-        if not 200 <= self.get_status_code(resp) < 300:
+        status_code = self.get_status_code(resp)
+        if not 200 <= status_code < 300:
             return resp
 
         try:
@@ -218,6 +217,12 @@ class CacheFilter(wsgi.Middleware):
         except TypeError:
             return resp
 
+        if method == 'GET' and status_code == 204:
+            # Bugfix:1251055 - Don't cache non-existent image files.
+            # NOTE: Both GET for an image without locations and DELETE return
+            # 204 but DELETE should be processed.
+            return resp
+
         method_str = '_process_%s_response' % method
         try:
             process_response_method = getattr(self, method_str)
@@ -235,11 +240,11 @@ class CacheFilter(wsgi.Middleware):
         return resp
 
     def _process_GET_response(self, resp, image_id):
-        image_checksum = resp.headers.get('Content-MD5', None)
+        image_checksum = resp.headers.get('Content-MD5')
 
         if not image_checksum:
             # API V1 stores the checksum in a different header:
-            image_checksum = resp.headers.get('x-image-meta-checksum', None)
+            image_checksum = resp.headers.get('x-image-meta-checksum')
 
         if not image_checksum:
             LOG.error(_("Checksum header is missing."))
