@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -58,7 +56,7 @@ class VersionNegotiationFilter(wsgi.Middleware):
         else:
             LOG.debug(_("Using url versioning"))
             # Remove version in url so it doesn't conflict later
-            req_version = req.path_info_pop()
+            req_version = self._pop_path_info(req)
 
         try:
             version = self._match_version_string(req_version)
@@ -69,7 +67,7 @@ class VersionNegotiationFilter(wsgi.Middleware):
         req.environ['api.version'] = version
         req.path_info = ''.join(('/v', str(version), req.path_info))
         LOG.debug(_("Matched version: v%d"), version)
-        LOG.debug('new uri %s' % req.path_info)
+        LOG.debug('new path %s' % req.path_info)
         return None
 
     def _match_version_string(self, subject):
@@ -83,9 +81,26 @@ class VersionNegotiationFilter(wsgi.Middleware):
         """
         if subject in ('v1', 'v1.0', 'v1.1') and CONF.enable_v1_api:
             major_version = 1
-        elif subject in ('v2', 'v2.0', 'v2.1') and CONF.enable_v2_api:
+        elif subject in ('v2', 'v2.0', 'v2.1', 'v2.2') and CONF.enable_v2_api:
             major_version = 2
         else:
             raise ValueError()
 
         return major_version
+
+    def _pop_path_info(self, req):
+        """
+        'Pops' off the next segment of PATH_INFO, returns the popped
+        segment. Do NOT push it onto SCRIPT_NAME.
+        """
+        path = req.path_info
+        if not path:
+            return None
+        while path.startswith('/'):
+            path = path[1:]
+        idx = path.find('/')
+        if idx == -1:
+            idx = len(path)
+        r = path[:idx]
+        req.path_info = path[idx:]
+        return r
