@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 #    Copyright 2013 Rackspace
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -23,14 +21,13 @@ class ProtectedImageFactoryProxy(glance.domain.proxy.ImageFactory):
     def __init__(self, image_factory, context, property_rules):
         self.image_factory = image_factory
         self.context = context
-        self.roles = self.context.roles
         self.property_rules = property_rules
         kwargs = {'context': self.context,
                   'property_rules': self.property_rules}
         super(ProtectedImageFactoryProxy, self).__init__(
-                                        image_factory,
-                                        proxy_class=ProtectedImageProxy,
-                                        proxy_kwargs=kwargs)
+            image_factory,
+            proxy_class=ProtectedImageProxy,
+            proxy_kwargs=kwargs)
 
     def new_image(self, **kwargs):
         extra_props = kwargs.pop('extra_properties', {})
@@ -38,7 +35,7 @@ class ProtectedImageFactoryProxy(glance.domain.proxy.ImageFactory):
         extra_properties = {}
         for key in extra_props.keys():
             if self.property_rules.check_property_rules(key, 'create',
-                                                        self.roles):
+                                                        self.context):
                 extra_properties[key] = extra_props[key]
             else:
                 raise exception.ReservedProperty(property=key)
@@ -54,8 +51,8 @@ class ProtectedImageRepoProxy(glance.domain.proxy.Repo):
         self.property_rules = property_rules
         proxy_kwargs = {'context': self.context}
         super(ProtectedImageRepoProxy, self).__init__(
-                image_repo, item_proxy_class=ProtectedImageProxy,
-                item_proxy_kwargs=proxy_kwargs)
+            image_repo, item_proxy_class=ProtectedImageProxy,
+            item_proxy_kwargs=proxy_kwargs)
 
     def get(self, image_id):
         return ProtectedImageProxy(self.image_repo.get(image_id),
@@ -72,30 +69,29 @@ class ProtectedImageProxy(glance.domain.proxy.Image):
     def __init__(self, image, context, property_rules):
         self.image = image
         self.context = context
-        self.roles = self.context.roles
         self.property_rules = property_rules
 
         self.image.extra_properties = ExtraPropertiesProxy(
-                                            self.roles,
-                                            self.image.extra_properties,
-                                            self.property_rules)
+            self.context,
+            self.image.extra_properties,
+            self.property_rules)
         super(ProtectedImageProxy, self).__init__(self.image)
 
 
 class ExtraPropertiesProxy(glance.domain.ExtraProperties):
 
-    def __init__(self, roles, extra_props, property_rules):
-        self.roles = roles
+    def __init__(self, context, extra_props, property_rules):
+        self.context = context
         self.property_rules = property_rules
         extra_properties = {}
         for key in extra_props.keys():
             if self.property_rules.check_property_rules(key, 'read',
-                                                        self.roles):
+                                                        self.context):
                 extra_properties[key] = extra_props[key]
         super(ExtraPropertiesProxy, self).__init__(extra_properties)
 
     def __getitem__(self, key):
-        if self.property_rules.check_property_rules(key, 'read', self.roles):
+        if self.property_rules.check_property_rules(key, 'read', self.context):
             return dict.__getitem__(self, key)
         else:
             raise KeyError
@@ -108,13 +104,13 @@ class ExtraPropertiesProxy(glance.domain.ExtraProperties):
         try:
             if self.__getitem__(key):
                 if self.property_rules.check_property_rules(key, 'update',
-                                                            self.roles):
+                                                            self.context):
                     return dict.__setitem__(self, key, value)
                 else:
                     raise exception.ReservedProperty(property=key)
         except KeyError:
             if self.property_rules.check_property_rules(key, 'create',
-                                                        self.roles):
+                                                        self.context):
                 return dict.__setitem__(self, key, value)
             else:
                 raise exception.ReservedProperty(property=key)
@@ -124,7 +120,7 @@ class ExtraPropertiesProxy(glance.domain.ExtraProperties):
             raise KeyError
 
         if self.property_rules.check_property_rules(key, 'delete',
-                                                    self.roles):
+                                                    self.context):
             return dict.__delitem__(self, key)
         else:
             raise exception.ReservedProperty(property=key)
