@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -23,14 +21,12 @@ import copy
 
 import eventlet
 from oslo.config import cfg
-from webob.exc import (HTTPError,
-                       HTTPNotFound,
-                       HTTPConflict,
-                       HTTPBadRequest,
-                       HTTPForbidden,
-                       HTTPRequestEntityTooLarge,
-                       HTTPInternalServerError,
-                       HTTPServiceUnavailable)
+import six.moves.urllib.parse as urlparse
+from webob.exc import HTTPBadRequest
+from webob.exc import HTTPConflict
+from webob.exc import HTTPForbidden
+from webob.exc import HTTPNotFound
+from webob.exc import HTTPRequestEntityTooLarge
 from webob import Response
 
 from glance.api import common
@@ -47,10 +43,12 @@ from glance import notifier
 import glance.openstack.common.log as logging
 from glance.openstack.common import strutils
 import glance.registry.client.v1.api as registry
-from glance.store import (get_from_backend,
-                          get_size_from_backend,
-                          get_store_from_location,
-                          get_store_from_scheme)
+from glance.store import get_from_backend
+from glance.store import get_known_schemes
+from glance.store import get_known_stores
+from glance.store import get_size_from_backend
+from glance.store import get_store_from_location
+from glance.store import get_store_from_scheme
 
 LOG = logging.getLogger(__name__)
 SUPPORTED_PARAMS = glance.api.v1.SUPPORTED_PARAMS
@@ -58,8 +56,10 @@ SUPPORTED_FILTERS = glance.api.v1.SUPPORTED_FILTERS
 ACTIVE_IMMUTABLE = glance.api.v1.ACTIVE_IMMUTABLE
 
 CONF = cfg.CONF
-CONF.import_opt('disk_formats', 'glance.domain')
-CONF.import_opt('container_formats', 'glance.domain')
+CONF.import_opt('disk_formats', 'glance.common.config', group='image_format')
+CONF.import_opt('container_formats', 'glance.common.config',
+                group='image_format')
+CONF.import_opt('image_property_quota', 'glance.common.config')
 
 
 def validate_image_meta(req, values):
@@ -69,13 +69,14 @@ def validate_image_meta(req, values):
     container_format = values.get('container_format')
 
     if 'disk_format' in values:
-        if disk_format not in CONF.disk_formats:
-            msg = "Invalid disk format '%s' for image." % disk_format
+        if disk_format not in CONF.image_format.disk_formats:
+            msg = _("Invalid disk format '%s' for image.") % disk_format
             raise HTTPBadRequest(explanation=msg, request=req)
 
     if 'container_format' in values:
-        if container_format not in CONF.container_formats:
-            msg = "Invalid container format '%s' for image." % container_format
+        if container_format not in CONF.image_format.container_formats:
+            msg = _("Invalid container format '%s' "
+                    "for image.") % container_format
             raise HTTPBadRequest(explanation=msg, request=req)
 
     if name and len(name) > 255:
@@ -138,7 +139,7 @@ class Controller(controller.BaseController):
         self.policy = policy.Enforcer()
         self.pool = eventlet.GreenPool(size=1024)
         if property_utils.is_property_protection_enabled():
-            self.prop_enforcer = property_utils.PropertyRules()
+            self.prop_enforcer = property_utils.PropertyRules(self.policy)
         else:
             self.prop_enforcer = None
 
@@ -149,6 +150,34 @@ class Controller(controller.BaseController):
         except exception.Forbidden:
             raise HTTPForbidden()
 
+    def _enforce_image_property_quota(self,
+                                      image_meta,
+                                      orig_image_meta=None,
+                                      purge_props=False,
+                                      req=None):
+        if CONF.image_property_quota < 0:
+            # If value is negative, allow unlimited number of properties
+            return
+
+        props = image_meta['properties'].keys()
+
+        # NOTE(ameade): If we are not removing existing properties,
+        # take them in to account
+        if (not purge_props) and orig_image_meta:
+            original_props = orig_image_meta['properties'].keys()
+            props.extend(original_props)
+            props = set(props)
+
+        if len(props) > CONF.image_property_quota:
+            msg = (_("The limit has been exceeded on the number of allowed "
+                     "image properties. Attempted: %(num)s, Maximum: "
+                     "%(quota)s") % {'num': len(props),
+                                     'quota': CONF.image_property_quota})
+            LOG.info(msg)
+            raise HTTPRequestEntityTooLarge(explanation=msg,
+                                            request=req,
+                                            content_type="text/plain")
+
     def _enforce_create_protected_props(self, create_props, req):
         """
         Check request is permitted to create certain properties
@@ -161,8 +190,8 @@ class Controller(controller.BaseController):
         if property_utils.is_property_protection_enabled():
             for key in create_props:
                 if (self.prop_enforcer.check_property_rules(
-                        key, 'create', req.context.roles) is False):
-                    msg = _("Property '%s' is protected" % key)
+                        key, 'create', req.context) is False):
+                    msg = _("Property '%s' is protected") % key
                     LOG.debug(msg)
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
@@ -178,7 +207,7 @@ class Controller(controller.BaseController):
         if property_utils.is_property_protection_enabled():
             for key in image_meta['properties'].keys():
                 if (self.prop_enforcer.check_property_rules(
-                        key, 'read', req.context.roles) is False):
+                        key, 'read', req.context) is False):
                     image_meta['properties'].pop(key)
 
     def _enforce_update_protected_props(self, update_props, image_meta,
@@ -201,12 +230,12 @@ class Controller(controller.BaseController):
         if property_utils.is_property_protection_enabled():
             for key in update_props:
                 has_read = self.prop_enforcer.check_property_rules(
-                        key, 'read', req.context.roles)
+                    key, 'read', req.context)
                 if ((self.prop_enforcer.check_property_rules(
-                        key, 'update', req.context.roles) is False and
+                        key, 'update', req.context) is False and
                         image_meta['properties'][key] !=
                         orig_meta['properties'][key]) or not has_read):
-                    msg = _("Property '%s' is protected" % key)
+                    msg = _("Property '%s' is protected") % key
                     LOG.debug(msg)
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
@@ -233,14 +262,14 @@ class Controller(controller.BaseController):
         if property_utils.is_property_protection_enabled():
             for key in delete_props:
                 if (self.prop_enforcer.check_property_rules(
-                        key, 'read', req.context.roles) is False):
+                        key, 'read', req.context) is False):
                     # NOTE(bourke): if read protected, re-add to image_meta to
                     # prevent deletion
                     image_meta['properties'][key] = \
                         orig_meta['properties'][key]
                 elif (self.prop_enforcer.check_property_rules(
-                        key, 'delete', req.context.roles) is False):
-                    msg = _("Property '%s' is protected" % key)
+                        key, 'delete', req.context) is False):
+                    msg = _("Property '%s' is protected") % key
                     LOG.debug(msg)
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
@@ -280,7 +309,7 @@ class Controller(controller.BaseController):
 
     def detail(self, req):
         """
-        Returns detailed information for all public, available images
+        Returns detailed information for all available images
 
         :param req: The WSGI/Webob Request object
         :retval The response body is a mapping of the following form::
@@ -348,9 +377,10 @@ class Controller(controller.BaseController):
             if param in SUPPORTED_FILTERS or param.startswith('property-'):
                 query_filters[param] = req.params.get(param)
                 if not filters.validate(param, query_filters[param]):
-                    raise HTTPBadRequest('Bad value passed to filter %s '
-                                         'got %s' % (param,
-                                                     query_filters[param]))
+                    raise HTTPBadRequest(_('Bad value passed to filter '
+                                           '%(filter)s got %(val)s')
+                                         % {'filter': param,
+                                            'val': query_filters[param]})
         return query_filters
 
     def meta(self, req, id):
@@ -381,8 +411,11 @@ class Controller(controller.BaseController):
         If the above constraint is violated, we reject with 400 "Bad Request".
         """
         if source:
-            for scheme in ['s3', 'swift', 'http', 'rbd', 'sheepdog', 'cinder']:
-                if source.lower().startswith(scheme):
+            pieces = urlparse.urlparse(source)
+            schemes = [scheme for scheme in get_known_schemes()
+                       if scheme != 'file']
+            for scheme in schemes:
+                if pieces.scheme == scheme:
                     return source
             msg = _("External sourcing not supported for store %s") % source
             LOG.debug(msg)
@@ -407,7 +440,7 @@ class Controller(controller.BaseController):
         try:
             image_data, image_size = get_from_backend(context, where)
         except exception.NotFound as e:
-            raise HTTPNotFound(explanation="%s" % e)
+            raise HTTPNotFound(explanation=e.msg)
         image_size = int(image_size) if image_size else None
         return image_data, image_size
 
@@ -455,12 +488,25 @@ class Controller(controller.BaseController):
         :raises HTTPBadRequest if image metadata is not valid
         """
         location = self._external_source(image_meta, req)
+        store = image_meta.get('store')
+        if store and store not in get_known_stores():
+            msg = _("Required store %s is invalid") % store
+            LOG.debug(msg)
+            raise HTTPBadRequest(explanation=msg,
+                                 content_type='text/plain')
 
         image_meta['status'] = ('active' if image_meta.get('size') == 0
                                 else 'queued')
 
         if location:
-            store = get_store_from_location(location)
+            try:
+                store = get_store_from_location(location)
+            except exception.BadStoreUri:
+                msg = _("Invalid location %s") % location
+                LOG.debug(msg)
+                raise HTTPBadRequest(explanation=msg,
+                                     request=req,
+                                     content_type="text/plain")
             # check the store exists before we hit the registry, but we
             # don't actually care what it is at this point
             self.get_store_or_400(req, store)
@@ -486,7 +532,7 @@ class Controller(controller.BaseController):
                                request=req,
                                content_type="text/plain")
         except exception.Invalid as e:
-            msg = (_("Failed to reserve image. Got error: %(e)s") % locals())
+            msg = _("Failed to reserve image. Got error: %(e)s") % {'e': e}
             for line in msg.split('\n'):
                 LOG.debug(line)
             raise HTTPBadRequest(explanation=msg,
@@ -526,7 +572,7 @@ class Controller(controller.BaseController):
             image_meta['size'] = image_size or image_meta['size']
         else:
             try:
-                req.get_content_type('application/octet-stream')
+                req.get_content_type(('application/octet-stream',))
             except exception.InvalidContentType:
                 upload_utils.safe_kill(req, image_meta['id'])
                 msg = _("Content-Type must be application/octet-stream")
@@ -545,7 +591,8 @@ class Controller(controller.BaseController):
                                        {'status': 'saving'})
 
         LOG.debug(_("Uploading image data for image %(image_id)s "
-                    "to %(scheme)s store"), locals())
+                    "to %(scheme)s store"), {'image_id': image_id,
+                                             'scheme': scheme})
 
         self.notifier.info("image.prepare", redact_loc(image_meta))
 
@@ -556,7 +603,8 @@ class Controller(controller.BaseController):
 
         return location, loc_meta
 
-    def _activate(self, req, image_id, location, location_metadata=None):
+    def _activate(self, req, image_id, location, location_metadata=None,
+                  from_state=None):
         """
         Sets the image status to `active` and the image's location
         attribute.
@@ -564,7 +612,7 @@ class Controller(controller.BaseController):
         :param req: The WSGI/Webob Request object
         :param image_id: Opaque image identifier
         :param location: Location of where Glance stored this image
-        :param location_metadata: a dictionary of storage specfic information
+        :param location_metadata: a dictionary of storage specific information
         """
         image_meta = {}
         image_meta['location'] = location
@@ -574,15 +622,26 @@ class Controller(controller.BaseController):
                                             'metadata': location_metadata}]
 
         try:
+            s = from_state
             image_meta_data = registry.update_image_metadata(req.context,
                                                              image_id,
-                                                             image_meta)
+                                                             image_meta,
+                                                             from_state=s)
             self.notifier.info("image.activate", redact_loc(image_meta_data))
             self.notifier.info("image.update", redact_loc(image_meta_data))
             return image_meta_data
+        except exception.Duplicate:
+            # Delete image data since it has been supersceded by another
+            # upload.
+            LOG.debug(_("duplicate operation - deleting image data for %(id)s "
+                      "(location:%(location)s)") %
+                      {'id': image_id, 'location': image_meta['location']})
+            upload_utils.initiate_deletion(req, image_meta['location'],
+                                           image_id, CONF.delayed_delete)
+            # Then propagate the exception.
+            raise
         except exception.Invalid as e:
-            msg = (_("Failed to activate image. Got error: %(e)s")
-                   % locals())
+            msg = _("Failed to activate image. Got error: %(e)s") % {'e': e}
             LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
@@ -608,7 +667,8 @@ class Controller(controller.BaseController):
         return self._activate(req,
                               image_id,
                               location,
-                              location_metadata) if location else None
+                              location_metadata,
+                              from_state='saving') if location else None
 
     def _get_size(self, context, image_meta, location):
         # retrieve the image size from remote store (if not provided)
@@ -648,9 +708,9 @@ class Controller(controller.BaseController):
                             image_size_store != image_size_meta):
                         msg = _("Provided image size must match the stored "
                                 "image size. (provided size: %(ps)d, "
-                                "stored size: %(ss)d)" %
-                                {"ps": image_size_meta,
-                                 "ss": image_size_store})
+                                "stored size: %(ss)d)") % {
+                                    "ps": image_size_meta,
+                                    "ss": image_size_store}
                         LOG.debug(msg)
                         raise HTTPConflict(explanation=msg,
                                            request=req,
@@ -720,10 +780,14 @@ class Controller(controller.BaseController):
             self._enforce(req, 'publicize_image')
         if Controller._copy_from(req):
             self._enforce(req, 'copy_from')
+        if image_data or Controller._copy_from(req):
+            self._enforce(req, 'upload_image')
 
         self._enforce_create_protected_props(image_meta['properties'].keys(),
                                              req)
 
+        self._enforce_image_property_quota(image_meta, req=req)
+
         image_meta = self._reserve(req, image_meta)
         id = image_meta['id']
 
@@ -753,6 +817,10 @@ class Controller(controller.BaseController):
         is_public = image_meta.get('is_public')
         if is_public:
             self._enforce(req, 'publicize_image')
+        if Controller._copy_from(req):
+            self._enforce(req, 'copy_from')
+        if image_data or Controller._copy_from(req):
+            self._enforce(req, 'upload_image')
 
         orig_image_meta = self.get_image_meta_or_404(req, id)
         orig_status = orig_image_meta['status']
@@ -770,7 +838,7 @@ class Controller(controller.BaseController):
             # modify certain core metadata keys
             for key in ACTIVE_IMMUTABLE:
                 if (orig_status == 'active' and image_meta.get(key) is not None
-                    and image_meta.get(key) != orig_image_meta.get(key)):
+                        and image_meta.get(key) != orig_image_meta.get(key)):
                     msg = _("Forbidden to modify '%s' of active image.") % key
                     raise HTTPForbidden(explanation=msg,
                                         request=req,
@@ -800,10 +868,17 @@ class Controller(controller.BaseController):
         activating = orig_status == 'queued' and (location or image_data)
 
         # Make image public in the backend store (if implemented)
-        orig_or_updated_loc = location or orig_image_meta.get('location', None)
+        orig_or_updated_loc = location or orig_image_meta.get('location')
         if orig_or_updated_loc:
-            self.update_store_acls(req, id, orig_or_updated_loc,
-                                   public=is_public)
+            try:
+                self.update_store_acls(req, id, orig_or_updated_loc,
+                                       public=is_public)
+            except exception.BadStoreUri:
+                msg = _("Invalid location %s") % location
+                LOG.debug(msg)
+                raise HTTPBadRequest(explanation=msg,
+                                     request=req,
+                                     content_type="text/plain")
 
         if reactivating:
             msg = _("Attempted to update Location field for an image "
@@ -817,14 +892,19 @@ class Controller(controller.BaseController):
         orig_keys = set(orig_image_meta['properties'])
         new_keys = set(image_meta['properties'])
         self._enforce_update_protected_props(
-                orig_keys.intersection(new_keys), image_meta,
-                orig_image_meta, req)
+            orig_keys.intersection(new_keys), image_meta,
+            orig_image_meta, req)
         self._enforce_create_protected_props(
-                new_keys.difference(orig_keys), req)
+            new_keys.difference(orig_keys), req)
         if purge_props:
             self._enforce_delete_protected_props(
-                    orig_keys.difference(new_keys), image_meta,
-                    orig_image_meta, req)
+                orig_keys.difference(new_keys), image_meta,
+                orig_image_meta, req)
+
+        self._enforce_image_property_quota(image_meta,
+                                           orig_image_meta=orig_image_meta,
+                                           purge_props=purge_props,
+                                           req=req)
 
         try:
             if location:
@@ -841,26 +921,31 @@ class Controller(controller.BaseController):
                                                  image_data)
 
         except exception.Invalid as e:
-            msg = (_("Failed to update image metadata. Got error: %(e)s")
-                   % locals())
+            msg = (_("Failed to update image metadata. Got error: %(e)s") %
+                   {'e': e})
             LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
         except exception.NotFound as e:
-            msg = (_("Failed to find image to update: %(e)s") % locals())
+            msg = _("Failed to find image to update: %(e)s") % {'e': e}
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Forbidden as e:
-            msg = (_("Forbidden to update image: %(e)s") % locals())
+            msg = _("Forbidden to update image: %(e)s") % {'e': e}
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPForbidden(explanation=msg,
                                 request=req,
                                 content_type="text/plain")
+        except (exception.Conflict, exception.Duplicate) as e:
+            LOG.info(unicode(e))
+            raise HTTPConflict(body='Image operation conflicts',
+                               request=req,
+                               content_type='text/plain')
         else:
             self.notifier.info('image.update', redact_loc(image_meta))
 
@@ -911,29 +996,37 @@ class Controller(controller.BaseController):
         else:
             status = 'deleted'
 
+        ori_status = image['status']
+
         try:
-            # Delete the image from the registry first, since we rely on it
+            # Update the image from the registry first, since we rely on it
             # for authorization checks.
             # See https://bugs.launchpad.net/glance/+bug/1065187
-            registry.update_image_metadata(req.context, id, {'status': status})
-            registry.delete_image_metadata(req.context, id)
+            image = registry.update_image_metadata(req.context, id,
+                                                   {'status': status})
 
-            # The image's location field may be None in the case
-            # of a saving or queued image, therefore don't ask a backend
-            # to delete the image if the backend doesn't yet store it.
-            # See https://bugs.launchpad.net/glance/+bug/747799
-            if image['location']:
-                upload_utils.initiate_deletion(req, image['location'], id,
-                                               CONF.delayed_delete)
+            try:
+                # The image's location field may be None in the case
+                # of a saving or queued image, therefore don't ask a backend
+                # to delete the image if the backend doesn't yet store it.
+                # See https://bugs.launchpad.net/glance/+bug/747799
+                if image['location']:
+                    upload_utils.initiate_deletion(req, image['location'], id,
+                                                   CONF.delayed_delete)
+            except Exception as e:
+                registry.update_image_metadata(req.context, id,
+                                               {'status': ori_status})
+                raise e
+            registry.delete_image_metadata(req.context, id)
         except exception.NotFound as e:
-            msg = (_("Failed to find image to delete: %(e)s") % locals())
+            msg = _("Failed to find image to delete: %(e)s") % {'e': e}
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
         except exception.Forbidden as e:
-            msg = (_("Forbidden to delete image: %(e)s") % locals())
+            msg = _("Forbidden to delete image: %(e)s") % {'e': e}
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPForbidden(explanation=msg,
@@ -970,11 +1063,10 @@ class ImageDeserializer(wsgi.JSONRequestDeserializer):
         result = {}
         try:
             result['image_meta'] = utils.get_image_meta_from_headers(request)
-        except exception.Invalid:
-            image_size_str = request.headers['x-image-meta-size']
-            msg = _("Incoming image size of %s was not convertible to "
-                    "an integer.") % image_size_str
-            raise HTTPBadRequest(explanation=msg, request=request)
+        except exception.InvalidParameterValue as e:
+            msg = unicode(e)
+            LOG.warn(msg, exc_info=True)
+            raise HTTPBadRequest(explanation=e.msg, request=request)
 
         image_meta = result['image_meta']
         image_meta = validate_image_meta(request, image_meta)
@@ -1060,7 +1152,7 @@ class ImageSerializer(wsgi.JSONResponseSerializer):
         # image_meta['size'] should be an int, but could possibly be a str
         expected_size = int(image_meta['size'])
         response.app_iter = common.size_checked_iter(
-                response, image_meta, expected_size, image_iter, self.notifier)
+            response, image_meta, expected_size, image_iter, self.notifier)
         # Using app_iter blanks content-length, so we set it here...
         response.headers['Content-Length'] = str(image_meta['size'])
         response.headers['Content-Type'] = 'application/octet-stream'
