@@ -12,54 +12,51 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-import glance_store as store_api
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_utils import encodeutils
-from oslo_utils import excutils
+
+from oslo.config import cfg
 import webob.exc
 
 from glance.common import exception
-from glance.common import store_utils
 from glance.common import utils
 import glance.db
-from glance.i18n import _, _LE, _LI
+from glance.openstack.common import excutils
+import glance.openstack.common.log as logging
 import glance.registry.client.v1.api as registry
+import glance.store
 
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
-def initiate_deletion(req, location_data, id):
+def initiate_deletion(req, location, id, delayed_delete=False):
     """
-    Deletes image data from the location of backend store.
+    Deletes image data from the backend store.
 
     :param req: The WSGI/Webob Request object
-    :param location_data: Location to the image data in a data store
-    :param id: Opaque image identifier
+    :param location: URL to the image data in a data store
+    :param image_id: Opaque image identifier
+    :param delayed_delete: whether data deletion will be delayed
     """
-    store_utils.delete_image_location_from_backend(req.context,
-                                                   id, location_data)
+    if delayed_delete:
+        glance.store.schedule_delayed_delete_from_backend(req.context,
+                                                          location, id)
+    else:
+        glance.store.safe_delete_from_backend(req.context, location, id)
 
 
-def _kill(req, image_id, from_state):
+def _kill(req, image_id):
     """
     Marks the image status to `killed`.
 
     :param req: The WSGI/Webob Request object
     :param image_id: Opaque image identifier
-    :param from_state: Permitted current status for transition to 'killed'
     """
-    # TODO(dosaboy): http://docs.openstack.org/developer/glance/statuses.html
-    # needs updating to reflect the fact that queued->killed and saving->killed
-    # are both allowed.
     registry.update_image_metadata(req.context, image_id,
-                                   {'status': 'killed'},
-                                   from_state=from_state)
+                                   {'status': 'killed'})
 
 
-def safe_kill(req, image_id, from_state):
+def safe_kill(req, image_id):
     """
     Mark image killed without raising exceptions if it fails.
 
@@ -68,12 +65,11 @@ def safe_kill(req, image_id, from_state):
 
     :param req: The WSGI/Webob Request object
     :param image_id: Opaque image identifier
-    :param from_state: Permitted current status for transition to 'killed'
     """
     try:
-        _kill(req, image_id, from_state)
+        _kill(req, image_id)
     except Exception:
-        LOG.exception(_LE("Unable to kill image %(id)s: ") % {'id': image_id})
+        LOG.exception(_("Unable to kill image %(id)s: ") % {'id': image_id})
 
 
 def upload_data_to_store(req, image_meta, image_data, store, notifier):
@@ -84,34 +80,23 @@ def upload_data_to_store(req, image_meta, image_data, store, notifier):
     """
     image_id = image_meta['id']
 
-    db_api = glance.db.get_api(v1_mode=True)
+    db_api = glance.db.get_api()
     image_size = image_meta.get('size')
 
     try:
-        # By default image_data will be passed as CooperativeReader object.
-        # But if 'user_storage_quota' is enabled and 'remaining' is not None
-        # then it will be passed as object of LimitingReader to
-        # 'store_add_to_backend' method.
-        image_data = utils.CooperativeReader(image_data)
-
         remaining = glance.api.common.check_quota(
             req.context, image_size, db_api, image_id=image_id)
         if remaining is not None:
             image_data = utils.LimitingReader(image_data, remaining)
 
-        (uri,
+        (location,
          size,
          checksum,
-         location_metadata) = store_api.store_add_to_backend(
+         locations_metadata) = glance.store.store_add_to_backend(
              image_meta['id'],
-             image_data,
+             utils.CooperativeReader(image_data),
              image_meta['size'],
-             store,
-             context=req.context)
-
-        location_data = {'url': uri,
-                         'metadata': location_metadata,
-                         'status': 'active'}
+             store)
 
         try:
             # recheck the quota in case there were simultaneous uploads that
@@ -119,11 +104,10 @@ def upload_data_to_store(req, image_meta, image_data, store, notifier):
             glance.api.common.check_quota(
                 req.context, size, db_api, image_id=image_id)
         except exception.StorageQuotaFull:
-            with excutils.save_and_reraise_exception():
-                LOG.info(_LI('Cleaning up %s after exceeding '
-                             'the quota'), image_id)
-                store_utils.safe_delete_from_backend(
-                    req.context, image_meta['id'], location_data)
+            LOG.info(_('Cleaning up %s after exceeding the quota') % image_id)
+            glance.store.safe_delete_from_backend(
+                location, req.context, image_meta['id'])
+            raise
 
         def _kill_mismatched(image_meta, attr, actual):
             supplied = image_meta.get(attr)
@@ -135,8 +119,8 @@ def upload_data_to_store(req, image_meta, image_data, store, notifier):
                                                    'supplied': supplied,
                                                    'actual': actual})
                 LOG.error(msg)
-                safe_kill(req, image_id, 'saving')
-                initiate_deletion(req, location_data, image_id)
+                safe_kill(req, image_id)
+                initiate_deletion(req, location, image_id, CONF.delayed_delete)
                 raise webob.exc.HTTPBadRequest(explanation=msg,
                                                content_type="text/plain",
                                                request=req)
@@ -148,62 +132,37 @@ def upload_data_to_store(req, image_meta, image_data, store, notifier):
 
         # Update the database with the checksum returned
         # from the backend store
-        LOG.debug("Updating image %(image_id)s data. "
+        LOG.debug(_("Updating image %(image_id)s data. "
                   "Checksum set to %(checksum)s, size set "
-                  "to %(size)d", {'image_id': image_id,
-                                  'checksum': checksum,
-                                  'size': size})
+                  "to %(size)d"), {'image_id': image_id,
+                                   'checksum': checksum,
+                                   'size': size})
         update_data = {'checksum': checksum,
                        'size': size}
         try:
-            try:
-                state = 'saving'
-                image_meta = registry.update_image_metadata(req.context,
-                                                            image_id,
-                                                            update_data,
-                                                            from_state=state)
-            except exception.Duplicate:
-                image = registry.get_image_metadata(req.context, image_id)
-                if image['status'] == 'deleted':
-                    raise exception.ImageNotFound()
-                else:
-                    raise
-        except exception.NotAuthenticated as e:
-            # Delete image data due to possible token expiration.
-            LOG.debug("Authentication error - the token may have "
-                      "expired during file upload. Deleting image data for "
-                      " %s " % image_id)
-            initiate_deletion(req, location_data, image_id)
-            raise webob.exc.HTTPUnauthorized(explanation=e.msg, request=req)
-        except exception.ImageNotFound:
-            msg = _("Image %s could not be found after upload. The image may"
-                    " have been deleted during the upload.") % image_id
+            image_meta = registry.update_image_metadata(req.context,
+                                                        image_id,
+                                                        update_data)
+
+        except exception.NotFound as e:
+            msg = _("Image %s could not be found after upload. The image may "
+                    "have been deleted during the upload.") % image_id
             LOG.info(msg)
 
             # NOTE(jculp): we need to clean up the datastore if an image
             # resource is deleted while the image data is being uploaded
             #
-            # We get "location_data" from above call to store.add(), any
+            # We get "location" from above call to store.add(), any
             # exceptions that occur there handle this same issue internally,
             # Since this is store-agnostic, should apply to all stores.
-            initiate_deletion(req, location_data, image_id)
+            initiate_deletion(req, location, image_id, CONF.delayed_delete)
             raise webob.exc.HTTPPreconditionFailed(explanation=msg,
                                                    request=req,
                                                    content_type='text/plain')
 
-    except store_api.StoreAddDisabled:
-        msg = _("Error in store configuration. Adding images to store "
-                "is disabled.")
-        LOG.exception(msg)
-        safe_kill(req, image_id, 'saving')
-        notifier.error('image.upload', msg)
-        raise webob.exc.HTTPGone(explanation=msg, request=req,
-                                 content_type='text/plain')
-
-    except (store_api.Duplicate, exception.Duplicate) as e:
-        msg = (_("Attempt to upload duplicate image: %s") %
-               encodeutils.exception_to_unicode(e))
-        LOG.warn(msg)
+    except exception.Duplicate as e:
+        msg = _("Attempt to upload duplicate image: %s") % e
+        LOG.debug(msg)
         # NOTE(dosaboy): do not delete the image since it is likely that this
         # conflict is a result of another concurrent upload that will be
         # successful.
@@ -213,30 +172,27 @@ def upload_data_to_store(req, image_meta, image_data, store, notifier):
                                      content_type="text/plain")
 
     except exception.Forbidden as e:
-        msg = (_("Forbidden upload attempt: %s") %
-               encodeutils.exception_to_unicode(e))
-        LOG.warn(msg)
-        safe_kill(req, image_id, 'saving')
+        msg = _("Forbidden upload attempt: %s") % e
+        LOG.debug(msg)
+        safe_kill(req, image_id)
         notifier.error('image.upload', msg)
         raise webob.exc.HTTPForbidden(explanation=msg,
                                       request=req,
                                       content_type="text/plain")
 
-    except store_api.StorageFull as e:
-        msg = (_("Image storage media is full: %s") %
-               encodeutils.exception_to_unicode(e))
+    except exception.StorageFull as e:
+        msg = _("Image storage media is full: %s") % e
         LOG.error(msg)
-        safe_kill(req, image_id, 'saving')
+        safe_kill(req, image_id)
         notifier.error('image.upload', msg)
         raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                   request=req,
                                                   content_type='text/plain')
 
-    except store_api.StorageWriteDenied as e:
-        msg = (_("Insufficient permissions on image storage media: %s") %
-               encodeutils.exception_to_unicode(e))
+    except exception.StorageWriteDenied as e:
+        msg = _("Insufficient permissions on image storage media: %s") % e
         LOG.error(msg)
-        safe_kill(req, image_id, 'saving')
+        safe_kill(req, image_id)
         notifier.error('image.upload', msg)
         raise webob.exc.HTTPServiceUnavailable(explanation=msg,
                                                request=req,
@@ -245,38 +201,38 @@ def upload_data_to_store(req, image_meta, image_data, store, notifier):
     except exception.ImageSizeLimitExceeded as e:
         msg = (_("Denying attempt to upload image larger than %d bytes.")
                % CONF.image_size_cap)
-        LOG.warn(msg)
-        safe_kill(req, image_id, 'saving')
+        LOG.info(msg)
+        safe_kill(req, image_id)
         notifier.error('image.upload', msg)
         raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                   request=req,
                                                   content_type='text/plain')
 
     except exception.StorageQuotaFull as e:
-        msg = (_("Denying attempt to upload image because it exceeds the "
-                 "quota: %s") % encodeutils.exception_to_unicode(e))
-        LOG.warn(msg)
-        safe_kill(req, image_id, 'saving')
+        msg = (_("Denying attempt to upload image because it exceeds the ."
+                 "quota: %s") % e)
+        LOG.info(msg)
+        safe_kill(req, image_id)
         notifier.error('image.upload', msg)
         raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                   request=req,
                                                   content_type='text/plain')
 
     except webob.exc.HTTPError:
-        # NOTE(bcwaldon): Ideally, we would just call 'raise' here,
+        #NOTE(bcwaldon): Ideally, we would just call 'raise' here,
         # but something in the above function calls is affecting the
         # exception context and we must explicitly re-raise the
         # caught exception.
-        msg = _LE("Received HTTP error while uploading image %s") % image_id
+        msg = _("Received HTTP error while uploading image %s") % image_id
         notifier.error('image.upload', msg)
         with excutils.save_and_reraise_exception():
             LOG.exception(msg)
-            safe_kill(req, image_id, 'saving')
+            safe_kill(req, image_id)
 
     except (ValueError, IOError) as e:
         msg = _("Client disconnected before sending all data to backend")
-        LOG.warn(msg)
-        safe_kill(req, image_id, 'saving')
+        LOG.debug(msg)
+        safe_kill(req, image_id)
         raise webob.exc.HTTPBadRequest(explanation=msg,
                                        content_type="text/plain",
                                        request=req)
@@ -284,10 +240,10 @@ def upload_data_to_store(req, image_meta, image_data, store, notifier):
     except Exception as e:
         msg = _("Failed to upload image %s") % image_id
         LOG.exception(msg)
-        safe_kill(req, image_id, 'saving')
+        safe_kill(req, image_id)
         notifier.error('image.upload', msg)
         raise webob.exc.HTTPInternalServerError(explanation=msg,
                                                 request=req,
                                                 content_type='text/plain')
 
-    return image_meta, location_data
+    return image_meta, location, locations_metadata
