@@ -15,42 +15,29 @@
 # under the License.
 
 import copy
-import http.client as http
-import urllib.parse as urlparse
-
-import debtcollector
-import glance_store
-from oslo_config import cfg
-from oslo_log import log as logging
-import oslo_serialization.jsonutils as json
-from oslo_utils import uuidutils
 import webob.exc
 
-from glance.api import common
+from oslo.config import cfg
+import six.moves.urllib.parse as urlparse
+
 from glance.api import policy
-from glance.api.v2 import policy as api_policy
 from glance.common import exception
-from glance.common import timeutils
+from glance.common import utils
 from glance.common import wsgi
 import glance.db
 import glance.gateway
-from glance.i18n import _, _LW
 import glance.notifier
+import glance.openstack.common.jsonutils as json
+import glance.openstack.common.log as logging
+from glance.openstack.common import timeutils
 import glance.schema
+import glance.store
 
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
 CONF.import_opt('task_time_to_live', 'glance.common.config', group='task')
 
-_DEPRECATION_MESSAGE = ("The task API is being deprecated and "
-                        "it will be superseded by the new image import "
-                        "API. Please refer to this link for more "
-                        "information about the aforementioned process: "
-                        "https://specs.openstack.org/openstack/glance-specs/"
-                        "specs/mitaka/approved/image-import/"
-                        "image-import-refactor.html")
-
 
 class TasksController(object):
     """Manages operations on tasks."""
@@ -60,42 +47,32 @@ class TasksController(object):
         self.db_api = db_api or glance.db.get_api()
         self.policy = policy_enforcer or policy.Enforcer()
         self.notifier = notifier or glance.notifier.Notifier()
-        self.store_api = store_api or glance_store
+        self.store_api = store_api or glance.store
         self.gateway = glance.gateway.Gateway(self.db_api, self.store_api,
                                               self.notifier, self.policy)
 
-    @debtcollector.removals.remove(message=_DEPRECATION_MESSAGE)
     def create(self, req, task):
-        # NOTE(rosmaita): access to this call is enforced in the deserializer
-
-        ctxt = req.context
-        task_factory = self.gateway.get_task_factory(ctxt)
-        executor_factory = self.gateway.get_task_executor_factory(ctxt)
-        task_repo = self.gateway.get_task_repo(ctxt)
+        task_factory = self.gateway.get_task_factory(req.context)
+        task_repo = self.gateway.get_task_repo(req.context)
+        live_time = CONF.task.task_time_to_live
         try:
-            new_task = task_factory.new_task(
-                task_type=task['type'],
-                owner=ctxt.owner,
-                task_input=task['input'],
-                image_id=task['input'].get('image_id'),
-                user_id=ctxt.user_id,
-                request_id=ctxt.request_id)
-            task_repo.add(new_task)
-            task_executor = executor_factory.new_task_executor(ctxt)
-            pool = common.get_thread_pool("tasks_pool")
-            pool.spawn(new_task.run, task_executor)
+            new_task = task_factory.new_task(task_type=task['type'],
+                                             owner=req.context.owner,
+                                             task_time_to_live=live_time)
+            new_task_details = task_factory.new_task_details(new_task.task_id,
+                                                             task['input'])
+            task_repo.add(new_task, new_task_details)
         except exception.Forbidden as e:
-            msg = (_LW("Forbidden to create task. Reason: %(reason)s")
-                   % {'reason': e})
-            LOG.warning(msg)
+            msg = (_("Forbidden to create task. Reason: %(reason)s")
+                   % {'reason': unicode(e)})
+            LOG.info(msg)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
-        return new_task
 
-    @debtcollector.removals.remove(message=_DEPRECATION_MESSAGE)
+        result = {'task': new_task, 'task_details': new_task_details}
+        return result
+
     def index(self, req, marker=None, limit=None, sort_key='created_at',
               sort_dir='desc', filters=None):
-        # NOTE(rosmaita): access to this call is enforced in the deserializer
-
         result = {}
         if filters is None:
             filters = {}
@@ -105,46 +82,43 @@ class TasksController(object):
             limit = CONF.limit_param_default
         limit = min(CONF.api_limit_max, limit)
 
-        task_repo = self.gateway.get_task_stub_repo(req.context)
+        task_repo = self.gateway.get_task_repo(req.context)
         try:
-            tasks = task_repo.list(marker, limit, sort_key,
-                                   sort_dir, filters)
+            tasks = task_repo.list_tasks(marker,
+                                         limit,
+                                         sort_key,
+                                         sort_dir,
+                                         filters)
             if len(tasks) != 0 and len(tasks) == limit:
                 result['next_marker'] = tasks[-1].task_id
         except (exception.NotFound, exception.InvalidSortKey,
                 exception.InvalidFilterRangeValue) as e:
-            LOG.warning(str(e))
+            LOG.info(unicode(e))
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except exception.Forbidden as e:
-            LOG.warning(str(e))
+            LOG.info(unicode(e))
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         result['tasks'] = tasks
         return result
 
-    @debtcollector.removals.remove(message=_DEPRECATION_MESSAGE)
     def get(self, req, task_id):
-        _enforce_access_policy(self.policy, req)
         try:
             task_repo = self.gateway.get_task_repo(req.context)
-            task = task_repo.get(task_id)
+            task, task_details = task_repo.get_task_and_details(task_id)
         except exception.NotFound as e:
-            msg = (_LW("Failed to find task %(task_id)s. Reason: %(reason)s")
-                   % {'task_id': task_id,
-                      'reason': e})
-            LOG.warning(msg)
+            msg = (_("Failed to find task %(task_id)s. Reason: %(reason)s") %
+                   {'task_id': task_id, 'reason': unicode(e)})
+            LOG.info(msg)
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Forbidden as e:
-            msg = (_LW("Forbidden to get task %(task_id)s. Reason:"
-                       " %(reason)s")
-                   % {'task_id': task_id,
-                      'reason': e})
-            LOG.warning(msg)
+            msg = (_("Forbidden to get task %(task_id)s. Reason: %(reason)s") %
+                   {'task_id': task_id, 'reason': unicode(e)})
+            LOG.info(msg)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
-        return task
+        result = {'task': task, 'task_details': task_details}
+        return result
 
-    @debtcollector.removals.remove(message=_DEPRECATION_MESSAGE)
     def delete(self, req, task_id):
-        _enforce_access_policy(self.policy, req)
         msg = (_("This operation is currently not permitted on Glance Tasks. "
                  "They are auto deleted after reaching the time based on "
                  "their expires_at property."))
@@ -159,7 +133,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
     def _get_request_body(self, request):
         output = super(RequestDeserializer, self).default(request)
         if 'body' not in output:
-            msg = _('Body expected in request.')
+            msg = _('Body expected in request')
             raise webob.exc.HTTPBadRequest(explanation=msg)
         return output['body']
 
@@ -186,7 +160,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         return filters
 
     def _validate_marker(self, marker):
-        if marker and not uuidutils.is_uuid_like(marker):
+        if marker and not utils.is_uuid_like(marker):
             msg = _('Invalid marker format')
             raise webob.exc.HTTPBadRequest(explanation=msg)
         return marker
@@ -209,16 +183,13 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         for param in self._required_properties:
             if param not in body:
                 msg = _("Task '%s' is required") % param
-                raise webob.exc.HTTPBadRequest(explanation=msg)
+                raise webob.exc.HTTPBadRequest(explanation=unicode(msg))
 
-    def __init__(self, schema=None, policy_engine=None):
+    def __init__(self, schema=None):
         super(RequestDeserializer, self).__init__()
         self.schema = schema or get_task_schema()
-        # want to enforce the access policy as early as possible
-        self.policy_engine = policy_engine or policy.Enforcer()
 
     def create(self, request):
-        _enforce_access_policy(self.policy_engine, request)
         body = self._get_request_body(request)
         self._validate_create_body(body)
         try:
@@ -235,7 +206,6 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         return dict(task=task)
 
     def index(self, request):
-        _enforce_access_policy(self.policy_engine, request)
         params = request.params.copy()
         limit = params.pop('limit', None)
         marker = params.pop('marker', None)
@@ -258,65 +228,51 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
     def __init__(self, task_schema=None, partial_task_schema=None):
         super(ResponseSerializer, self).__init__()
         self.task_schema = task_schema or get_task_schema()
-        self.partial_task_schema = (partial_task_schema
-                                    or _get_partial_task_schema())
+        self.partial_task_schema = partial_task_schema \
+            or _get_partial_task_schema()
 
     def _inject_location_header(self, response, task):
         location = self._get_task_location(task)
-        response.headers['Location'] = location
+        response.headers['Location'] = location.encode('utf-8')
 
     def _get_task_location(self, task):
         return '/v2/tasks/%s' % task.task_id
 
-    def _format_task(self, schema, task):
-        task_view = {
-            'id': task.task_id,
-            'input': task.task_input,
-            'type': task.type,
-            'status': task.status,
-            'owner': task.owner,
-            'message': task.message,
-            'result': task.result,
-            'created_at': timeutils.isotime(task.created_at),
-            'updated_at': timeutils.isotime(task.updated_at),
-            'self': self._get_task_location(task),
-            'schema': '/v2/schemas/task'
-        }
-        if task.image_id:
-            task_view['image_id'] = task.image_id
-        if task.request_id:
-            task_view['request_id'] = task.request_id
-        if task.user_id:
-            task_view['user_id'] = task.user_id
+    def _format_task(self, schema, task, task_details=None):
+        task_view = {}
+        task_attributes = ['type', 'status', 'owner']
+        task_details_attributes = ['input', 'result', 'message']
+        for key in task_attributes:
+            task_view[key] = getattr(task, key)
+        if task_details:
+            for key in task_details_attributes:
+                task_view[key] = getattr(task_details, key)
+        task_view['id'] = task.task_id
         if task.expires_at:
             task_view['expires_at'] = timeutils.isotime(task.expires_at)
+        task_view['created_at'] = timeutils.isotime(task.created_at)
+        task_view['updated_at'] = timeutils.isotime(task.updated_at)
+        task_view['self'] = self._get_task_location(task)
+        task_view['schema'] = '/v2/schemas/task'
         task_view = schema.filter(task_view)  # domain
         return task_view
 
-    def _format_task_stub(self, schema, task):
-        task_view = {
-            'id': task.task_id,
-            'type': task.type,
-            'status': task.status,
-            'owner': task.owner,
-            'created_at': timeutils.isotime(task.created_at),
-            'updated_at': timeutils.isotime(task.updated_at),
-            'self': self._get_task_location(task),
-            'schema': '/v2/schemas/task'
-        }
-        if task.expires_at:
-            task_view['expires_at'] = timeutils.isotime(task.expires_at)
-        task_view = schema.filter(task_view)  # domain
-        return task_view
-
-    def create(self, response, task):
-        response.status_int = http.CREATED
+    def create(self, response, result):
+        response.status_int = 201
+        task = result['task']
+        task_details = result['task_details']
         self._inject_location_header(response, task)
-        self.get(response, task)
+        self._get(response, task, task_details)
 
-    def get(self, response, task):
-        task_view = self._format_task(self.task_schema, task)
-        response.unicode_body = json.dumps(task_view, ensure_ascii=False)
+    def get(self, response, result):
+        task = result['task']
+        task_details = result['task_details']
+        self._get(response, task, task_details)
+
+    def _get(self, response, task, task_details):
+        task_view = self._format_task(self.task_schema, task, task_details)
+        body = json.dumps(task_view, ensure_ascii=False)
+        response.unicode_body = unicode(body)
         response.content_type = 'application/json'
 
     def index(self, response, result):
@@ -324,7 +280,7 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
         params.pop('marker', None)
         query = urlparse.urlencode(params)
         body = {
-            'tasks': [self._format_task_stub(self.partial_task_schema, task)
+            'tasks': [self._format_task(self.partial_task_schema, task)
                       for task in result['tasks']],
             'first': '/v2/tasks',
             'schema': '/v2/schemas/tasks',
@@ -335,7 +291,7 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             params['marker'] = result['next_marker']
             next_query = urlparse.urlencode(params)
             body['next'] = '/v2/tasks?%s' % next_query
-        response.unicode_body = json.dumps(body, ensure_ascii=False)
+        response.unicode_body = unicode(json.dumps(body, ensure_ascii=False))
         response.content_type = 'application/json'
 
 
@@ -350,8 +306,6 @@ _TASK_SCHEMA = {
         "description": _("The type of task represented by this content"),
         "enum": [
             "import",
-            "api_image_import",
-            "location_import",
         ],
         "type": "string"
     },
@@ -367,11 +321,11 @@ _TASK_SCHEMA = {
     },
     "input": {
         "description": _("The parameters required by task, JSON blob"),
-        "type": ["null", "object"],
+        "type": "object"
     },
     "result": {
         "description": _("The result of current task, JSON blob"),
-        "type": ["null", "object"],
+        "type": "object",
     },
     "owner": {
         "description": _("An identifier for the owner of this task"),
@@ -382,22 +336,10 @@ _TASK_SCHEMA = {
                          " when appropriate (usually on failure)"),
         "type": "string",
     },
-    "image_id": {
-        "description": _("Image associated with the task"),
-        "type": "string",
-    },
-    "request_id": {
-        "description": _("Human-readable informative request-id"),
-        "type": "string",
-    },
-    "user_id": {
-        "description": _("User associated with the task"),
-        "type": "string",
-    },
     "expires_at": {
         "description": _("Datetime when this resource would be"
                          " subject to removal"),
-        "type": ["null", "string"]
+        "type": "string"
     },
     "created_at": {
         "description": _("Datetime when this resource was created"),
@@ -407,23 +349,11 @@ _TASK_SCHEMA = {
         "description": _("Datetime when this resource was updated"),
         "type": "string"
     },
-    'self': {
-        'readOnly': True,
-        'type': 'string'
-    },
-    'schema': {
-        'readOnly': True,
-        'type': 'string'
-    }
+    'self': {'type': 'string'},
+    'schema': {'type': 'string'}
 }
 
 
-def _enforce_access_policy(policy_engine, request):
-    api_policy.TasksAPIPolicy(
-        request.context,
-        enforcer=policy_engine).tasks_api_access()
-
-
 def get_task_schema():
     properties = copy.deepcopy(_TASK_SCHEMA)
     schema = glance.schema.Schema('task', properties)
