@@ -1,7 +1,6 @@
 #!/usr/bin/env python
 
 # Copyright 2012 Michael Still and Canonical Inc
-# Copyright 2014 SoftLayer Technologies, Inc.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,73 +15,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import http.client as http
+from __future__ import print_function
+
+import httplib
+import logging
+import logging.config
+import logging.handlers
+import optparse
 import os
+import re
 import sys
-import urllib.parse as urlparse
+import uuid
 
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_serialization import jsonutils
-from oslo_utils import encodeutils
-from oslo_utils import uuidutils
-from webob import exc
+import six.moves.urllib.parse as urlparse
 
-from glance.common import config
-from glance.common import exception
-from glance.common import utils
-from glance.i18n import _, _LE, _LI, _LW
-
-LOG = logging.getLogger(__name__)
-
-
-# NOTE: positional arguments <args> will be parsed before <command> until
-# this bug is corrected https://bugs.launchpad.net/oslo.config/+bug/1392428
-cli_opts = [
-    cfg.IntOpt('chunksize',
-               short='c',
-               default=65536,
-               help="Amount of data to transfer per HTTP write."),
-    cfg.StrOpt('dontreplicate',
-               short='D',
-               default=('created_at date deleted_at location updated_at'),
-               help="List of fields to not replicate."),
-    cfg.BoolOpt('metaonly',
-                short='m',
-                default=False,
-                help="Only replicate metadata, not images."),
-    cfg.StrOpt('token',
-               short='t',
-               default='',
-               help=("Pass in your authentication token if you have "
-                     "one. If you use this option the same token is "
-                     "used for both the source and the target.")),
-    cfg.StrOpt('command',
-               positional=True,
-               required=False,
-               help="Command to be given to replicator"),
-    cfg.MultiStrOpt('args',
-                    positional=True,
-                    required=False,
-                    help="Arguments for the command"),
-]
-
-CONF = cfg.CONF
-CONF.register_cli_opts(cli_opts)
-
-CONF.register_opt(
-    cfg.StrOpt('sourcetoken',
-               default='',
-               help=("Pass in your authentication token if you have "
-                     "one. This is the token used for the source.")))
-CONF.register_opt(
-    cfg.StrOpt('targettoken',
-               default='',
-               help=("Pass in your authentication token if you have "
-                     "one. This is the token used for the target.")))
-
-logging.register_options(CONF)
-CONF.set_default(name='use_stderr', default=True)
+from glance.openstack.common import jsonutils
 
 # If ../glance/__init__.py exists, add ../ to Python search path, so that
 # it will override what happens to be installed in /usr/(local/)lib/python...
@@ -97,7 +44,7 @@ COMMANDS = """Commands:
 
     help <command>  Output help for one of the commands below
 
-    compare         What is missing from the target glance?
+    compare         What is missing from the slave glance?
     dump            Dump the contents of a glance instance to local disk.
     livecopy        Load the contents of one glance instance into another.
     load            Load the contents of a local directory into glance.
@@ -106,19 +53,36 @@ COMMANDS = """Commands:
 
 
 IMAGE_ALREADY_PRESENT_MESSAGE = _('The image %s is already present on '
-                                  'the target, but our check for it did '
+                                  'the slave, but our check for it did '
                                   'not find it. This indicates that we '
                                   'do not have permissions to see all '
-                                  'the images on the target server.')
+                                  'the images on the slave server.')
+
+SERVER_PORT_REGEX = '\w+:\w+'
+
+
+class AuthenticationException(Exception):
+    pass
+
+
+class ImageAlreadyPresentException(Exception):
+    pass
+
+
+class ServerErrorException(Exception):
+    pass
+
+
+class UploadException(Exception):
+    pass
 
 
 class ImageService(object):
     def __init__(self, conn, auth_token):
         """Initialize the ImageService.
 
-        :param conn: a http.client.HTTPConnection to the glance server
-        :param auth_token: authentication token to pass in the x-auth-token
-            header
+        conn: a httplib.HTTPConnection to the glance server
+        auth_token: authentication token to pass in the x-auth-token header
         """
         self.auth_token = auth_token
         self.conn = conn
@@ -133,52 +97,41 @@ class ImageService(object):
         body: body to send with the request
         ignore_result_body: the body of the result will be ignored
 
-        :returns: A http.client response object
+        Returns: a httplib response object
         """
         if self.auth_token:
             headers.setdefault('x-auth-token', self.auth_token)
 
-        LOG.debug('Request: %(method)s http://%(server)s:%(port)s'
-                  '%(url)s with headers %(headers)s',
-                  {'method': method,
-                   'server': self.conn.host,
-                   'port': self.conn.port,
-                   'url': url,
-                   'headers': repr(headers)})
+        logging.debug(_('Request: %(method)s http://%(server)s:%(port)s'
+                        '%(url)s with headers %(headers)s')
+                      % {'method': method,
+                         'server': self.conn.host,
+                         'port': self.conn.port,
+                         'url': url,
+                         'headers': repr(headers)})
         self.conn.request(method, url, body, headers)
 
         response = self.conn.getresponse()
         headers = self._header_list_to_dict(response.getheaders())
         code = response.status
-        code_description = http.responses[code]
-        LOG.debug('Response: %(code)s %(status)s %(headers)s',
-                  {'code': code,
-                   'status': code_description,
-                   'headers': repr(headers)})
+        code_description = httplib.responses[code]
+        logging.debug(_('Response: %(code)s %(status)s %(headers)s')
+                      % {'code': code,
+                         'status': code_description,
+                         'headers': repr(headers)})
 
-        if code == http.BAD_REQUEST:
-            raise exc.HTTPBadRequest(
-                explanation=response.read())
+        if code in [400, 500]:
+            raise ServerErrorException(response.read())
 
-        if code == http.INTERNAL_SERVER_ERROR:
-            raise exc.HTTPInternalServerError(
-                explanation=response.read())
+        if code in [401, 403]:
+            raise AuthenticationException(response.read())
 
-        if code == http.UNAUTHORIZED:
-            raise exc.HTTPUnauthorized(
-                explanation=response.read())
-
-        if code == http.FORBIDDEN:
-            raise exc.HTTPForbidden(
-                explanation=response.read())
-
-        if code == http.CONFLICT:
-            raise exc.HTTPConflict(
-                explanation=response.read())
+        if code == 409:
+            raise ImageAlreadyPresentException(response.read())
 
         if ignore_result_body:
             # NOTE: because we are pipelining requests through a single HTTP
-            # connection, http.client requires that we read the response body
+            # connection, httplib requires that we read the response body
             # before we can make another request. If the caller knows they
             # don't care about the body, they can ask us to do that for them.
             response.read()
@@ -200,7 +153,7 @@ class ImageService(object):
             response = self._http_request('GET', url, {}, '')
             result = jsonutils.loads(response.read())
 
-            if not result or 'images' not in result or not result['images']:
+            if not result or not 'images' in result or not result['images']:
                 return
             for image in result.get('images', []):
                 params['marker'] = image['id']
@@ -211,7 +164,7 @@ class ImageService(object):
 
         image_uuid: the id of an image
 
-        :returns: a http.client Response object where the body is the image.
+        Returns: a httplib Response object where the body is the image.
         """
         url = '/v1/images/%s' % image_uuid
         return self._http_request('GET', url, {}, '')
@@ -284,7 +237,7 @@ class ImageService(object):
         response = self._http_request('POST', url, headers, image_data)
         headers = self._header_list_to_dict(response.getheaders())
 
-        LOG.debug('Image post done')
+        logging.debug(_('Image post done'))
         body = response.read()
         return headers, body
 
@@ -303,7 +256,7 @@ class ImageService(object):
         response = self._http_request('PUT', url, headers, '')
         headers = self._header_list_to_dict(response.getheaders())
 
-        LOG.debug('Image post done')
+        logging.debug(_('Image post done'))
         body = response.read()
         return headers, body
 
@@ -316,14 +269,6 @@ def get_image_service():
     return ImageService
 
 
-def _human_readable_size(num, suffix='B'):
-    for unit in ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi']:
-        if abs(num) < 1024.0:
-            return "%3.1f %s%s" % (num, unit, suffix)
-        num /= 1024.0
-    return "%.1f %s%s" % (num, 'Yi', suffix)
-
-
 def replication_size(options, args):
     """%(prog)s size <server:port>
 
@@ -333,27 +278,30 @@ def replication_size(options, args):
     """
 
     # Make sure server info is provided
-    if args is None or len(args) < 1:
+    if len(args) < 1:
         raise TypeError(_("Too few arguments."))
 
-    server, port = utils.parse_valid_host_port(args.pop())
+    server_port = args.pop()
+
+    if not re.match(SERVER_PORT_REGEX, server_port):
+        raise ValueError(_("Bad format of the given arguments."))
+
+    server, port = server_port.split(':')
 
     total_size = 0
     count = 0
 
     imageservice = get_image_service()
-    client = imageservice(http.HTTPConnection(server, port),
-                          options.targettoken)
+    client = imageservice(httplib.HTTPConnection(server, port),
+                          options.slavetoken)
     for image in client.get_images():
-        LOG.debug('Considering image: %(image)s', {'image': image})
+        logging.debug(_('Considering image: %(image)s') % {'image': image})
         if image['status'] == 'active':
             total_size += int(image['size'])
             count += 1
 
-    print(_('Total size is %(size)d bytes (%(human_size)s) across '
-            '%(img_count)d images') %
+    print(_('Total size is %(size)d bytes across %(img_count)d images') %
           {'size': total_size,
-           'human_size': _human_readable_size(total_size),
            'img_count': count})
 
 
@@ -371,30 +319,25 @@ def replication_dump(options, args):
         raise TypeError(_("Too few arguments."))
 
     path = args.pop()
-    server, port = utils.parse_valid_host_port(args.pop())
+    server_port = args.pop()
+
+    if not re.match(SERVER_PORT_REGEX, server_port):
+        raise ValueError(_("Bad format of the given arguments."))
+
+    server, port = server_port.split(':')
 
     imageservice = get_image_service()
-    client = imageservice(http.HTTPConnection(server, port),
-                          options.sourcetoken)
+    client = imageservice(httplib.HTTPConnection(server, port),
+                          options.mastertoken)
     for image in client.get_images():
-        LOG.debug('Considering: %(image_id)s (%(image_name)s) '
-                  '(%(image_size)d bytes)',
-                  {'image_id': image['id'],
-                   'image_name': image.get('name', '--unnamed--'),
-                   'image_size': image['size']})
+        logging.info(_('Considering: %s') % image['id'])
 
         data_path = os.path.join(path, image['id'])
-        data_filename = data_path + '.img'
         if not os.path.exists(data_path):
-            LOG.info(_LI('Storing: %(image_id)s (%(image_name)s)'
-                         ' (%(image_size)d bytes) in %(data_filename)s'),
-                     {'image_id': image['id'],
-                      'image_name': image.get('name', '--unnamed--'),
-                      'image_size': image['size'],
-                      'data_filename': data_filename})
+            logging.info(_('... storing'))
 
             # Dump glance information
-            with open(data_path, 'w', encoding='utf-8') as f:
+            with open(data_path, 'w') as f:
                 f.write(jsonutils.dumps(image))
 
             if image['status'] == 'active' and not options.metaonly:
@@ -402,9 +345,9 @@ def replication_dump(options, args):
                 # is the same as that which we got from the detailed images
                 # request earlier, so we can ignore it here. Note that we also
                 # only dump active images.
-                LOG.debug('Image %s is active', image['id'])
+                logging.info(_('... image is active'))
                 image_response = client.get_image(image['id'])
-                with open(data_filename, 'wb') as f:
+                with open(data_path + '.img', 'wb') as f:
                     while True:
                         chunk = image_response.read(options.chunksize)
                         if not chunk:
@@ -420,25 +363,33 @@ def _dict_diff(a, b):
 
     Returns: True if the dictionaries are different
     """
-    # Only things the source has which the target lacks matter
+    # Only things the master has which the slave lacks matter
     if set(a.keys()) - set(b.keys()):
-        LOG.debug('metadata diff -- source has extra keys: %(keys)s',
-                  {'keys': ' '.join(set(a.keys()) - set(b.keys()))})
+        logging.debug(_('metadata diff -- master has extra keys: %(keys)s')
+                      % {'keys': ' '.join(set(a.keys()) - set(b.keys()))})
         return True
 
     for key in a:
         if str(a[key]) != str(b[key]):
-            LOG.debug('metadata diff -- value differs for key '
-                      '%(key)s: source "%(source_value)s" vs '
-                      'target "%(target_value)s"',
-                      {'key': key,
-                       'source_value': a[key],
-                       'target_value': b[key]})
+            logging.debug(_('metadata diff -- value differs for key '
+                          '%(key)s: master "%(master_value)s" vs '
+                          'slave "%(slave_value)s"') %
+                          {'key': key, 'master_value': a[key],
+                           'slave_value': b[key]})
             return True
 
     return False
 
 
+# This is lifted from openstack-common, but copied here to reduce dependancies
+def is_uuid_like(value):
+    try:
+        uuid.UUID(value)
+        return True
+    except Exception:
+        return False
+
+
 def replication_load(options, args):
     """%(prog)s load <server:port> <path>
 
@@ -453,18 +404,23 @@ def replication_load(options, args):
         raise TypeError(_("Too few arguments."))
 
     path = args.pop()
-    server, port = utils.parse_valid_host_port(args.pop())
+    server_port = args.pop()
+
+    if not re.match(SERVER_PORT_REGEX, server_port):
+        raise ValueError(_("Bad format of the given arguments."))
+
+    server, port = server_port.split(':')
 
     imageservice = get_image_service()
-    client = imageservice(http.HTTPConnection(server, port),
-                          options.targettoken)
+    client = imageservice(httplib.HTTPConnection(server, port),
+                          options.slavetoken)
 
     updated = []
 
     for ent in os.listdir(path):
-        if uuidutils.is_uuid_like(ent):
+        if is_uuid_like(ent):
             image_uuid = ent
-            LOG.info(_LI('Considering: %s'), image_uuid)
+            logging.info(_('Considering: %s') % image_uuid)
 
             meta_file_name = os.path.join(path, image_uuid)
             with open(meta_file_name) as meta_file:
@@ -473,32 +429,31 @@ def replication_load(options, args):
             # Remove keys which don't make sense for replication
             for key in options.dontreplicate.split(' '):
                 if key in meta:
-                    LOG.debug('Stripping %(header)s from saved '
-                              'metadata', {'header': key})
+                    logging.debug(_('Stripping %(header)s from saved '
+                                    'metadata'), {'header': key})
                     del meta[key]
 
             if _image_present(client, image_uuid):
                 # NOTE(mikal): Perhaps we just need to update the metadata?
                 # Note that we don't attempt to change an image file once it
                 # has been uploaded.
-                LOG.debug('Image %s already present', image_uuid)
+                logging.debug(_('Image %s already present'), image_uuid)
                 headers = client.get_image_meta(image_uuid)
                 for key in options.dontreplicate.split(' '):
                     if key in headers:
-                        LOG.debug('Stripping %(header)s from target '
-                                  'metadata', {'header': key})
+                        logging.debug(_('Stripping %(header)s from slave '
+                                        'metadata'), {'header': key})
                         del headers[key]
 
                 if _dict_diff(meta, headers):
-                    LOG.info(_LI('Image %s metadata has changed'), image_uuid)
+                    logging.info(_('... metadata has changed'))
                     headers, body = client.add_image_meta(meta)
                     _check_upload_response_headers(headers, body)
                     updated.append(meta['id'])
 
             else:
                 if not os.path.exists(os.path.join(path, image_uuid + '.img')):
-                    LOG.debug('%s dump is missing image data, skipping',
-                              image_uuid)
+                    logging.info(_('... dump is missing image data, skipping'))
                     continue
 
                 # Upload the image itself
@@ -507,11 +462,9 @@ def replication_load(options, args):
                         headers, body = client.add_image(meta, img_file)
                         _check_upload_response_headers(headers, body)
                         updated.append(meta['id'])
-                    except exc.HTTPConflict:
-                        # NOTE(tkajinam): noqa does not work with multi-line,
-                        # so split this interpolation to a separate line
-                        msg = _LE(IMAGE_ALREADY_PRESENT_MESSAGE) % image_uuid
-                        LOG.error(msg)
+                    except ImageAlreadyPresentException:
+                        logging.error(IMAGE_ALREADY_PRESENT_MESSAGE
+                                      % image_uuid)
 
     return updated
 
@@ -521,75 +474,74 @@ def replication_livecopy(options, args):
 
     Load the contents of one glance instance into another.
 
-    fromserver:port: the location of the source glance instance.
-    toserver:port:   the location of the target glance instance.
+    fromserver:port: the location of the master glance instance.
+    toserver:port:   the location of the slave glance instance.
     """
 
     # Make sure from-server and to-server are provided
     if len(args) < 2:
         raise TypeError(_("Too few arguments."))
 
+    slave_server_port = args.pop()
+    master_server_port = args.pop()
+
+    if not re.match(SERVER_PORT_REGEX, slave_server_port) or \
+            not re.match(SERVER_PORT_REGEX, master_server_port):
+        raise ValueError(_("Bad format of the given arguments."))
+
     imageservice = get_image_service()
 
-    target_server, target_port = utils.parse_valid_host_port(args.pop())
-    target_conn = http.HTTPConnection(target_server, target_port)
-    target_client = imageservice(target_conn, options.targettoken)
+    slave_server, slave_port = slave_server_port.split(':')
+    slave_conn = httplib.HTTPConnection(slave_server, slave_port)
+    slave_client = imageservice(slave_conn, options.slavetoken)
 
-    source_server, source_port = utils.parse_valid_host_port(args.pop())
-    source_conn = http.HTTPConnection(source_server, source_port)
-    source_client = imageservice(source_conn, options.sourcetoken)
+    master_server, master_port = master_server_port.split(':')
+    master_conn = httplib.HTTPConnection(master_server, master_port)
+    master_client = imageservice(master_conn, options.mastertoken)
 
     updated = []
 
-    for image in source_client.get_images():
-        LOG.debug('Considering %(id)s', {'id': image['id']})
+    for image in master_client.get_images():
+        logging.info(_('Considering %(id)s') % {'id': image['id']})
         for key in options.dontreplicate.split(' '):
             if key in image:
-                LOG.debug('Stripping %(header)s from source metadata',
-                          {'header': key})
+                logging.debug(_('Stripping %(header)s from master metadata'),
+                              {'header': key})
                 del image[key]
 
-        if _image_present(target_client, image['id']):
+        if _image_present(slave_client, image['id']):
             # NOTE(mikal): Perhaps we just need to update the metadata?
             # Note that we don't attempt to change an image file once it
             # has been uploaded.
-            headers = target_client.get_image_meta(image['id'])
+            headers = slave_client.get_image_meta(image['id'])
             if headers['status'] == 'active':
                 for key in options.dontreplicate.split(' '):
                     if key in image:
-                        LOG.debug('Stripping %(header)s from source '
-                                  'metadata', {'header': key})
+                        logging.debug(_('Stripping %(header)s from master '
+                                        'metadata'), {'header': key})
                         del image[key]
                     if key in headers:
-                        LOG.debug('Stripping %(header)s from target '
-                                  'metadata', {'header': key})
+                        logging.debug(_('Stripping %(header)s from slave '
+                                        'metadata'), {'header': key})
                         del headers[key]
 
                 if _dict_diff(image, headers):
-                    LOG.info(_LI('Image %(image_id)s (%(image_name)s) '
-                                 'metadata has changed'),
-                             {'image_id': image['id'],
-                              'image_name': image.get('name', '--unnamed--')})
-                    headers, body = target_client.add_image_meta(image)
+                    logging.info(_('... metadata has changed'))
+                    headers, body = slave_client.add_image_meta(image)
                     _check_upload_response_headers(headers, body)
                     updated.append(image['id'])
 
         elif image['status'] == 'active':
-            LOG.info(_LI('Image %(image_id)s (%(image_name)s) '
-                         '(%(image_size)d bytes) '
-                         'is being synced'),
-                     {'image_id': image['id'],
-                      'image_name': image.get('name', '--unnamed--'),
-                      'image_size': image['size']})
+            logging.info(_('%s is being synced') % image['id'])
             if not options.metaonly:
-                image_response = source_client.get_image(image['id'])
+                image_response = master_client.get_image(image['id'])
                 try:
-                    headers, body = target_client.add_image(image,
-                                                            image_response)
+                    headers, body = slave_client.add_image(image,
+                                                           image_response)
                     _check_upload_response_headers(headers, body)
                     updated.append(image['id'])
-                except exc.HTTPConflict:
-                    LOG.error(_LE(IMAGE_ALREADY_PRESENT_MESSAGE) % image['id'])  # noqa
+                except ImageAlreadyPresentException:
+                    logging.error(IMAGE_ALREADY_PRESENT_MESSAGE % image['id'])
 
     return updated
 
@@ -599,59 +551,64 @@ def replication_compare(options, args):
 
     Compare the contents of fromserver with those of toserver.
 
-    fromserver:port: the location of the source glance instance.
-    toserver:port:   the location of the target glance instance.
+    fromserver:port: the location of the master glance instance.
+    toserver:port:   the location of the slave glance instance.
     """
 
     # Make sure from-server and to-server are provided
     if len(args) < 2:
         raise TypeError(_("Too few arguments."))
 
+    slave_server_port = args.pop()
+    master_server_port = args.pop()
+
+    if not re.match(SERVER_PORT_REGEX, slave_server_port) or \
+            not re.match(SERVER_PORT_REGEX, master_server_port):
+        raise ValueError(_("Bad format of the given arguments."))
+
     imageservice = get_image_service()
 
-    target_server, target_port = utils.parse_valid_host_port(args.pop())
-    target_conn = http.HTTPConnection(target_server, target_port)
-    target_client = imageservice(target_conn, options.targettoken)
+    slave_server, slave_port = slave_server_port.split(':')
+    slave_conn = httplib.HTTPConnection(slave_server, slave_port)
+    slave_client = imageservice(slave_conn, options.slavetoken)
 
-    source_server, source_port = utils.parse_valid_host_port(args.pop())
-    source_conn = http.HTTPConnection(source_server, source_port)
-    source_client = imageservice(source_conn, options.sourcetoken)
+    master_server, master_port = master_server_port.split(':')
+    master_conn = httplib.HTTPConnection(master_server, master_port)
+    master_client = imageservice(master_conn, options.mastertoken)
 
     differences = {}
 
-    for image in source_client.get_images():
-        if _image_present(target_client, image['id']):
-            headers = target_client.get_image_meta(image['id'])
+    for image in master_client.get_images():
+        if _image_present(slave_client, image['id']):
+            headers = slave_client.get_image_meta(image['id'])
             for key in options.dontreplicate.split(' '):
                 if key in image:
-                    LOG.debug('Stripping %(header)s from source metadata',
-                              {'header': key})
+                    logging.debug(_('Stripping %(header)s from master '
+                                    'metadata'), {'header': key})
                     del image[key]
                 if key in headers:
-                    LOG.debug('Stripping %(header)s from target metadata',
-                              {'header': key})
+                    logging.debug(_('Stripping %(header)s from slave '
+                                    'metadata'), {'header': key})
                     del headers[key]
 
             for key in image:
-                if image[key] != headers.get(key):
-                    LOG.warning(_LW('%(image_id)s: field %(key)s differs '
-                                    '(source is %(source_value)s, destination '
-                                    'is %(target_value)s)'),
-                                {'image_id': image['id'],
-                                 'key': key,
-                                 'source_value': image[key],
-                                 'target_value': headers.get(key,
-                                                             'undefined')})
+                if image[key] != headers.get(key, None):
+                    logging.info(_('%(image_id)s: field %(key)s differs '
+                                 '(source is %(master_value)s, destination '
+                                 'is %(slave_value)s)')
+                                 % {'image_id': image['id'],
+                                    'key': key,
+                                    'master_value': image[key],
+                                    'slave_value': headers.get(key,
+                                                               'undefined')})
                     differences[image['id']] = 'diff'
                 else:
-                    LOG.debug('%(image_id)s is identical',
-                              {'image_id': image['id']})
+                    logging.debug(_('%(image_id)s is identical')
+                                  % {'image_id': image['id']})
 
         elif image['status'] == 'active':
-            LOG.warning(_LW('Image %(image_id)s ("%(image_name)s") '
-                            'entirely missing from the destination'),
-                        {'image_id': image['id'],
-                         'image_name': image.get('name', '--unnamed')})
+            logging.info(_('%s: entirely missing from the destination')
+                         % image['id'])
             differences[image['id']] = 'missing'
 
     return differences
@@ -671,7 +628,7 @@ def _check_upload_response_headers(headers, body):
                 return
 
         except Exception:
-            raise exception.UploadException(body)
+            raise UploadException('Image upload problem: %s' % body)
 
 
 def _image_present(client, image_uuid):
@@ -686,23 +643,57 @@ def _image_present(client, image_uuid):
     return 'status' in headers
 
 
+def parse_options(parser, cli_args):
+    """Returns the parsed CLI options, command to run and its arguments, merged
+    with any same-named options found in a configuration file
+
+    parser: the option parser
+    cli_args: the arguments passed on the command line
+
+    Returns: a tuple of (the parsed options, the command, the command name)
+    """
+    if not cli_args:
+        cli_args.append('-h')  # Show options in usage output...
+
+    (options, args) = parser.parse_args(cli_args)
+
+    # HACK(sirp): Make the parser available to the print_help method
+    # print_help is a command, so it only accepts (options, args); we could
+    # one-off have it take (parser, options, args), however, for now, I think
+    # this little hack will suffice
+    options.__parser = parser
+
+    if not args:
+        parser.print_usage()
+        sys.exit(0)
+
+    command_name = args.pop(0)
+    command = lookup_command(parser, command_name)
+
+    return (options, command, args)
+
+
 def print_help(options, args):
     """Print help specific to a command.
 
     options: the parsed command line options
     args: the command line
     """
-    if not args:
+    if len(args) != 1:
         print(COMMANDS)
-    else:
-        command_name = args.pop()
-        command = lookup_command(command_name)
-        print(command.__doc__ % {'prog': os.path.basename(sys.argv[0])})
+        sys.exit(1)
+
+    parser = options.__parser
+    command_name = args.pop()
+    command = lookup_command(parser, command_name)
+
+    print(command.__doc__ % {'prog': os.path.basename(sys.argv[0])})
 
 
-def lookup_command(command_name):
+def lookup_command(parser, command_name):
     """Lookup a command.
 
+    parser: the command parser
     command_name: the command name
 
     Returns: a method which implements that command
@@ -722,40 +713,96 @@ def lookup_command(command_name):
     try:
         command = commands[command_name]
     except KeyError:
-        if command_name:
-            sys.exit(_("Unknown command: %s") % command_name)
-        else:
-            command = commands['help']
+        parser.print_usage()
+        sys.exit(_("Unknown command: %s") % command_name)
+
     return command
 
 
+def logging_excepthook(type, value, tb):
+    extra = {}
+    extra['exc_info'] = (type, value, tb)
+    logging.critical(str(value), **extra)
+
+
 def main():
     """The main function."""
 
-    try:
-        config.parse_args()
-    except RuntimeError as e:
-        sys.exit("ERROR: %s" % encodeutils.exception_to_unicode(e))
-    except SystemExit:
-        sys.exit("Please specify one command")
+    usage = """
+%%prog <command> [options] [args]
+
+%s
+""" % COMMANDS
+
+    oparser = optparse.OptionParser(usage=usage.strip())
+
+    # Options
+    oparser.add_option('-c', '--chunksize', action="store", default=65536,
+                       help="Amount of data to transfer per HTTP write.")
+    oparser.add_option('-d', '--debug', action="store_true", default=False,
+                       help="Print debugging information.")
+    oparser.add_option('-D', '--dontreplicate', action="store",
+                       default=('created_at date deleted_at location '
+                                'updated_at'),
+                       help="List of fields to not replicate.")
+    oparser.add_option('-m', '--metaonly', action="store_true", default=False,
+                       help="Only replicate metadata, not images.")
+    oparser.add_option('-l', '--logfile', action="store", default='',
+                       help="Path of file to log to.")
+    oparser.add_option('-s', '--syslog', action="store_true", default=False,
+                       help="Log to syslog instead of a file.")
+    oparser.add_option('-t', '--token', action="store", default='',
+                       help=("Pass in your authentication token if you have "
+                             "one. If you use this option the same token is "
+                             "used for both the master and the slave."))
+    oparser.add_option('-M', '--mastertoken', action="store", default='',
+                       help=("Pass in your authentication token if you have "
+                             "one. This is the token used for the master."))
+    oparser.add_option('-S', '--slavetoken', action="store", default='',
+                       help=("Pass in your authentication token if you have "
+                             "one. This is the token used for the slave."))
+    oparser.add_option('-v', '--verbose', action="store_true", default=False,
+                       help="Print more verbose output.")
+
+    (options, command, args) = parse_options(oparser, sys.argv[1:])
 
     # Setup logging
-    logging.setup(CONF, 'glance')
+    root_logger = logging.root
+    if options.debug:
+        root_logger.setLevel(logging.DEBUG)
+    elif options.verbose:
+        root_logger.setLevel(logging.INFO)
+    else:
+        root_logger.setLevel(logging.WARNING)
 
-    if CONF.token:
-        CONF.sourcetoken = CONF.token
-        CONF.targettoken = CONF.token
+    formatter = logging.Formatter()
 
-    command = lookup_command(CONF.command)
+    if options.syslog:
+        handler = logging.handlers.SysLogHandler(address='/dev/log')
+    elif options.logfile:
+        handler = logging.handlers.WatchedFileHandler(options.logfile)
+    else:
+        handler = logging.StreamHandler(sys.stdout)
+
+    sys.excepthook = logging_excepthook
+
+    if options.token:
+        options.slavetoken = options.token
+        options.mastertoken = options.token
+
+    handler.setFormatter(formatter)
+    root_logger.addHandler(handler)
 
     try:
-        command(CONF, CONF.args)
+        command(options, args)
     except TypeError as e:
-        LOG.error(_LE(command.__doc__) % {'prog': command.__name__})  # noqa
-        sys.exit("ERROR: %s" % encodeutils.exception_to_unicode(e))
+        logging.error(command.__doc__ % {'prog': command.__name__})
+        sys.exit("ERROR: %s" % e)
     except ValueError as e:
-        LOG.error(_LE(command.__doc__) % {'prog': command.__name__})  # noqa
-        sys.exit("ERROR: %s" % encodeutils.exception_to_unicode(e))
+        logging.error(command.__doc__ % {'prog': command.__name__})
+        sys.exit("ERROR: %s" % e)
+    except Exception as e:
+        raise
 
 
 if __name__ == '__main__':
