@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,7 +14,7 @@
 #    under the License.
 
 """
-This auth module is intended to allow Openstack client-tools to select from a
+This auth module is intended to allow OpenStack client-tools to select from a
 variety of authentication strategies, including NoAuth (the default), and
 Keystone (an identity management system).
 
@@ -30,12 +28,12 @@ Keystone (an identity management system).
     > auth_plugin.management_url
     http://service_endpoint/
 """
-import json
-import urlparse
-
 import httplib2
 
+import six.moves.urllib.parse as urlparse
+
 from glance.common import exception
+from glance.openstack.common import jsonutils
 import glance.openstack.common.log as logging
 
 
@@ -76,9 +74,10 @@ class NoAuthStrategy(BaseStrategy):
 class KeystoneStrategy(BaseStrategy):
     MAX_REDIRECTS = 10
 
-    def __init__(self, creds, insecure=False):
+    def __init__(self, creds, insecure=False, configure_via_auth=True):
         self.creds = creds
         self.insecure = insecure
+        self.configure_via_auth = configure_via_auth
         super(KeystoneStrategy, self).__init__()
 
     def check_auth_params(self):
@@ -137,7 +136,7 @@ class KeystoneStrategy(BaseStrategy):
                 if 'v2.0' not in auth_url:
                     auth_url = urlparse.urljoin(auth_url, 'v2.0/')
             else:
-                # If we sucessfully auth'd, then memorize the correct auth_url
+                # If we successfully auth'd, then memorize the correct auth_url
                 # for future use.
                 self.creds['auth_url'] = auth_url
                 break
@@ -170,12 +169,13 @@ class KeystoneStrategy(BaseStrategy):
 
         if resp.status in (200, 204):
             try:
-                self.management_url = _management_url(self, resp)
+                if self.configure_via_auth:
+                    self.management_url = _management_url(self, resp)
                 self.auth_token = resp['x-auth-token']
             except KeyError:
                 raise exception.AuthorizationFailure()
         elif resp.status == 305:
-            raise exception.AuthorizationRedirect(resp['location'])
+            raise exception.AuthorizationRedirect(uri=resp['location'])
         elif resp.status == 400:
             raise exception.AuthBadRequest(url=token_url)
         elif resp.status == 401:
@@ -201,16 +201,18 @@ class KeystoneStrategy(BaseStrategy):
 
         headers = {}
         headers['Content-Type'] = 'application/json'
-        req_body = json.dumps(creds)
+        req_body = jsonutils.dumps(creds)
 
         resp, resp_body = self._do_request(
-                token_url, 'POST', headers=headers, body=req_body)
+            token_url, 'POST', headers=headers, body=req_body)
 
         if resp.status == 200:
-            resp_auth = json.loads(resp_body)['access']
+            resp_auth = jsonutils.loads(resp_body)['access']
             creds_region = self.creds.get('region')
-            self.management_url = get_endpoint(resp_auth['serviceCatalog'],
-                                               endpoint_region=creds_region)
+            if self.configure_via_auth:
+                endpoint = get_endpoint(resp_auth['serviceCatalog'],
+                                        endpoint_region=creds_region)
+                self.management_url = endpoint
             self.auth_token = resp_auth['token']['id']
         elif resp.status == 305:
             raise exception.RedirectException(resp['location'])
@@ -241,11 +243,13 @@ class KeystoneStrategy(BaseStrategy):
         return resp, resp_body
 
 
-def get_plugin_from_strategy(strategy, creds=None, insecure=False):
+def get_plugin_from_strategy(strategy, creds=None, insecure=False,
+                             configure_via_auth=True):
     if strategy == 'noauth':
         return NoAuthStrategy()
     elif strategy == 'keystone':
-        return KeystoneStrategy(creds, insecure)
+        return KeystoneStrategy(creds, insecure,
+                                configure_via_auth=configure_via_auth)
     else:
         raise Exception(_("Unknown auth strategy '%s'") % strategy)
 
