@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2010-2011 OpenStack, LLC
+# Copyright 2010-2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -20,28 +18,33 @@
 #   577548-https-httplib-client-connection-with-certificate-v/
 
 import collections
+import copy
 import errno
 import functools
 import httplib
 import os
 import re
-import urllib
-import urlparse
 
 try:
-    from eventlet.green import socket, ssl
+    from eventlet.green import socket
+    from eventlet.green import ssl
 except ImportError:
     import socket
     import ssl
 
 try:
-    import sendfile
+    import sendfile  # noqa
     SENDFILE_SUPPORTED = True
 except ImportError:
     SENDFILE_SUPPORTED = False
 
+import six
+import six.moves.urllib.parse as urlparse
+from six.moves import xrange
+
 from glance.common import auth
-from glance.common import exception, utils
+from glance.common import exception
+from glance.common import utils
 import glance.openstack.common.log as logging
 from glance.openstack.common import strutils
 
@@ -250,21 +253,21 @@ class BaseClient(object):
                 raise exception.ClientConnectionError(msg)
 
             if (self.key_file is not None and
-                not os.path.exists(self.key_file)):
+                    not os.path.exists(self.key_file)):
                 msg = _("The key file you specified %s does not "
                         "exist") % self.key_file
                 raise exception.ClientConnectionError(msg)
             connect_kwargs['key_file'] = self.key_file
 
             if (self.cert_file is not None and
-                not os.path.exists(self.cert_file)):
+                    not os.path.exists(self.cert_file)):
                 msg = _("The cert file you specified %s does not "
                         "exist") % self.cert_file
                 raise exception.ClientConnectionError(msg)
             connect_kwargs['cert_file'] = self.cert_file
 
             if (self.ca_file is not None and
-                not os.path.exists(self.ca_file)):
+                    not os.path.exists(self.ca_file)):
                 msg = _("The CA file you specified %s does not "
                         "exist") % self.ca_file
                 raise exception.ClientConnectionError(msg)
@@ -315,8 +318,9 @@ class BaseClient(object):
             if self.DEFAULT_DOC_ROOT:
                 doc_root = self.DEFAULT_DOC_ROOT.lstrip('/')
                 self.doc_root += '/' + doc_root
-                msg = _("Appending doc_root %(doc_root)s to URL %(url)s")
-                LOG.debug(msg % locals())
+                msg = (_("Appending doc_root %(doc_root)s to URL %(url)s") %
+                       {'doc_root': doc_root, 'url': url})
+                LOG.debug(msg)
 
         # ensure connection kwargs are re-evaluated after the service catalog
         # publicURL is parsed for potential SSL usage
@@ -327,7 +331,8 @@ class BaseClient(object):
         Returns an instantiated authentication plugin.
         """
         strategy = creds.get('strategy', 'noauth')
-        plugin = auth.get_plugin_from_strategy(strategy, creds, insecure)
+        plugin = auth.get_plugin_from_strategy(strategy, creds, insecure,
+                                               self.configure_via_auth)
         return plugin
 
     def get_connection_type(self):
@@ -373,14 +378,18 @@ class BaseClient(object):
             self._authenticate()
 
         url = self._construct_url(action, params)
-        return self._do_request(method=method, url=url, body=body,
-                                headers=headers)
+        # NOTE(ameade): We need to copy these kwargs since they can be altered
+        # in _do_request but we need the originals if handle_unauthenticated
+        # calls this function again.
+        return self._do_request(method=method, url=url,
+                                body=copy.deepcopy(body),
+                                headers=copy.deepcopy(headers))
 
     def _construct_url(self, action, params=None):
         """
         Create a URL object we can use to pass to _do_request().
         """
-        action = urllib.quote(action)
+        action = urlparse.quote(action)
         path = '/'.join([self.doc_root or '', action.lstrip('/')])
         scheme = "https" if self.use_ssl else "http"
         netloc = "%s:%d" % (self.host, self.port)
@@ -390,10 +399,10 @@ class BaseClient(object):
                 if value is None:
                     del params[key]
                     continue
-                if not isinstance(value, basestring):
+                if not isinstance(value, six.string_types):
                     value = str(value)
                 params[key] = strutils.safe_encode(value)
-            query = urllib.urlencode(params)
+            query = urlparse.urlencode(params)
         else:
             query = None
 
@@ -457,7 +466,7 @@ class BaseClient(object):
                 return method.lower() in ('post', 'put')
 
             def _simple(body):
-                return body is None or isinstance(body, basestring)
+                return body is None or isinstance(body, six.string_types)
 
             def _filelike(body):
                 return hasattr(body, 'read')
@@ -537,7 +546,7 @@ class BaseClient(object):
                 raise exception.UnexpectedStatus(status=status_code,
                                                  body=res.read())
 
-        except (socket.error, IOError), e:
+        except (socket.error, IOError) as e:
             raise exception.ClientConnectionError(e)
 
     def _seekable(self, body):
