@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -18,7 +16,8 @@
 
 """Glance exception subclasses"""
 
-import urlparse
+import six
+import six.moves.urllib.parse as urlparse
 
 _FATAL_EXCEPTION_FORMAT_ERRORS = False
 
@@ -42,19 +41,22 @@ class GlanceException(Exception):
         if not message:
             message = self.message
         try:
-            message = message % kwargs
+            if kwargs:
+                message = message % kwargs
         except Exception:
             if _FATAL_EXCEPTION_FORMAT_ERRORS:
                 raise
             else:
                 # at least get the core message out if something happened
                 pass
-
+        self.msg = message
         super(GlanceException, self).__init__(message)
 
-
-class MissingArgumentError(GlanceException):
-    message = _("Missing required argument.")
+    def __unicode__(self):
+        # NOTE(flwang): By default, self.msg is an instance of Message, which
+        # can't be converted by str(). Based on the definition of
+        # __unicode__, it should return unicode always.
+        return six.text_type(self.msg)
 
 
 class MissingCredentialError(GlanceException):
@@ -82,6 +84,11 @@ class Duplicate(GlanceException):
     message = _("An object with the same identifier already exists.")
 
 
+class Conflict(GlanceException):
+    message = _("An object with the same identifier is currently being "
+                "operated on.")
+
+
 class StorageFull(GlanceException):
     message = _("There is not enough disk space on the image storage media.")
 
@@ -123,11 +130,6 @@ class ProtectedImageDelete(Forbidden):
     message = _("Image %(image_id)s is protected and cannot be deleted.")
 
 
-#NOTE(bcwaldon): here for backwards-compatability, need to deprecate.
-class NotAuthorized(Forbidden):
-    message = _("You are not authorized to complete this action.")
-
-
 class Invalid(GlanceException):
     message = _("Data supplied was not valid.")
 
@@ -136,6 +138,10 @@ class InvalidSortKey(Invalid):
     message = _("Sort key supplied was not valid.")
 
 
+class InvalidPropertyProtectionConfiguration(Invalid):
+    message = _("Invalid configuration in property protection file.")
+
+
 class InvalidFilterRangeValue(Invalid):
     message = _("Unable to filter using the specified range.")
 
@@ -152,10 +158,6 @@ class AuthorizationRedirect(GlanceException):
     message = _("Redirecting to %(uri)s for authorization.")
 
 
-class DatabaseMigrationError(GlanceException):
-    message = _("There was an error migrating the database.")
-
-
 class ClientConnectionError(GlanceException):
     message = _("There was an error connecting to a server")
 
@@ -237,10 +239,6 @@ class StoreAddDisabled(GlanceException):
                 "store is disabled.")
 
 
-class InvalidNotifierStrategy(GlanceException):
-    message = _("'%(strategy)s' is not an available notifier strategy.")
-
-
 class MaxRedirectsExceeded(GlanceException):
     message = _("Maximum redirects (%(redirects)s) was exceeded.")
 
@@ -285,5 +283,65 @@ class ImageSizeLimitExceeded(GlanceException):
     message = _("The provided image is too large.")
 
 
+class ImageMemberLimitExceeded(LimitExceeded):
+    message = _("The limit has been exceeded on the number of allowed image "
+                "members for this image. Attempted: %(attempted)s, "
+                "Maximum: %(maximum)s")
+
+
+class ImagePropertyLimitExceeded(LimitExceeded):
+    message = _("The limit has been exceeded on the number of allowed image "
+                "properties. Attempted: %(attempted)s, Maximum: %(maximum)s")
+
+
+class ImageTagLimitExceeded(LimitExceeded):
+    message = _("The limit has been exceeded on the number of allowed image "
+                "tags. Attempted: %(attempted)s, Maximum: %(maximum)s")
+
+
+class ImageLocationLimitExceeded(LimitExceeded):
+    message = _("The limit has been exceeded on the number of allowed image "
+                "locations. Attempted: %(attempted)s, Maximum: %(maximum)s")
+
+
 class RPCError(GlanceException):
     message = _("%(cls)s exception was raised in the last rpc call: %(val)s")
+
+
+class TaskException(GlanceException):
+    message = _("An unknown task exception occurred")
+
+
+class TaskNotFound(TaskException, NotFound):
+    message = _("Task with the given id %(task_id)s was not found")
+
+
+class InvalidTaskStatus(TaskException, Invalid):
+    message = _("Provided status of task is unsupported: %(status)s")
+
+
+class InvalidTaskType(TaskException, Invalid):
+    message = _("Provided type of task is unsupported: %(type)s")
+
+
+class InvalidTaskStatusTransition(TaskException, Invalid):
+    message = _("Status transition from %(cur_status)s to"
+                " %(new_status)s is not allowed")
+
+
+class DuplicateLocation(Duplicate):
+    message = _("The location %(location)s already exists")
+
+
+class ImageDataNotFound(NotFound):
+    message = _("No image data could be found")
+
+
+class InvalidParameterValue(Invalid):
+    message = _("Invalid value '%(value)s' for parameter '%(param)s': "
+                "%(extra_msg)s")
+
+
+class InvalidImageStatusTransition(Invalid):
+    message = _("Image status transition from %(cur_status)s to"
+                " %(new_status)s is not allowed")
