@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011-2012 OpenStack LLC.
+# Copyright 2011-2012 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,9 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import uuid
+
 from glance.api import policy
 from glance.openstack.common import local
-from glance.openstack.common import uuidutils
 
 
 class RequestContext(object):
@@ -26,10 +25,13 @@ class RequestContext(object):
     accesses the system, as well as additional request information.
     """
 
+    user_idt_format = '{user} {tenant} {domain} {user_domain} {p_domain}'
+
     def __init__(self, auth_tok=None, user=None, tenant=None, roles=None,
                  is_admin=False, read_only=False, show_deleted=False,
                  owner_is_tenant=True, service_catalog=None,
-                 policy_enforcer=None):
+                 policy_enforcer=None, domain=None, user_domain=None,
+                 project_domain=None):
         self.auth_tok = auth_tok
         self.user = user
         self.tenant = tenant
@@ -37,10 +39,13 @@ class RequestContext(object):
         self.read_only = read_only
         self._show_deleted = show_deleted
         self.owner_is_tenant = owner_is_tenant
-        self.request_id = uuidutils.generate_uuid()
+        self.request_id = str(uuid.uuid4())
         self.service_catalog = service_catalog
         self.policy_enforcer = policy_enforcer or policy.Enforcer()
         self.is_admin = is_admin
+        self.domain = domain
+        self.user_domain = user_domain
+        self.project_domain = project_domain
         if not self.is_admin:
             self.is_admin = \
                 self.policy_enforcer.check_is_admin(self)
@@ -51,6 +56,14 @@ class RequestContext(object):
     def to_dict(self):
         # NOTE(ameade): These keys are named to correspond with the default
         # format string for logging the context in openstack common
+
+        user_idt = (
+            self.user_idt_format.format(user=self.user or '-',
+                                        tenant=self.tenant or '-',
+                                        domain=self.domain or '-',
+                                        user_domain=self.user_domain or '-',
+                                        p_domain=self.project_domain or '-'))
+
         return {
             'request_id': self.request_id,
 
@@ -68,6 +81,7 @@ class RequestContext(object):
             'roles': self.roles,
             'auth_token': self.auth_tok,
             'service_catalog': self.service_catalog,
+            'user_identity': user_idt
         }
 
     @classmethod
