@@ -1,4 +1,5 @@
 # Copyright 2012 OpenStack, Foundation
+# Copyright 2013 IBM Corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,11 +16,11 @@
 
 import copy
 import functools
+import uuid
 
 from glance.common import exception
 import glance.openstack.common.log as logging
 from glance.openstack.common import timeutils
-from glance.openstack.common import uuidutils
 
 
 LOG = logging.getLogger(__name__)
@@ -29,6 +30,8 @@ DATA = {
     'members': {},
     'tags': {},
     'locations': [],
+    'tasks': {},
+    'task_info': {}
 }
 
 
@@ -54,18 +57,11 @@ def reset():
         'members': [],
         'tags': {},
         'locations': [],
+        'tasks': {},
+        'task_info': {}
     }
 
 
-def setup_db_env(*args, **kwargs):
-    """
-    Setup global environment configuration variables.
-
-    We have no connection-oriented environment variables, so this is a NOOP.
-    """
-    pass
-
-
 def clear_db_env(*args, **kwargs):
     """
     Setup global environment configuration variables.
@@ -82,7 +78,7 @@ def _get_session():
 def _image_locations_format(image_id, value, meta_data):
     dt = timeutils.utcnow()
     return {
-        'id': uuidutils.generate_uuid(),
+        'id': str(uuid.uuid4()),
         'image_id': image_id,
         'created_at': dt,
         'updated_at': dt,
@@ -106,7 +102,7 @@ def _image_property_format(image_id, name, value):
 def _image_member_format(image_id, tenant_id, can_share, status='pending'):
     dt = timeutils.utcnow()
     return {
-        'id': uuidutils.generate_uuid(),
+        'id': str(uuid.uuid4()),
         'image_id': image_id,
         'member': tenant_id,
         'can_share': can_share,
@@ -116,6 +112,53 @@ def _image_member_format(image_id, tenant_id, can_share, status='pending'):
     }
 
 
+def _pop_task_info_values(values):
+    task_info_values = {}
+    for k, v in values.items():
+        if k in ['input', 'result', 'message']:
+            values.pop(k)
+            task_info_values[k] = v
+
+    return task_info_values
+
+
+def _format_task_from_db(task_ref, task_info_ref):
+    task = copy.deepcopy(task_ref)
+    if task_info_ref:
+        task_info = copy.deepcopy(task_info_ref)
+        task_info_values = _pop_task_info_values(task_info)
+        task.update(task_info_values)
+    return task
+
+
+def _task_format(task_id, **values):
+    dt = timeutils.utcnow()
+    task = {
+        'id': task_id,
+        'type': 'import',
+        'status': 'pending',
+        'owner': None,
+        'expires_at': None,
+        'created_at': dt,
+        'updated_at': dt,
+        'deleted_at': None,
+        'deleted': False,
+    }
+    task.update(values)
+    return task
+
+
+def _task_info_format(task_id, **values):
+    task_info = {
+        'task_id': task_id,
+        'input': None,
+        'result': None,
+        'message': None,
+    }
+    task_info.update(values)
+    return task_info
+
+
 def _image_format(image_id, **values):
     dt = timeutils.utcnow()
     image = {
@@ -131,6 +174,7 @@ def _image_format(image_id, **values):
         'min_ram': 0,
         'min_disk': 0,
         'size': None,
+        'virtual_size': None,
         'checksum': None,
         'tags': [],
         'created_at': dt,
@@ -201,7 +245,7 @@ def _filter_images(images, filters, context,
             if not image['is_public'] == is_public:
                 continue
 
-        add = True
+        to_add = True
         for k, value in filters.iteritems():
             key = k
             if k.endswith('_min') or k.endswith('_max'):
@@ -213,29 +257,30 @@ def _filter_images(images, filters, context,
                             "with a non-numeric value.")
                     raise exception.InvalidFilterRangeValue(msg)
             if k.endswith('_min'):
-                add = image.get(key) >= value
+                to_add = image.get(key) >= value
             elif k.endswith('_max'):
-                add = image.get(key) <= value
+                to_add = image.get(key) <= value
             elif k != 'is_public' and image.get(k) is not None:
-                add = image.get(key) == value
+                to_add = image.get(key) == value
             elif k == 'tags':
                 filter_tags = value
                 image_tags = image_tag_get_all(context, image['id'])
                 for tag in filter_tags:
                     if tag not in image_tags:
-                        add = False
+                        to_add = False
                         break
             else:
-                properties = {}
+                to_add = False
                 for p in image['properties']:
                     properties = {p['name']: p['value'],
                                   'deleted': p['deleted']}
-                add = (properties.get(key) == value and
-                       properties.get('deleted') is False)
-            if not add:
+                    to_add |= (properties.get(key) == value and
+                               properties.get('deleted') is False)
+
+            if not to_add:
                 break
 
-        if add:
+        if to_add:
             filtered_images.append(image)
 
     return filtered_images
@@ -367,7 +412,21 @@ def image_member_find(context, image_id=None, member=None, status=None):
 
     for f in filters:
         members = filter(f, members)
-    return [copy.deepcopy(member) for member in members]
+    return [copy.deepcopy(m) for m in members]
+
+
+@log_call
+def image_member_count(context, image_id):
+    """Return the number of image members for this image
+
+    :param image_id: identifier of image entity
+    """
+    if not image_id:
+        msg = _("Image id is required.")
+        raise exception.Invalid(msg)
+
+    members = DATA['members']
+    return len(filter(lambda x: x['image_id'] == image_id, members))
 
 
 @log_call
@@ -443,7 +502,7 @@ def _image_location_get_all(image_id):
 @log_call
 def image_create(context, image_values):
     global DATA
-    image_id = image_values.get('id', uuidutils.generate_uuid())
+    image_id = image_values.get('id', str(uuid.uuid4()))
 
     if image_id in DATA['images']:
         raise exception.Duplicate()
@@ -452,10 +511,10 @@ def image_create(context, image_values):
         raise exception.Invalid('status is a required attribute')
 
     allowed_keys = set(['id', 'name', 'status', 'min_ram', 'min_disk', 'size',
-                        'checksum', 'locations', 'owner', 'protected',
-                        'is_public', 'container_format', 'disk_format',
-                        'created_at', 'updated_at', 'deleted_at', 'deleted',
-                        'properties', 'tags'])
+                        'virtual_size', 'checksum', 'locations', 'owner',
+                        'protected', 'is_public', 'container_format',
+                        'disk_format', 'created_at', 'updated_at', 'deleted',
+                        'deleted_at', 'properties', 'tags'])
 
     incorrect_keys = set(image_values.keys()) - allowed_keys
     if incorrect_keys:
@@ -465,7 +524,7 @@ def image_create(context, image_values):
     image = _image_format(image_id, **image_values)
     DATA['images'][image_id] = image
 
-    location_data = image_values.get('locations', None)
+    location_data = image_values.get('locations')
     if location_data is not None:
         _image_locations_set(image_id, location_data)
 
@@ -475,12 +534,13 @@ def image_create(context, image_values):
 
 
 @log_call
-def image_update(context, image_id, image_values, purge_props=False):
+def image_update(context, image_id, image_values, purge_props=False,
+                 from_state=None):
     global DATA
     try:
         image = DATA['images'][image_id]
     except KeyError:
-        raise exception.NotFound(image_id=image_id)
+        raise exception.NotFound()
 
     location_data = image_values.pop('locations', None)
     if location_data is not None:
@@ -589,40 +649,6 @@ def is_image_mutable(context, image):
     return image['owner'] == context.owner
 
 
-def is_image_sharable(context, image, **kwargs):
-    """Return True if the image can be shared to others in this context."""
-    # Is admin == image sharable
-    if context.is_admin:
-        return True
-
-    # Only allow sharing if we have an owner
-    if context.owner is None:
-        return False
-
-    # If we own the image, we can share it
-    if context.owner == image['owner']:
-        return True
-
-    # Let's get the membership association
-    if 'membership' in kwargs:
-        member = kwargs['membership']
-        if member is None:
-            # Not shared with us anyway
-            return False
-    else:
-        members = image_member_find(context,
-                                    image_id=image['id'],
-                                    member=context.owner)
-        if members:
-            member = members[0]
-        else:
-            # Not shared with us anyway
-            return False
-
-    # It's the can_share attribute we're now interested in
-    return member['can_share']
-
-
 def is_image_visible(context, image, status=None):
     """Return True if the image is visible in this context."""
     # Is admin == image visible
@@ -668,3 +694,239 @@ def user_get_storage_usage(context, owner_id, image_id=None, session=None):
                          if not l.get('deleted', False)]
             total += (image['size'] * len(locations))
     return total
+
+
+@log_call
+def task_create(context, values):
+    """Create a task object"""
+    global DATA
+
+    task_values = copy.deepcopy(values)
+    task_id = task_values.get('id', str(uuid.uuid4()))
+    required_attributes = ['type', 'status', 'input']
+    allowed_attributes = ['id', 'type', 'status', 'input', 'result', 'owner',
+                          'message', 'expires_at', 'created_at',
+                          'updated_at', 'deleted_at', 'deleted']
+
+    if task_id in DATA['tasks']:
+        raise exception.Duplicate()
+
+    for key in required_attributes:
+        if key not in task_values:
+            raise exception.Invalid('%s is a required attribute' % key)
+
+    incorrect_keys = set(task_values.keys()) - set(allowed_attributes)
+    if incorrect_keys:
+        raise exception.Invalid(
+            'The keys %s are not valid' % str(incorrect_keys))
+
+    task_info_values = _pop_task_info_values(task_values)
+    task = _task_format(task_id, **task_values)
+    DATA['tasks'][task_id] = task
+    task_info = _task_info_create(task['id'], task_info_values)
+
+    return _format_task_from_db(task, task_info)
+
+
+@log_call
+def task_update(context, task_id, values):
+    """Update a task object"""
+    global DATA
+    task_values = copy.deepcopy(values)
+    task_info_values = _pop_task_info_values(task_values)
+    try:
+        task = DATA['tasks'][task_id]
+    except KeyError:
+        msg = (_("No task found with ID %s") % task_id)
+        LOG.debug(msg)
+        raise exception.TaskNotFound(task_id=task_id)
+
+    task.update(task_values)
+    task['updated_at'] = timeutils.utcnow()
+    DATA['tasks'][task_id] = task
+    task_info = _task_info_update(task['id'], task_info_values)
+
+    return _format_task_from_db(task, task_info)
+
+
+@log_call
+def task_get(context, task_id, force_show_deleted=False):
+    task, task_info = _task_get(context, task_id, force_show_deleted)
+    return _format_task_from_db(task, task_info)
+
+
+def _task_get(context, task_id, force_show_deleted=False):
+    try:
+        task = DATA['tasks'][task_id]
+    except KeyError:
+        msg = _('Could not find task %s') % task_id
+        LOG.info(msg)
+        raise exception.TaskNotFound(task_id=task_id)
+
+    if task['deleted'] and not (force_show_deleted or context.show_deleted):
+        msg = _('Unable to get deleted task %s') % task_id
+        LOG.info(msg)
+        raise exception.TaskNotFound(task_id=task_id)
+
+    if not _is_task_visible(context, task):
+        msg = (_("Forbidding request, task %s is not visible") % task_id)
+        LOG.debug(msg)
+        raise exception.Forbidden(msg)
+
+    task_info = _task_info_get(task_id)
+
+    return task, task_info
+
+
+@log_call
+def task_delete(context, task_id):
+    global DATA
+    try:
+        DATA['tasks'][task_id]['deleted'] = True
+        DATA['tasks'][task_id]['deleted_at'] = timeutils.utcnow()
+        DATA['tasks'][task_id]['updated_at'] = timeutils.utcnow()
+        return copy.deepcopy(DATA['tasks'][task_id])
+    except KeyError:
+        msg = (_("No task found with ID %s") % task_id)
+        LOG.debug(msg)
+        raise exception.TaskNotFound(task_id=task_id)
+
+
+@log_call
+def task_get_all(context, filters=None, marker=None, limit=None,
+                 sort_key='created_at', sort_dir='desc'):
+    """
+    Get all tasks that match zero or more filters.
+
+    :param filters: dict of filter keys and values.
+    :param marker: task id after which to start page
+    :param limit: maximum number of tasks to return
+    :param sort_key: task attribute by which results should be sorted
+    :param sort_dir: direction in which results should be sorted (asc, desc)
+    :return: tasks set
+    """
+    filters = filters or {}
+    tasks = DATA['tasks'].values()
+    tasks = _filter_tasks(tasks, filters, context)
+    tasks = _sort_tasks(tasks, sort_key, sort_dir)
+    tasks = _paginate_tasks(context, tasks, marker, limit,
+                            filters.get('deleted'))
+
+    filtered_tasks = []
+    for task in tasks:
+        filtered_tasks.append(_format_task_from_db(task, task_info_ref=None))
+
+    return filtered_tasks
+
+
+def _is_task_visible(context, task):
+    """Return True if the task is visible in this context."""
+    # Is admin == task visible
+    if context.is_admin:
+        return True
+
+    # No owner == task visible
+    if task['owner'] is None:
+        return True
+
+    # Perform tests based on whether we have an owner
+    if context.owner is not None:
+        if context.owner == task['owner']:
+            return True
+
+    return False
+
+
+def _filter_tasks(tasks, filters, context, admin_as_user=False):
+    filtered_tasks = []
+
+    for task in tasks:
+        has_ownership = context.owner and task['owner'] == context.owner
+        can_see = (has_ownership or (context.is_admin and not admin_as_user))
+        if not can_see:
+            continue
+
+        add = True
+        for k, value in filters.iteritems():
+            add = task[k] == value and task['deleted'] is False
+            if not add:
+                break
+
+        if add:
+            filtered_tasks.append(task)
+
+    return filtered_tasks
+
+
+def _sort_tasks(tasks, sort_key, sort_dir):
+    reverse = False
+    if tasks and not (sort_key in tasks[0]):
+        raise exception.InvalidSortKey()
+    keyfn = lambda x: (x[sort_key] if x[sort_key] is not None else '',
+                       x['created_at'], x['id'])
+    reverse = sort_dir == 'desc'
+    tasks.sort(key=keyfn, reverse=reverse)
+
+    return tasks
+
+
+def _paginate_tasks(context, tasks, marker, limit, show_deleted):
+    start = 0
+    end = -1
+    if marker is None:
+        start = 0
+    else:
+        # Check that the task is accessible
+        _task_get(context, marker, force_show_deleted=show_deleted)
+
+        for i, task in enumerate(tasks):
+            if task['id'] == marker:
+                start = i + 1
+                break
+        else:
+            if task:
+                raise exception.TaskNotFound(task_id=task['id'])
+            else:
+                msg = _("Task does not exist")
+                raise exception.NotFound(message=msg)
+
+    end = start + limit if limit is not None else None
+    return tasks[start:end]
+
+
+def _task_info_create(task_id, values):
+    """Create a Task Info for Task with given task ID"""
+    global DATA
+    task_info = _task_info_format(task_id, **values)
+    DATA['task_info'][task_id] = task_info
+
+    return task_info
+
+
+def _task_info_update(task_id, values):
+    """Update Task Info for Task with given task ID and updated values"""
+    global DATA
+    try:
+        task_info = DATA['task_info'][task_id]
+    except KeyError:
+        msg = (_("No task info found with task id %s") % task_id)
+        LOG.debug(msg)
+        raise exception.TaskNotFound(task_id=task_id)
+
+    task_info.update(values)
+    DATA['task_info'][task_id] = task_info
+
+    return task_info
+
+
+def _task_info_get(task_id):
+    """Get Task Info for Task with given task ID"""
+    global DATA
+    try:
+        task_info = DATA['task_info'][task_id]
+    except KeyError:
+        msg = _('Could not find task info %s') % task_id
+        LOG.info(msg)
+        raise exception.TaskNotFound(task_id=task_id)
+
+    return task_info
