@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,7 +16,7 @@
 from sqlalchemy.schema import (Column, MetaData, Table)
 
 from glance.db.sqlalchemy.migrate_repo.schema import (
-    Boolean, DateTime, Integer, String, Text, create_tables, drop_tables)
+    Boolean, DateTime, Integer, String, Text, create_tables, drop_tables)  # noqa
 
 
 def define_images_table(meta):
