@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation.
 # All Rights Reserved.
 #
@@ -15,9 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import urllib
-import urlparse
-
+import six.moves.urllib.parse as urlparse
 import sqlalchemy
 
 from glance.common import exception
@@ -53,14 +49,20 @@ def migrate_location_credentials(migrate_engine, to_quoted):
                                       'swift')).execute())
 
     for image in images:
-        fixed_uri = legacy_parse_uri(image['location'], to_quoted,
-                                     image['id'])
-        images_table.update()\
-                    .where(images_table.c.id == image['id'])\
-                    .values(location=fixed_uri).execute()
+        try:
+            fixed_uri = legacy_parse_uri(image['location'], to_quoted)
+            images_table.update()\
+                        .where(images_table.c.id == image['id'])\
+                        .values(location=fixed_uri).execute()
+        except exception.BadStoreUri as e:
+            err_msg = _("Invalid store uri for image: %(image_id)s. "
+                        "Details: %(reason)s") % {'image_id': image.id,
+                                                  'reason': unicode(e)}
+            LOG.exception(err_msg)
+            raise
 
 
-def legacy_parse_uri(uri, to_quote, image_id):
+def legacy_parse_uri(uri, to_quote):
     """
     Parse URLs. This method fixes an issue where credentials specified
     in the URL are interpreted differently in Python 2.6.1+ than prior
@@ -88,7 +90,6 @@ def legacy_parse_uri(uri, to_quote, image_id):
                    "like so: "
                    "swift+http://user:pass@authurl.com/v1/container/obj")
 
-        LOG.error(_("Invalid store uri for image %s: %s") % (image_id, reason))
         raise exception.BadStoreUri(message=reason)
 
     pieces = urlparse.urlparse(uri)
@@ -120,9 +121,8 @@ def legacy_parse_uri(uri, to_quote, image_id):
         if to_quote:
             if len(cred_parts) == 1:
                 reason = (_("Badly formed credentials '%(creds)s' in Swift "
-                            "URI") % locals())
-                LOG.error(reason)
-                raise exception.BadStoreUri()
+                            "URI") % {'creds': creds})
+                raise exception.BadStoreUri(message=reason)
             elif len(cred_parts) == 3:
                 user = ':'.join(cred_parts[0:2])
             else:
@@ -133,11 +133,10 @@ def legacy_parse_uri(uri, to_quote, image_id):
         else:
             if len(cred_parts) != 2:
                 reason = (_("Badly formed credentials in Swift URI."))
-                LOG.debug(reason)
-                raise exception.BadStoreUri()
+                raise exception.BadStoreUri(message=reason)
             user, key = cred_parts
-            user = urllib.unquote(user)
-            key = urllib.unquote(key)
+            user = urlparse.unquote(user)
+            key = urlparse.unquote(key)
     else:
         user = None
         key = None
@@ -150,9 +149,8 @@ def legacy_parse_uri(uri, to_quote, image_id):
             path_parts.insert(0, netloc)
             auth_or_store_url = '/'.join(path_parts)
     except IndexError:
-        reason = _("Badly formed S3 URI: %s") % uri
-        LOG.error(message=reason)
-        raise exception.BadStoreUri()
+        reason = _("Badly formed S3 URI: %(uri)s") % {'uri': uri}
+        raise exception.BadStoreUri(message=reason)
 
     if auth_or_store_url.startswith('http://'):
         auth_or_store_url = auth_or_store_url[len('http://'):]
@@ -162,8 +160,8 @@ def legacy_parse_uri(uri, to_quote, image_id):
     credstring = ''
     if user and key:
         if to_quote:
-            quote_user = urllib.quote(user)
-            quote_key = urllib.quote(key)
+            quote_user = urlparse.quote(user)
+            quote_key = urlparse.quote(key)
         else:
             quote_user = user
             quote_key = key
