@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2013 OpenStack LLC.
+# Copyright 2013 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -28,17 +26,16 @@ migration performs the following steps for every entry in the images table:
 
 Fixes bug #1081043
 """
-
-import types
-import urlparse
+import types  # noqa
 
 from oslo.config import cfg
+import six.moves.urllib.parse as urlparse
 import sqlalchemy
 
 from glance.common import crypt
 from glance.common import exception
 import glance.openstack.common.log as logging
-import glance.store.swift
+import glance.store.swift  # noqa
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
@@ -85,8 +82,14 @@ def migrate_location_credentials(migrate_engine, to_quoted):
                         .where(images_table.c.id == image['id'])\
                         .values(location=fixed_uri).execute()
         except exception.Invalid:
-            msg = _("Failed to decrypt location value for image %s")
-            LOG.warn(msg % image['id'])
+            msg = _("Failed to decrypt location value for image %(image_id)s")
+            LOG.warn(msg % {'image_id': image['id']})
+        except exception.BadStoreUri as e:
+            err_msg = _("Invalid store uri for image: %(image_id)s. "
+                        "Details: %(reason)s") % {'image_id': image.id,
+                                                  'reason': unicode(e)}
+            LOG.exception(err_msg)
+            raise
 
 
 def decrypt_location(uri):
@@ -112,15 +115,6 @@ def fix_uri_credentials(uri, to_quoted):
     """
     if not uri:
         return
-    location = glance.store.swift.StoreLocation({})
-    if to_quoted:
-        # The legacy parse_uri doesn't unquote credentials
-        location.parse_uri = types.MethodType(legacy_parse_uri, location)
-    else:
-        # The legacy _get_credstring doesn't quote credentials
-        location._get_credstring = types.MethodType(legacy__get_credstring,
-                                                    location)
-    decrypted_uri = None
     try:
         decrypted_uri = decrypt_location(uri)
     #NOTE (ameade): If a uri is not encrypted or incorrectly encoded then we
@@ -128,17 +122,10 @@ def fix_uri_credentials(uri, to_quoted):
     except (TypeError, ValueError) as e:
         raise exception.Invalid(str(e))
 
-    location.parse_uri(decrypted_uri)
-    return encrypt_location(location.get_uri())
+    return legacy_parse_uri(decrypted_uri, to_quoted)
 
 
-def legacy__get_credstring(self):
-    if self.user:
-        return '%s:%s@' % (self.user, self.key)
-    return ''
-
-
-def legacy_parse_uri(self, uri):
+def legacy_parse_uri(uri, to_quote):
     """
     Parse URLs. This method fixes an issue where credentials specified
     in the URL are interpreted differently in Python 2.6.1+ than prior
@@ -146,6 +133,14 @@ def legacy_parse_uri(self, uri):
     Swift URIs have where a username can contain a ':', like so:
 
         swift://account:user:pass@authurl.com/container/obj
+
+    If to_quoted is True, the uri is assumed to have credentials that
+    have not been quoted, and the resulting uri will contain quoted
+    credentials.
+
+    If to_quoted is False, the uri is assumed to have credentials that
+    have been quoted, and the resulting uri will contain credentials
+    that have not been quoted.
     """
     # Make sure that URIs that contain multiple schemes, such as:
     # swift://user:pass@http://authurl.com/v1/container/obj
@@ -157,13 +152,11 @@ def legacy_parse_uri(self, uri):
                    ", you need to change it to use the swift+http:// scheme, "
                    "like so: "
                    "swift+http://user:pass@authurl.com/v1/container/obj")
-
-        LOG.error(_("Invalid store uri %(uri)s: %(reason)s") % locals())
         raise exception.BadStoreUri(message=reason)
 
     pieces = urlparse.urlparse(uri)
     assert pieces.scheme in ('swift', 'swift+http', 'swift+https')
-    self.scheme = pieces.scheme
+    scheme = pieces.scheme
     netloc = pieces.netloc
     path = pieces.path.lstrip('/')
     if netloc != '':
@@ -187,29 +180,59 @@ def legacy_parse_uri(self, uri):
         # User can be account:user, in which case cred_parts[0:2] will be
         # the account and user. Combine them into a single username of
         # account:user
-        if len(cred_parts) == 1:
-            reason = (_("Badly formed credentials '%(creds)s' in Swift "
-                        "URI") % locals())
-            LOG.error(reason)
-            raise exception.BadStoreUri()
-        elif len(cred_parts) == 3:
-            user = ':'.join(cred_parts[0:2])
+        if to_quote:
+            if len(cred_parts) == 1:
+                reason = (_("Badly formed credentials '%(creds)s' in Swift "
+                            "URI") % {'creds': creds})
+                raise exception.BadStoreUri(message=reason)
+            elif len(cred_parts) == 3:
+                user = ':'.join(cred_parts[0:2])
+            else:
+                user = cred_parts[0]
+            key = cred_parts[-1]
+            user = user
+            key = key
         else:
-            user = cred_parts[0]
-        key = cred_parts[-1]
-        self.user = user
-        self.key = key
+            if len(cred_parts) != 2:
+                reason = (_("Badly formed credentials in Swift URI."))
+                raise exception.BadStoreUri(message=reason)
+            user, key = cred_parts
+            user = urlparse.unquote(user)
+            key = urlparse.unquote(key)
     else:
-        self.user = None
+        user = None
+        key = None
     path_parts = path.split('/')
     try:
-        self.obj = path_parts.pop()
-        self.container = path_parts.pop()
+        obj = path_parts.pop()
+        container = path_parts.pop()
         if not netloc.startswith('http'):
             # push hostname back into the remaining to build full authurl
             path_parts.insert(0, netloc)
-            self.auth_or_store_url = '/'.join(path_parts)
+            auth_or_store_url = '/'.join(path_parts)
     except IndexError:
-        reason = _("Badly formed S3 URI: %s") % uri
-        LOG.error(message=reason)
-        raise exception.BadStoreUri()
+        reason = _("Badly formed S3 URI: %(uri)s") % {'uri': uri}
+        raise exception.BadStoreUri(message=reason)
+
+    if auth_or_store_url.startswith('http://'):
+        auth_or_store_url = auth_or_store_url[len('http://'):]
+    elif auth_or_store_url.startswith('https://'):
+        auth_or_store_url = auth_or_store_url[len('https://'):]
+
+    credstring = ''
+    if user and key:
+        if to_quote:
+            quote_user = urlparse.quote(user)
+            quote_key = urlparse.quote(key)
+        else:
+            quote_user = user
+            quote_key = key
+        credstring = '%s:%s@' % (quote_user, quote_key)
+
+    auth_or_store_url = auth_or_store_url.strip('/')
+    container = container.strip('/')
+    obj = obj.strip('/')
+
+    uri = '%s://%s%s/%s/%s' % (scheme, credstring, auth_or_store_url,
+                               container, obj)
+    return encrypt_location(uri)
