@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 Rackspace Hosting
 # All Rights Reserved.
 #
@@ -19,7 +17,7 @@ import json
 import pickle
 
 import sqlalchemy
-from sqlalchemy import MetaData, Table, Column
+from sqlalchemy import MetaData, Table, Column  # noqa
 from glance.db.sqlalchemy import models
 
 
