@@ -17,7 +17,7 @@
 from sqlalchemy.schema import (Column, MetaData, Table, Index)
 
 from glance.db.sqlalchemy.migrate_repo.schema import (
-    Boolean, DateTime, String, Text, create_tables)  # noqa
+    Boolean, DateTime, String, Text, create_tables, drop_tables)  # noqa
 
 
 def define_tasks_table(meta):
@@ -39,7 +39,6 @@ def define_tasks_table(meta):
                          nullable=False,
                          default=False),
                   mysql_engine='InnoDB',
-                  mysql_charset='utf8',
                   extend_existing=True)
 
     Index('ix_tasks_type', tasks.c.type)
@@ -56,3 +55,10 @@ def upgrade(migrate_engine):
     meta.bind = migrate_engine
     tables = [define_tasks_table(meta)]
     create_tables(tables)
+
+
+def downgrade(migrate_engine):
+    meta = MetaData()
+    meta.bind = migrate_engine
+    tables = [define_tasks_table(meta)]
+    drop_tables(tables)
