@@ -1,4 +1,4 @@
-# Copyright 2018 RedHat Inc.
+# Copyright 2012 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,17 +15,15 @@
 
 import os
 
-from oslo_serialization import jsonutils as json
-
 from glance.common import client as base_client
 from glance.common import exception
-from glance.i18n import _
+import glance.openstack.common.jsonutils as json
 
 
 class CacheClient(base_client.BaseClient):
 
     DEFAULT_PORT = 9292
-    DEFAULT_DOC_ROOT = '/v2'
+    DEFAULT_DOC_ROOT = '/v1'
 
     def delete_cached_image(self, image_id):
         """
@@ -84,10 +82,10 @@ class CacheClient(base_client.BaseClient):
 
 
 def get_client(host, port=None, timeout=None, use_ssl=False, username=None,
-               password=None, project=None,
-               user_domain_id=None, project_domain_id=None,
+               password=None, tenant=None,
                auth_url=None, auth_strategy=None,
-               auth_token=None, region=None, insecure=False):
+               auth_token=None, region=None,
+               is_silent_upload=False, insecure=False):
     """
     Returns a new client Glance client object based on common kwargs.
     If an option isn't specified falls back to common environment variable
@@ -104,8 +102,8 @@ def get_client(host, port=None, timeout=None, use_ssl=False, username=None,
         os.getenv('OS_AUTH_USER', os.getenv('OS_USERNAME')),
         'password': password or
         os.getenv('OS_AUTH_KEY', os.getenv('OS_PASSWORD')),
-        'project': project or
-        os.getenv('OS_AUTH_PROJECT', os.getenv('OS_PROJECT_NAME')),
+        'tenant': tenant or
+        os.getenv('OS_AUTH_TENANT', os.getenv('OS_TENANT_NAME')),
         'auth_url': auth_url or
         os.getenv('OS_AUTH_URL'),
         'strategy': force_strategy or
@@ -113,10 +111,6 @@ def get_client(host, port=None, timeout=None, use_ssl=False, username=None,
         os.getenv('OS_AUTH_STRATEGY', 'noauth'),
         'region': region or
         os.getenv('OS_REGION_NAME'),
-        'user_domain_id': user_domain_id or os.getenv(
-            'OS_USER_DOMAIN_ID', 'default'),
-        'project_domain_id': project_domain_id or os.getenv(
-            'OS_PROJECT_DOMAIN_ID', 'default')
     }
 
     if creds['strategy'] == 'keystone' and not creds['auth_url']:
@@ -129,8 +123,7 @@ def get_client(host, port=None, timeout=None, use_ssl=False, username=None,
         port=port,
         timeout=timeout,
         use_ssl=use_ssl,
-        auth_token=auth_token or
+        auth_tok=auth_token or
         os.getenv('OS_TOKEN'),
         creds=creds,
-        insecure=insecure,
-        configure_via_auth=False)
+        insecure=insecure)
