@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -25,7 +23,8 @@ import os
 import stat
 import time
 
-from eventlet import sleep, timeout
+from eventlet import sleep
+from eventlet import timeout
 from oslo.config import cfg
 import sqlite3
 
@@ -71,7 +70,7 @@ class SqliteConnection(sqlite3.Connection):
 
     def execute(self, *args, **kwargs):
         return self._timeout(lambda: sqlite3.Connection.execute(
-                                        self, *args, **kwargs))
+            self, *args, **kwargs))
 
     def commit(self):
         return self._timeout(lambda: sqlite3.Connection.commit(self))
@@ -281,8 +280,12 @@ class Driver(base.Driver):
                 return None
 
         path = self.get_image_filepath(image_id)
-        file_info = os.stat(path)
-        return image_id, file_info[stat.ST_SIZE]
+        try:
+            file_info = os.stat(path)
+            size = file_info[stat.ST_SIZE]
+        except OSError:
+            size = 0
+        return image_id, size
 
     @contextmanager
     def open_for_write(self, image_id):
@@ -323,7 +326,10 @@ class Driver(base.Driver):
 
                     LOG.debug(_("Fetch of cache file failed (%(e)s), rolling "
                                 "back by moving '%(incomplete_path)s' to "
-                                "'%(invalid_path)s'") % locals())
+                                "'%(invalid_path)s'"),
+                              {'e': e,
+                               'incomplete_path': incomplete_path,
+                               'invalid_path': invalid_path})
                     os.rename(incomplete_path, invalid_path)
 
                 db.execute("""DELETE FROM cached_images
@@ -417,7 +423,7 @@ class Driver(base.Driver):
         path = self.get_image_filepath(image_id, 'queue')
 
         # Touch the file to add it to the queue
-        with open(path, "w") as f:
+        with open(path, "w"):
             pass
 
         return True
