@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2010-2011 OpenStack LLC.
+# Copyright 2010-2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -29,7 +27,6 @@ import glance.db
 import glance.openstack.common.log as logging
 from glance.openstack.common import strutils
 from glance.openstack.common import timeutils
-from glance.openstack.common import uuidutils
 
 
 LOG = logging.getLogger(__name__)
@@ -92,12 +89,9 @@ class Controller(object):
 
     def __init__(self):
         self.db_api = glance.db.get_api()
-        self.db_api.setup_db_env()
 
     def _get_images(self, context, filters, **params):
-        """
-        Get images, wrapping in exception if necessary.
-        """
+        """Get images, wrapping in exception if necessary."""
         # NOTE(markwash): for backwards compatibility, is_public=True for
         # admins actually means "treat me as if I'm not an admin and show me
         # all my images"
@@ -107,13 +101,22 @@ class Controller(object):
         try:
             return self.db_api.image_get_all(context, filters=filters,
                                              **params)
-        except (exception.NotFound, exception.Forbidden) as e:
+        except exception.NotFound:
+            LOG.info(_("Invalid marker. Image %(id)s could not be "
+                       "found.") % {'id': params.get('marker')})
             msg = _("Invalid marker. Image could not be found.")
             raise exc.HTTPBadRequest(explanation=msg)
+        except exception.Forbidden:
+            LOG.info(_("Access denied to image %(id)s but returning "
+                       "'not found'") % {'id': params.get('marker')})
+            msg = _("Invalid marker. Image could not be found.")
+            raise exc.HTTPBadRequest(explanation=msg)
+        except Exception:
+            LOG.exception(_("Unable to get images"))
+            raise
 
     def index(self, req):
-        """
-        Return a basic filtered list of public, non-deleted images
+        """Return a basic filtered list of public, non-deleted images
 
         :param req: the Request object coming from the wsgi layer
         :retval a mapping of the following form::
@@ -145,8 +148,7 @@ class Controller(object):
         return dict(images=results)
 
     def detail(self, req):
-        """
-        Return a filtered list of public, non-deleted images in detail
+        """Return a filtered list of public, non-deleted images in detail
 
         :param req: the Request object coming from the wsgi layer
         :retval a mapping of the following form::
@@ -164,8 +166,7 @@ class Controller(object):
         return dict(images=image_dicts)
 
     def _get_query_params(self, req):
-        """
-        Extract necessary query parameters from http request.
+        """Extract necessary query parameters from http request.
 
         :param req: the Request object coming from the wsgi layer
         :retval dictionary of filters to apply to list of images
@@ -192,8 +193,7 @@ class Controller(object):
         return params
 
     def _get_filters(self, req):
-        """
-        Return a dictionary of query param filters from the request
+        """Return a dictionary of query param filters from the request
 
         :param req: the Request object coming from the wsgi layer
         :retval a dict of key/value filters
@@ -254,7 +254,7 @@ class Controller(object):
         """Parse a marker query param into something usable."""
         marker = req.params.get('marker', None)
 
-        if marker and not uuidutils.is_uuid_like(marker):
+        if marker and not utils.is_uuid_like(marker):
             msg = _('Invalid marker format')
             raise exc.HTTPBadRequest(explanation=msg)
 
@@ -328,13 +328,15 @@ class Controller(object):
             msg = _("Access denied to image %(id)s but returning 'not found'")
             LOG.info(msg % {'id': id})
             raise exc.HTTPNotFound()
+        except Exception:
+            LOG.exception(_("Unable to show image %s"), id)
+            raise
 
         return dict(image=make_image_dict(image))
 
     @utils.mutating
     def delete(self, req, id):
-        """
-        Deletes an existing image with the registry.
+        """Deletes an existing image with the registry.
 
         :param req: wsgi Request object
         :param id:  The opaque internal identifier for the image
@@ -361,11 +363,13 @@ class Controller(object):
             msg = _("Image %(id)s not found")
             LOG.info(msg % {'id': id})
             return exc.HTTPNotFound()
+        except Exception:
+            LOG.exception(_("Unable to delete image %s"), id)
+            raise
 
     @utils.mutating
     def create(self, req, body):
-        """
-        Registers a new image with the registry.
+        """Registers a new image with the registry.
 
         :param req: wsgi Request object
         :param body: Dictionary of information about the image
@@ -384,7 +388,7 @@ class Controller(object):
             image_data['owner'] = req.context.owner
 
         image_id = image_data.get('id')
-        if image_id and not uuidutils.is_uuid_like(image_id):
+        if image_id and not utils.is_uuid_like(image_id):
             msg = _("Rejecting image creation request for invalid image "
                     "id '%(bad_id)s'")
             LOG.info(msg % {'bad_id': image_id})
@@ -397,23 +401,26 @@ class Controller(object):
         try:
             image_data = _normalize_image_location_for_db(image_data)
             image_data = self.db_api.image_create(req.context, image_data)
-            msg = _("Successfully created image %(id)s")
-            LOG.info(msg % {'id': image_id})
-            return dict(image=make_image_dict(image_data))
+            image_data = dict(image=make_image_dict(image_data))
+            msg = _("Successfully created image %(id)s") % image_data['image']
+            LOG.info(msg)
+            return image_data
         except exception.Duplicate:
-            msg = (_("Image with identifier %s already exists!") % image_id)
+            msg = _("Image with identifier %s already exists!") % image_id
             LOG.error(msg)
             return exc.HTTPConflict(msg)
         except exception.Invalid as e:
             msg = (_("Failed to add image metadata. "
-                     "Got error: %(e)s") % locals())
+                     "Got error: %(e)s") % {'e': e})
             LOG.error(msg)
             return exc.HTTPBadRequest(msg)
+        except Exception:
+            LOG.exception(_("Unable to create image %s"), image_id)
+            raise
 
     @utils.mutating
     def update(self, req, id, body):
-        """
-        Updates an existing image with the registry.
+        """Updates an existing image with the registry.
 
         :param req: wsgi Request object
         :param body: Dictionary of information about the image
@@ -422,6 +429,7 @@ class Controller(object):
         :retval Returns the updated image information as a mapping,
         """
         image_data = body['image']
+        from_state = body.get('from_state', None)
 
         # Prohibit modification of 'owner'
         if not req.context.is_admin and 'owner' in image_data:
@@ -433,20 +441,25 @@ class Controller(object):
         purge_props = req.headers.get("X-Glance-Registry-Purge-Props", "false")
         try:
             LOG.debug(_("Updating image %(id)s with metadata: "
-                        "%(image_data)r") % locals())
+                        "%(image_data)r"), {'id': id,
+                                            'image_data': image_data})
             image_data = _normalize_image_location_for_db(image_data)
             if purge_props == "true":
-                updated_image = self.db_api.image_update(req.context, id,
-                                                         image_data, True)
+                purge_props = True
             else:
-                updated_image = self.db_api.image_update(req.context, id,
-                                                         image_data)
+                purge_props = False
+
+            updated_image = self.db_api.image_update(req.context, id,
+                                                     image_data,
+                                                     purge_props=purge_props,
+                                                     from_state=from_state)
+
             msg = _("Updating metadata for image %(id)s")
             LOG.info(msg % {'id': id})
             return dict(image=make_image_dict(updated_image))
         except exception.Invalid as e:
             msg = (_("Failed to update image metadata. "
-                     "Got error: %(e)s") % locals())
+                     "Got error: %(e)s") % {'e': e})
             LOG.error(msg)
             return exc.HTTPBadRequest(msg)
         except exception.NotFound:
@@ -467,6 +480,14 @@ class Controller(object):
             raise exc.HTTPNotFound(body='Image not found',
                                    request=req,
                                    content_type='text/plain')
+        except exception.Conflict as e:
+            LOG.info(unicode(e))
+            raise exc.HTTPConflict(body='Image operation conflicts',
+                                   request=req,
+                                   content_type='text/plain')
+        except Exception:
+            LOG.exception(_("Unable to update image %s"), id)
+            raise
 
 
 def _limit_locations(image):
@@ -479,8 +500,7 @@ def _limit_locations(image):
 
 
 def make_image_dict(image):
-    """
-    Create a dict representation of an image which we can use to
+    """Create a dict representation of an image which we can use to
     serialize the image.
     """
 
