@@ -17,17 +17,15 @@
 RPC Controller
 """
 
-from oslo_config import cfg
-from oslo_log import log as logging
+from oslo.config import cfg
 
 from glance.common import rpc
 from glance.common import wsgi
 import glance.db
-from glance import i18n
+import glance.openstack.common.log as logging
 
 
 LOG = logging.getLogger(__name__)
-_ = i18n._
 
 CONF = cfg.CONF
 
