@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2010-2011 OpenStack, LLC
+# Copyright 2010-2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,21 +14,20 @@
 #    under the License.
 
 import collections
-import os
+import copy
 import sys
-import time
 
 from oslo.config import cfg
+import six
 
-from glance.common import crypt
 from glance.common import exception
 from glance.common import utils
 import glance.context
 import glance.domain.proxy
 from glance.openstack.common import importutils
 import glance.openstack.common.log as logging
+from glance import scrubber
 from glance.store import location
-from glance.store import scrubber
 
 LOG = logging.getLogger(__name__)
 
@@ -38,12 +35,7 @@ store_opts = [
     cfg.ListOpt('known_stores',
                 default=[
                     'glance.store.filesystem.Store',
-                    'glance.store.http.Store',
-                    'glance.store.rbd.Store',
-                    'glance.store.s3.Store',
-                    'glance.store.swift.Store',
-                    'glance.store.sheepdog.Store',
-                    'glance.store.cinder.Store',
+                    'glance.store.http.Store'
                 ],
                 help=_('List of which store classes and store class locations '
                        'are currently known to glance at startup.')),
@@ -56,7 +48,7 @@ store_opts = [
                help=_('Directory that the scrubber will use to track '
                       'information about what to delete. '
                       'Make sure this is set in glance-api.conf and '
-                      'glance-scrubber.conf')),
+                      'glance-scrubber.conf.')),
     cfg.BoolOpt('delayed_delete', default=False,
                 help=_('Turn on/off delayed delete.')),
     cfg.BoolOpt('use_user_token', default=True,
@@ -67,9 +59,22 @@ store_opts = [
                       'performing a delete.')),
 ]
 
+REGISTERED_STORES = set()
 CONF = cfg.CONF
 CONF.register_opts(store_opts)
 
+_ALL_STORES = [
+    'glance.store.filesystem.Store',
+    'glance.store.http.Store',
+    'glance.store.rbd.Store',
+    'glance.store.s3.Store',
+    'glance.store.swift.Store',
+    'glance.store.sheepdog.Store',
+    'glance.store.cinder.Store',
+    'glance.store.gridfs.Store',
+    'glance.store.vmware_datastore.Store'
+]
+
 
 class BackendException(Exception):
     pass
@@ -143,6 +148,20 @@ class Indexable(object):
         return self.size
 
 
+def _register_stores(store_classes):
+    """
+    Given a set of store names, add them to a globally available set
+    of store names.
+    """
+    for store_cls in store_classes:
+        REGISTERED_STORES.add(store_cls.__module__.split('.')[2])
+    # NOTE (spredzy): The actual class name is filesystem but in order
+    # to maintain backward compatibility we need to keep the 'file' store
+    # as a known store
+    if 'filesystem' in REGISTERED_STORES:
+        REGISTERED_STORES.add('file')
+
+
 def _get_store_class(store_entry):
     store_cls = None
     try:
@@ -162,12 +181,24 @@ def create_stores():
     """
     store_count = 0
     store_classes = set()
-    for store_entry in CONF.known_stores:
+    for store_entry in set(CONF.known_stores + _ALL_STORES):
         store_entry = store_entry.strip()
         if not store_entry:
             continue
         store_cls = _get_store_class(store_entry)
-        store_instance = store_cls()
+        try:
+            store_instance = store_cls()
+        except exception.BadStoreConfiguration as e:
+            if store_entry in CONF.known_stores:
+                LOG.warn(_("%s Skipping store driver.") % unicode(e))
+            continue
+        finally:
+            # NOTE(flaper87): To be removed in Juno
+            if store_entry not in CONF.known_stores:
+                LOG.deprecated(_("%s not found in `known_store`. "
+                                 "Stores need to be explicitly enabled in "
+                                 "the configuration file.") % store_entry)
+
         schemes = store_instance.get_schemes()
         if not schemes:
             raise BackendException('Unable to register store %s. '
@@ -189,6 +220,7 @@ def create_stores():
                 store_count += 1
             else:
                 LOG.debug("Store %s already registered", store_cls)
+    _register_stores(store_classes)
     return store_count
 
 
@@ -202,6 +234,16 @@ def verify_default_store():
         raise RuntimeError(msg)
 
 
+def get_known_schemes():
+    """Returns list of known schemes"""
+    return location.SCHEME_TO_CLS_MAP.keys()
+
+
+def get_known_stores():
+    """Returns list of known stores"""
+    return list(REGISTERED_STORES)
+
+
 def get_store_from_scheme(context, scheme, loc=None):
     """
     Given a scheme, return the appropriate store object
@@ -278,11 +320,12 @@ def safe_delete_from_backend(context, uri, image_id, **kwargs):
         msg = _('Failed to delete image %s in store from URI')
         LOG.warn(msg % image_id)
     except exception.StoreDeleteNotSupported as e:
-        LOG.warn(str(e))
+        LOG.warn(six.text_type(e))
     except UnsupportedBackend:
         exc_type = sys.exc_info()[0].__name__
-        msg = (_('Failed to delete image %s from store (%s)') %
-               (image_id, exc_type))
+        msg = (_('Failed to delete image %(image_id)s from store '
+                 '(%(error)s)') % {'image_id': image_id,
+                                   'error': exc_type})
         LOG.error(msg)
 
 
@@ -306,19 +349,20 @@ def delete_image_from_backend(context, store_api, image_id, uri):
 
 
 def check_location_metadata(val, key=''):
-    t = type(val)
-    if t == dict:
+    if isinstance(val, dict):
         for key in val:
             check_location_metadata(val[key], key=key)
-    elif t == list:
+    elif isinstance(val, list):
         ndx = 0
         for v in val:
             check_location_metadata(v, key='%s[%d]' % (key, ndx))
             ndx = ndx + 1
-    elif t != unicode:
-        raise BackendException(_("The image metadata key %s has an invalid "
-                                 "type of %s.  Only dict, list, and unicode "
-                                 "are supported.") % (key, str(t)))
+    elif not isinstance(val, unicode):
+        raise BackendException(_("The image metadata key %(key)s has an "
+                                 "invalid type of %(val)s.  Only dict, list, "
+                                 "and unicode are supported.") %
+                               {'key': key,
+                                'val': type(val)})
 
 
 def store_add_to_backend(image_id, data, size, store):
@@ -337,18 +381,21 @@ def store_add_to_backend(image_id, data, size, store):
     """
     (location, size, checksum, metadata) = store.add(image_id, data, size)
     if metadata is not None:
-        if type(metadata) != dict:
-            msg = (_("The storage driver %s returned invalid metadata %s"
-                     "This must be a dictionary type") %
-                   (str(store), str(metadata)))
+        if not isinstance(metadata, dict):
+            msg = (_("The storage driver %(store)s returned invalid metadata "
+                     "%(metadata)s. This must be a dictionary type") %
+                   {'store': six.text_type(store),
+                    'metadata': six.text_type(metadata)})
             LOG.error(msg)
             raise BackendException(msg)
         try:
             check_location_metadata(metadata)
         except BackendException as e:
             e_msg = (_("A bad metadata structure was returned from the "
-                       "%s storage driver: %s.  %s.") %
-                     (str(store), str(metadata), str(e)))
+                       "%(store)s storage driver: %(metadata)s.  %(error)s.") %
+                     {'store': six.text_type(store),
+                      'metadata': six.text_type(metadata),
+                      'error': six.text_type(e)})
             LOG.error(e_msg)
             raise BackendException(e_msg)
     return (location, size, checksum, metadata)
@@ -429,6 +476,17 @@ def _check_image_location(context, store_api, location):
     store_api.check_location_metadata(location['metadata'])
 
 
+def _set_image_size(context, image, locations):
+    if not image.size:
+        for location in locations:
+            size_from_backend = glance.store.get_size_from_backend(
+                context, location['url'])
+            if size_from_backend:
+                # NOTE(flwang): This assumes all locations have the same size
+                image.size = size_from_backend
+                break
+
+
 class ImageFactoryProxy(glance.domain.proxy.ImageFactory):
     def __init__(self, factory, context, store_api):
         self.context = context
@@ -439,8 +497,13 @@ class ImageFactoryProxy(glance.domain.proxy.ImageFactory):
                                                 proxy_kwargs=proxy_kwargs)
 
     def new_image(self, **kwargs):
-        for l in kwargs.get('locations', []):
+        locations = kwargs.get('locations', [])
+        for l in locations:
             _check_image_location(self.context, self.store_api, l)
+
+            if locations.count(l) > 1:
+                raise exception.DuplicateLocation(location=l['url'])
+
         return super(ImageFactoryProxy, self).new_image(**kwargs)
 
 
@@ -459,25 +522,31 @@ class StoreLocations(collections.MutableSequence):
             self.value = list(value)
 
     def append(self, location):
-        _check_image_location(self.image_proxy.context,
-                              self.image_proxy.store_api, location)
-        self.value.append(location)
+        # NOTE(flaper87): Insert this
+        # location at the very end of
+        # the value list.
+        self.insert(len(self.value), location)
 
     def extend(self, other):
         if isinstance(other, StoreLocations):
-            self.value.extend(other.value)
+            locations = other.value
         else:
             locations = list(other)
-            for location in locations:
-                _check_image_location(self.image_proxy.context,
-                                      self.image_proxy.store_api,
-                                      location)
-            self.value.extend(locations)
+
+        for location in locations:
+            self.append(location)
 
     def insert(self, i, location):
         _check_image_location(self.image_proxy.context,
                               self.image_proxy.store_api, location)
+
+        if location in self.value:
+            raise exception.DuplicateLocation(location=location['url'])
+
         self.value.insert(i, location)
+        _set_image_size(self.image_proxy.context,
+                        self.image_proxy,
+                        [location])
 
     def pop(self, i=-1):
         location = self.value.pop(i)
@@ -516,6 +585,9 @@ class StoreLocations(collections.MutableSequence):
         _check_image_location(self.image_proxy.context,
                               self.image_proxy.store_api, location)
         self.value.__setitem__(i, location)
+        _set_image_size(self.image_proxy.context,
+                        self.image_proxy,
+                        [location])
 
     def __delitem__(self, i):
         location = None
@@ -545,15 +617,7 @@ class StoreLocations(collections.MutableSequence):
             self.value.__delitem__(i)
 
     def __iadd__(self, other):
-        if isinstance(other, StoreLocations):
-            self.value += other.value
-        else:
-            locations = list(other)
-            for location in locations:
-                _check_image_location(self.image_proxy.context,
-                                      self.image_proxy.store_api,
-                                      location)
-            self.value += locations
+        self.extend(other)
         return self
 
     def __contains__(self, location):
@@ -574,6 +638,15 @@ class StoreLocations(collections.MutableSequence):
     def __iter__(self):
         return iter(self.value)
 
+    def __copy__(self):
+        return type(self)(self.image_proxy, self.value)
+
+    def __deepcopy__(self, memo):
+        # NOTE(zhiyan): Only copy location entries, others can be reused.
+        value = copy.deepcopy(self.value, memo)
+        self.image_proxy.image.locations = value
+        return type(self)(self.image_proxy, value)
+
 
 def _locations_proxy(target, attr):
     """
@@ -599,6 +672,10 @@ def _locations_proxy(target, attr):
             for location in value:
                 _check_image_location(self.context, self.store_api,
                                       location)
+
+                if value.count(location) > 1:
+                    raise exception.DuplicateLocation(location=location['url'])
+            _set_image_size(self.context, getattr(self, target), value)
             return setattr(getattr(self, target), attr, list(value))
 
     def del_attr(self):
@@ -627,14 +704,12 @@ class ImageProxy(glance.domain.proxy.Image):
             'store_api': store_api,
         }
         super(ImageProxy, self).__init__(
-                image, member_repo_proxy_class=ImageMemberRepoProxy,
-                member_repo_proxy_kwargs=proxy_kwargs)
+            image, member_repo_proxy_class=ImageMemberRepoProxy,
+            member_repo_proxy_kwargs=proxy_kwargs)
 
     def delete(self):
         self.image.delete()
         if self.image.locations:
-            if CONF.delayed_delete:
-                self.image.status = 'pending_delete'
             for location in self.image.locations:
                 self.store_api.delete_image_from_backend(self.context,
                                                          self.store_api,
@@ -645,8 +720,8 @@ class ImageProxy(glance.domain.proxy.Image):
         if size is None:
             size = 0  # NOTE(markwash): zero -> unknown size
         location, size, checksum, loc_meta = self.store_api.add_to_backend(
-                self.context, CONF.default_store,
-                self.image.image_id, utils.CooperativeReader(data), size)
+            self.context, CONF.default_store,
+            self.image.image_id, utils.CooperativeReader(data), size)
         self.image.locations = [{'url': location, 'metadata': loc_meta}]
         self.image.size = size
         self.image.checksum = checksum
@@ -665,7 +740,7 @@ class ImageProxy(glance.domain.proxy.Image):
             except Exception as e:
                 LOG.warn(_('Get image %(id)s data failed: '
                            '%(err)s.') % {'id': self.image.image_id,
-                                          'err': e})
+                                          'err': six.text_type(e)})
                 err = e
         # tried all locations
         LOG.error(_('Glance tried all locations to get data for image %s '
@@ -690,11 +765,9 @@ class ImageMemberRepoProxy(glance.domain.proxy.Repo):
                                         read_tenants=member_ids)
 
     def add(self, member):
-        result = super(ImageMemberRepoProxy, self).add(member)
+        super(ImageMemberRepoProxy, self).add(member)
         self._set_acls()
-        return result
 
     def remove(self, member):
-        result = super(ImageMemberRepoProxy, self).remove(member)
+        super(ImageMemberRepoProxy, self).remove(member)
         self._set_acls()
-        return result
