@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2010 OpenStack, LLC
+# Copyright 2010 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,7 +14,8 @@
 #    under the License.
 
 import httplib
-import urlparse
+
+import six.moves.urllib.parse as urlparse
 
 from glance.common import exception
 import glance.openstack.common.log as logging
@@ -152,6 +151,9 @@ class Store(glance.store.base.Store):
 
     def _query(self, location, verb, depth=0):
         if depth > MAX_REDIRECTS:
+            reason = (_("The HTTP URL exceeded %s maximum "
+                        "redirects.") % MAX_REDIRECTS)
+            LOG.debug(reason)
             raise exception.MaxRedirectsExceeded(redirects=MAX_REDIRECTS)
         loc = location.store_location
         conn_class = self._get_conn_class(loc)
@@ -162,13 +164,15 @@ class Store(glance.store.base.Store):
         # Check for bad status codes
         if resp.status >= 400:
             reason = _("HTTP URL returned a %s status code.") % resp.status
+            LOG.debug(reason)
             raise exception.BadStoreUri(loc.path, reason)
 
         location_header = resp.getheader("location")
         if location_header:
             if resp.status not in (301, 302):
-                reason = _("The HTTP URL attempted to redirect with an "
-                           "invalid status code.")
+                reason = (_("The HTTP URL attempted to redirect with an "
+                            "invalid %s status code.") % resp.status)
+                LOG.debug(reason)
                 raise exception.BadStoreUri(loc.path, reason)
             location_class = glance.store.location.Location
             new_loc = location_class(location.store_name,
