@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2010-2011 OpenStack, LLC
+# Copyright 2010-2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -22,14 +20,13 @@ from __future__ import absolute_import
 import hashlib
 import httplib
 import math
-import sys
-import urllib
-import urlparse
 
 from oslo.config import cfg
+import six.moves.urllib.parse as urlparse
 
 from glance.common import auth
 from glance.common import exception
+from glance.openstack.common import excutils
 import glance.openstack.common.log as logging
 import glance.store
 import glance.store.base
@@ -56,14 +53,17 @@ swift_opts = [
                       'is listening.')),
     cfg.StrOpt('swift_store_user', secret=True,
                help=_('The user to authenticate against the Swift '
-                      'authentication service')),
+                      'authentication service.')),
     cfg.StrOpt('swift_store_key', secret=True,
                help=_('Auth key for the user authenticating against the '
                       'Swift authentication service.')),
     cfg.StrOpt('swift_store_auth_version', default='2',
                help=_('Version of the authentication service to use. '
                       'Valid versions are 2 for keystone and 1 for swauth '
-                      'and rackspace')),
+                      'and rackspace.')),
+    cfg.BoolOpt('swift_store_auth_insecure', default=False,
+                help=_('If True, swiftclient won\'t check for a valid SSL '
+                       'certificate when authenticating.')),
     cfg.StrOpt('swift_store_region',
                help=_('The region of the swift endpoint to be used for '
                       'single tenant. This setting is only necessary if the '
@@ -84,7 +84,7 @@ swift_opts = [
     cfg.IntOpt('swift_store_large_object_size',
                default=DEFAULT_LARGE_OBJECT_SIZE,
                help=_('The size, in MB, that Glance will start chunking image '
-                      'files and do a large object manifest in Swift')),
+                      'files and do a large object manifest in Swift.')),
     cfg.IntOpt('swift_store_large_object_chunk_size',
                default=DEFAULT_LARGE_OBJECT_CHUNK_SIZE,
                help=_('The amount of data written to a temporary disk buffer '
@@ -100,12 +100,55 @@ swift_opts = [
                 help=_('A list of tenants that will be granted read/write '
                        'access on all Swift containers created by Glance in '
                        'multi-tenant mode.')),
+    cfg.BoolOpt('swift_store_ssl_compression', default=True,
+                help=_('If set to False, disables SSL layer compression of '
+                       'https swift requests. Setting to False may improve '
+                       'performance for images which are already in a '
+                       'compressed format, eg qcow2.')),
+    cfg.IntOpt('swift_store_retry_get_count', default=0,
+               help=_('The number of times a Swift download will be retried '
+                      'before the request fails.'))
 ]
 
 CONF = cfg.CONF
 CONF.register_opts(swift_opts)
 
 
+def swift_retry_iter(resp_iter, length, store, location):
+    length = length if length else (resp_iter.len
+                                    if hasattr(resp_iter, 'len') else 0)
+    retries = 0
+    bytes_read = 0
+
+    while retries <= CONF.swift_store_retry_get_count:
+        try:
+            for chunk in resp_iter:
+                yield chunk
+                bytes_read += len(chunk)
+        except swiftclient.ClientException as e:
+            LOG.warn(_("Swift exception raised %s") % e)
+
+        if bytes_read != length:
+            if retries == CONF.swift_store_retry_get_count:
+                # terminate silently and let higher level decide
+                LOG.error(_("Stopping Swift retries after %d "
+                            "attempts") % retries)
+                break
+            else:
+                retries += 1
+                LOG.info(_("Retrying Swift connection "
+                           "(%(retries)d/%(max_retries)d) with "
+                           "range=%(start)d-%(end)d") %
+                         {'retries': retries,
+                          'max_retries': CONF.swift_store_retry_get_count,
+                          'start': bytes_read,
+                          'end': length})
+                (resp_headers, resp_iter) = store._get_object(location, None,
+                                                              bytes_read)
+        else:
+            break
+
+
 class StoreLocation(glance.store.location.StoreLocation):
 
     """
@@ -136,7 +179,8 @@ class StoreLocation(glance.store.location.StoreLocation):
 
     def _get_credstring(self):
         if self.user and self.key:
-            return '%s:%s@' % (urllib.quote(self.user), urllib.quote(self.key))
+            return '%s:%s@' % (urlparse.quote(self.user),
+                               urlparse.quote(self.key))
         return ''
 
     def get_uri(self):
@@ -173,7 +217,7 @@ class StoreLocation(glance.store.location.StoreLocation):
                        ", you need to change it to use the "
                        "swift+http:// scheme, like so: "
                        "swift+http://user:pass@authurl.com/v1/container/obj")
-            LOG.debug(_("Invalid store URI: %(reason)s") % locals())
+            LOG.debug(_("Invalid store URI: %(reason)s"), {'reason': reason})
             raise exception.BadStoreUri(message=reason)
 
         pieces = urlparse.urlparse(uri)
@@ -203,8 +247,8 @@ class StoreLocation(glance.store.location.StoreLocation):
                 LOG.debug(reason)
                 raise exception.BadStoreUri()
             user, key = cred_parts
-            self.user = urllib.unquote(user)
-            self.key = urllib.unquote(key)
+            self.user = urlparse.unquote(user)
+            self.key = urlparse.unquote(key)
         else:
             self.user = None
             self.key = None
@@ -264,24 +308,35 @@ class BaseStore(glance.store.base.Store):
         self.service_type = CONF.swift_store_service_type
         self.endpoint_type = CONF.swift_store_endpoint_type
         self.snet = CONF.swift_enable_snet
+        self.insecure = CONF.swift_store_auth_insecure
+        self.ssl_compression = CONF.swift_store_ssl_compression
 
-    def get(self, location, connection=None):
-        location = location.store_location
+    def _get_object(self, location, connection=None, start=None):
         if not connection:
             connection = self.get_connection(location)
+        headers = {}
+        if start is not None:
+            bytes_range = 'bytes=%d-' % start
+            headers = {'Range': bytes_range}
 
         try:
             resp_headers, resp_body = connection.get_object(
-                    container=location.container, obj=location.obj,
-                    resp_chunk_size=self.CHUNKSIZE)
-        except swiftclient.ClientException, e:
+                container=location.container, obj=location.obj,
+                resp_chunk_size=self.CHUNKSIZE, headers=headers)
+        except swiftclient.ClientException as e:
             if e.http_status == httplib.NOT_FOUND:
-                uri = location.get_uri()
-                msg = _("Swift could not find image at URI.")
+                msg = _("Swift could not find object %s.") % location.obj
+                LOG.warn(msg)
                 raise exception.NotFound(msg)
             else:
                 raise
 
+        return (resp_headers, resp_body)
+
+    def get(self, location, connection=None):
+        location = location.store_location
+        (resp_headers, resp_body) = self._get_object(location, connection)
+
         class ResponseIndexable(glance.store.Indexable):
             def another(self):
                 try:
@@ -290,6 +345,8 @@ class BaseStore(glance.store.base.Store):
                     return ''
 
         length = int(resp_headers.get('content-length', 0))
+        if CONF.swift_store_retry_get_count > 0:
+            resp_body = swift_retry_iter(resp_body, length, self, location)
         return (ResponseIndexable(resp_body, length), length)
 
     def get_size(self, location, connection=None):
@@ -298,7 +355,7 @@ class BaseStore(glance.store.base.Store):
             connection = self.get_connection(location)
         try:
             resp_headers = connection.head_object(
-                    container=location.container, obj=location.obj)
+                container=location.container, obj=location.obj)
             return int(resp_headers.get('content-length', 0))
         except Exception:
             return 0
@@ -307,12 +364,23 @@ class BaseStore(glance.store.base.Store):
         result = getattr(CONF, param)
         if not result:
             reason = (_("Could not find %(param)s in configuration "
-                        "options.") % locals())
+                        "options.") % {'param': param})
             LOG.error(reason)
             raise exception.BadStoreConfiguration(store_name="swift",
                                                   reason=reason)
         return result
 
+    def _delete_stale_chunks(self, connection, container, chunk_list):
+        for chunk in chunk_list:
+            LOG.debug(_("Deleting chunk %s") % chunk)
+            try:
+                connection.delete_object(container, chunk)
+            except Exception:
+                msg = _("Failed to delete orphaned chunk "
+                        "%(container)s/%(chunk)s")
+                LOG.exception(msg % {'container': container,
+                                     'chunk': chunk})
+
     def add(self, image_id, image_file, image_size, connection=None):
         location = self.create_location(image_id)
         if not connection:
@@ -367,27 +435,25 @@ class BaseStore(glance.store.base.Store):
                             content_length=content_length)
                         written_chunks.append(chunk_name)
                     except Exception:
-                        # Save original traceback
-                        exc_type, exc_val, exc_tb = sys.exc_info()
+                        # Delete orphaned segments from swift backend
+                        with excutils.save_and_reraise_exception():
+                            LOG.exception(_("Error during chunked upload to "
+                                            "backend, deleting stale chunks"))
+                            self._delete_stale_chunks(connection,
+                                                      location.container,
+                                                      written_chunks)
 
-                        # Delete now stale segments from swift backend
-                        for chunk in written_chunks:
-                            LOG.debug(_("Deleting chunk %s" % chunk))
-                            try:
-                                connection.delete_object(location.container,
-                                                         chunk)
-                            except Exception:
-                                msg = _("Failed to delete orphan chunk %s/%s")
-                                LOG.exception(msg, location.container, chunk)
-
-                        # reraise original exception with traceback intact
-                        raise exc_type, exc_val, exc_tb
                     bytes_read = reader.bytes_read
-                    msg = _("Wrote chunk %(chunk_name)s (%(chunk_id)d/"
-                            "%(total_chunks)s) of length %(bytes_read)d "
-                            "to Swift returning MD5 of content: "
-                            "%(chunk_etag)s")
-                    LOG.debug(msg % locals())
+                    msg = (_("Wrote chunk %(chunk_name)s (%(chunk_id)d/"
+                             "%(total_chunks)s) of length %(bytes_read)d "
+                             "to Swift returning MD5 of content: "
+                             "%(chunk_etag)s") %
+                           {'chunk_name': chunk_name,
+                            'chunk_id': chunk_id,
+                            'total_chunks': total_chunks,
+                            'bytes_read': bytes_read,
+                            'chunk_etag': chunk_etag})
+                    LOG.debug(msg)
 
                     if bytes_read == 0:
                         # Delete the last chunk, because it's of zero size.
@@ -426,13 +492,13 @@ class BaseStore(glance.store.base.Store):
             # the location attribute from GET /images/<ID> and
             # GET /images/details
 
-            return (location.get_uri(), image_size, obj_etag)
-        except swiftclient.ClientException, e:
+            return (location.get_uri(), image_size, obj_etag, {})
+        except swiftclient.ClientException as e:
             if e.http_status == httplib.CONFLICT:
                 raise exception.Duplicate(_("Swift already has an image at "
                                             "this location"))
             msg = (_("Failed to add object to Swift.\n"
-                     "Got error from Swift: %(e)s") % locals())
+                     "Got error from Swift: %(e)s") % {'e': e})
             LOG.error(msg)
             raise glance.store.BackendException(msg)
 
@@ -449,16 +515,16 @@ class BaseStore(glance.store.base.Store):
             manifest = None
             try:
                 headers = connection.head_object(
-                        location.container, location.obj)
+                    location.container, location.obj)
                 manifest = headers.get('x-object-manifest')
-            except swiftclient.ClientException, e:
+            except swiftclient.ClientException as e:
                 if e.http_status != httplib.NOT_FOUND:
                     raise
             if manifest:
                 # Delete all the chunks before the object manifest itself
                 obj_container, obj_prefix = manifest.split('/', 1)
                 segments = connection.get_container(
-                        obj_container, prefix=obj_prefix)[1]
+                    obj_container, prefix=obj_prefix)[1]
                 for segment in segments:
                     # TODO(jaypipes): This would be an easy area to parallelize
                     # since we're simply sending off parallelizable requests
@@ -470,9 +536,8 @@ class BaseStore(glance.store.base.Store):
             # Delete object (or, in segmented case, the manifest)
             connection.delete_object(location.container, location.obj)
 
-        except swiftclient.ClientException, e:
+        except swiftclient.ClientException as e:
             if e.http_status == httplib.NOT_FOUND:
-                uri = location.get_uri()
                 msg = _("Swift could not find image at URI.")
                 raise exception.NotFound(msg)
             else:
@@ -488,21 +553,21 @@ class BaseStore(glance.store.base.Store):
         """
         try:
             connection.head_container(container)
-        except swiftclient.ClientException, e:
+        except swiftclient.ClientException as e:
             if e.http_status == httplib.NOT_FOUND:
                 if CONF.swift_store_create_container_on_put:
                     try:
                         connection.put_container(container)
-                    except swiftclient.ClientException, e:
-                        msg = _("Failed to add container to Swift.\n"
-                                "Got error from Swift: %(e)s") % locals()
+                    except swiftclient.ClientException as e:
+                        msg = (_("Failed to add container to Swift.\n"
+                                 "Got error from Swift: %(e)s") % {'e': e})
                         raise glance.store.BackendException(msg)
                 else:
                     msg = (_("The container %(container)s does not exist in "
                              "Swift. Please set the "
                              "swift_store_create_container_on_put option"
                              "to add container to Swift automatically.") %
-                           locals())
+                           {'container': container})
                     raise glance.store.BackendException(msg)
             else:
                 raise
@@ -569,9 +634,10 @@ class SingleTenantStore(BaseStore):
         os_options['service_type'] = self.service_type
 
         return swiftclient.Connection(
-                auth_url, user, location.key,
-                tenant_name=tenant_name, snet=self.snet,
-                auth_version=self.auth_version, os_options=os_options)
+            auth_url, user, location.key, insecure=self.insecure,
+            tenant_name=tenant_name, snet=self.snet,
+            auth_version=self.auth_version, os_options=os_options,
+            ssl_compression=self.ssl_compression)
 
 
 class MultiTenantStore(BaseStore):
@@ -589,8 +655,8 @@ class MultiTenantStore(BaseStore):
             raise exception.BadStoreConfiguration(store_name="swift",
                                                   reason=reason)
         self.storage_url = auth.get_endpoint(
-                self.context.service_catalog, service_type=self.service_type,
-                endpoint_region=self.region, endpoint_type=self.endpoint_type)
+            self.context.service_catalog, service_type=self.service_type,
+            endpoint_region=self.region, endpoint_type=self.endpoint_type)
         if self.storage_url.startswith('http://'):
             self.scheme = 'swift+http'
         else:
@@ -617,21 +683,22 @@ class MultiTenantStore(BaseStore):
         if public:
             headers['X-Container-Read'] = ".r:*,.rlistings"
         elif read_tenants:
-            headers['X-Container-Read'] = ','.join(read_tenants)
+            headers['X-Container-Read'] = ','.join('%s:*' % i
+                                                   for i in read_tenants)
         else:
             headers['X-Container-Read'] = ''
 
         write_tenants.extend(self.admin_tenants)
         if write_tenants:
-            headers['X-Container-Write'] = ','.join(write_tenants)
+            headers['X-Container-Write'] = ','.join('%s:*' % i
+                                                    for i in write_tenants)
         else:
             headers['X-Container-Write'] = ''
 
         try:
             connection.post_container(location.container, headers=headers)
-        except swiftclient.ClientException, e:
+        except swiftclient.ClientException as e:
             if e.http_status == httplib.NOT_FOUND:
-                uri = location.get_uri()
                 msg = _("Swift could not find image at URI.")
                 raise exception.NotFound(msg)
             else:
@@ -646,11 +713,12 @@ class MultiTenantStore(BaseStore):
 
     def get_connection(self, location):
         return swiftclient.Connection(
-                None, self.context.user, None,
-                preauthurl=location.swift_url,
-                preauthtoken=self.context.auth_tok,
-                tenant_name=self.context.tenant,
-                auth_version='2', snet=self.snet)
+            None, self.context.user, None,
+            preauthurl=location.swift_url,
+            preauthtoken=self.context.auth_tok,
+            tenant_name=self.context.tenant,
+            auth_version='2', snet=self.snet, insecure=self.insecure,
+            ssl_compression=self.ssl_compression)
 
 
 class ChunkReader(object):
