@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack, LLC
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -25,7 +23,6 @@ and spinning down the servers.
 
 import atexit
 import datetime
-import json
 import logging
 import os
 import re
@@ -35,14 +32,16 @@ import socket
 import sys
 import tempfile
 import time
-import urlparse
 
 import fixtures
-from sqlalchemy import create_engine
+import six.moves.urllib.parse as urlparse
 import testtools
 
-from glance import tests as glance_tests
 from glance.common import utils
+from glance.db.sqlalchemy import api as db_api
+from glance.openstack.common import jsonutils
+from glance.openstack.common import units
+from glance import tests as glance_tests
 from glance.tests import utils as test_utils
 
 execute, get_unused_port = test_utils.execute, test_utils.get_unused_port
@@ -76,6 +75,8 @@ class Server(object):
         self.property_protection_file = ''
         self.enable_v1_api = True
         self.enable_v2_api = True
+        self.enable_v1_registry = True
+        self.enable_v2_registry = True
         self.needs_database = False
         self.log_file = None
         self.sock = sock
@@ -215,7 +216,7 @@ class Server(object):
                 os.system('cp %s %s/tests.sqlite'
                           % (db_location, self.test_dir))
             else:
-                cmd = ('%s -m glance.cmd.manage --config-file %s db_sync' %
+                cmd = ('%s -m glance.cmd.manage --config-file %s db sync' %
                        (sys.executable, conf_filepath))
                 execute(cmd, no_venv=self.no_venv, exec_env=self.exec_env,
                         expect_exit=True)
@@ -293,7 +294,7 @@ class ApiServer(Server):
         self.swift_store_container = kwargs.get("swift_store_container", "")
         self.swift_store_create_container_on_put = kwargs.get(
             "swift_store_create_container_on_put", "True")
-        self.swift_store_large_object_size = 5 * 1024
+        self.swift_store_large_object_size = 5 * units.Ki
         self.swift_store_large_object_chunk_size = 200
         self.swift_store_multi_tenant = False
         self.swift_store_admin_tenants = []
@@ -310,6 +311,11 @@ class ApiServer(Server):
         self.image_cache_driver = 'sqlite'
         self.policy_file = policy_file
         self.policy_default_rule = 'default'
+        self.property_protection_rule_format = 'roles'
+        self.image_member_quota = 10
+        self.image_property_quota = 10
+        self.image_tag_quota = 10
+        self.image_location_quota = 2
 
         self.needs_database = True
         default_sql_connection = 'sqlite:////%s/tests.sqlite' % self.test_dir
@@ -318,6 +324,9 @@ class ApiServer(Server):
         self.user_storage_quota = 0
         self.lock_path = self.test_dir
 
+        self.location_strategy = 'location_order'
+        self.store_type_location_strategy_preference = ""
+
         self.conf_base = """[DEFAULT]
 verbose = %(verbose)s
 debug = %(debug)s
@@ -370,8 +379,16 @@ enable_v2_api = %(enable_v2_api)s
 lock_path = %(lock_path)s
 enable_v2_api= %(enable_v2_api)s
 property_protection_file = %(property_protection_file)s
+property_protection_rule_format = %(property_protection_rule_format)s
+image_member_quota=%(image_member_quota)s
+image_property_quota=%(image_property_quota)s
+image_tag_quota=%(image_tag_quota)s
+image_location_quota=%(image_location_quota)s
+location_strategy=%(location_strategy)s
 [paste_deploy]
 flavor = %(deployment_flavor)s
+[store_type_location_strategy]
+store_type_preference = %(store_type_location_strategy_preference)s
 """
         self.paste_conf_base = """[pipeline:glance-api]
 pipeline = versionnegotiation gzip unauthenticated-context rootapp
@@ -470,6 +487,7 @@ sql_idle_timeout = 3600
 api_limit_max = 1000
 limit_param_default = 25
 owner_is_tenant = %(owner_is_tenant)s
+enable_v2_registry = %(enable_v2_registry)s
 workers = %(workers)s
 user_storage_quota = %(user_storage_quota)s
 [paste_deploy]
@@ -482,7 +500,7 @@ pipeline = unauthenticated-context registryapp
 pipeline = fakeauth context registryapp
 
 [app:registryapp]
-paste.app_factory = glance.registry.api.v%(api_version)s:API.factory
+paste.app_factory = glance.registry.api:API.factory
 
 [filter:context]
 paste.filter_factory = glance.api.middleware.context:ContextMiddleware.factory
@@ -575,8 +593,12 @@ class FunctionalTest(test_utils.BaseTestCase):
         self.copy_data_file('schema-image.json', conf_dir)
         self.copy_data_file('policy.json', conf_dir)
         self.copy_data_file('property-protections.conf', conf_dir)
-        self.property_file = os.path.join(conf_dir,
-                                          'property-protections.conf')
+        self.copy_data_file('property-protections-policies.conf', conf_dir)
+        self.property_file_roles = os.path.join(conf_dir,
+                                                'property-protections.conf')
+        property_policies = 'property-protections-policies.conf'
+        self.property_file_policies = os.path.join(conf_dir,
+                                                   property_policies)
         self.policy_file = os.path.join(conf_dir, 'policy.json')
 
         self.api_server = ApiServer(self.test_dir,
@@ -612,7 +634,7 @@ class FunctionalTest(test_utils.BaseTestCase):
 
     def set_policy_rules(self, rules):
         fap = open(self.policy_file, 'w')
-        fap.write(json.dumps(rules))
+        fap.write(jsonutils.dumps(rules))
         fap.close()
 
     def _reset_database(self, conn_string):
@@ -637,9 +659,10 @@ class FunctionalTest(test_utils.BaseTestCase):
                 if auth_pieces[1].strip():
                     password = "-p%s" % auth_pieces[1]
             sql = ("drop database if exists %(database)s; "
-                   "create database %(database)s;") % locals()
+                   "create database %(database)s;") % {'database': database}
             cmd = ("mysql -u%(user)s %(password)s -h%(host)s "
-                   "-e\"%(sql)s\"") % locals()
+                   "-e\"%(sql)s\"") % {'user': user, 'password': password,
+                                       'host': host, 'sql': sql}
             exitcode, out, err = execute(cmd)
             self.assertEqual(0, exitcode)
 
@@ -718,7 +741,7 @@ class FunctionalTest(test_utils.BaseTestCase):
                             to exit in a timely fashion
         """
         launch_msg = None
-        for i in range(0, max_retries):
+        for i in range(max_retries):
             exitcode, out, err = server.start(expect_exit=not expect_launch,
                                               **kwargs)
             name = server.server_name
@@ -895,8 +918,7 @@ class FunctionalTest(test_utils.BaseTestCase):
         DB verification within the functional tests.
         The raw result set is returned.
         """
-        engine = create_engine(self.registry_server.sql_connection,
-                               pool_recycle=30)
+        engine = db_api.get_engine()
         return engine.execute(sql)
 
     def copy_data_file(self, file_name, dst_dir):
