@@ -13,10 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_serialization import jsonutils
 import requests
-from six.moves import http_client as http
 
+from glance.openstack.common import jsonutils
 from glance.tests import functional
 
 
@@ -43,7 +42,7 @@ class TestRegistryURLVisibility(functional.FunctionalTest):
         path = self._url('/rpc')
         response = requests.post(path, headers=self._headers(),
                                  data=self.req_body)
-        self.assertEqual(http.NOT_FOUND, response.status_code)
+        self.assertEqual(404, response.status_code)
         self.stop_servers()
 
     def test_v2_enabled(self):
@@ -52,5 +51,5 @@ class TestRegistryURLVisibility(functional.FunctionalTest):
         path = self._url('/rpc')
         response = requests.post(path, headers=self._headers(),
                                  data=self.req_body)
-        self.assertEqual(http.OK, response.status_code)
+        self.assertEqual(200, response.status_code)
         self.stop_servers()
