@@ -1,4 +1,5 @@
 # Copyright 2012 OpenStack Foundation
+# Copyright 2013 IBM Corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,7 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from glance.api import CONF
+from glance.common import exception
 import glance.db.sqlalchemy.api
 from glance.db.sqlalchemy import models as db_models
 import glance.tests.functional.db as db_tests
@@ -21,17 +22,15 @@ from glance.tests.functional.db import base
 
 
 def get_db(config):
-    config(sql_connection='sqlite://', verbose=False, debug=False)
-    CONF.set_override('data_api', 'glance.db.sqlalchemy.api')
-    db_api = glance.db.get_api()
-    db_api.setup_db_env()
-    db_api.get_engine()
+    config(connection='sqlite://', group='database')
+    config(verbose=False, debug=False)
+    db_api = glance.db.sqlalchemy.api
     return db_api
 
 
 def reset_db(db_api):
-    db_models.unregister_models(db_api._ENGINE)
-    db_models.register_models(db_api._ENGINE)
+    db_models.unregister_models(db_api.get_engine())
+    db_models.register_models(db_api.get_engine())
 
 
 class TestSqlAlchemyDriver(base.TestDriver, base.DriverTests):
@@ -41,6 +40,27 @@ class TestSqlAlchemyDriver(base.TestDriver, base.DriverTests):
         super(TestSqlAlchemyDriver, self).setUp()
         self.addCleanup(db_tests.reset)
 
+    def test_get_image_with_invalid_long_image_id(self):
+        image_id = '343f9ba5-0197-41be-9543-16bbb32e12aa-xxxxxx'
+        self.assertRaises(exception.NotFound, self.db_api._image_get,
+                          self.context, image_id)
+
+    def test_image_tag_delete_with_invalid_long_image_id(self):
+        image_id = '343f9ba5-0197-41be-9543-16bbb32e12aa-xxxxxx'
+        self.assertRaises(exception.NotFound, self.db_api.image_tag_delete,
+                          self.context, image_id, 'fake')
+
+    def test_image_tag_get_all_with_invalid_long_image_id(self):
+        image_id = '343f9ba5-0197-41be-9543-16bbb32e12aa-xxxxxx'
+        self.assertRaises(exception.NotFound, self.db_api.image_tag_get_all,
+                          self.context, image_id)
+
+    def test_user_get_storage_usage_with_invalid_long_image_id(self):
+        image_id = '343f9ba5-0197-41be-9543-16bbb32e12aa-xxxxxx'
+        self.assertRaises(exception.NotFound,
+                          self.db_api.user_get_storage_usage,
+                          self.context, 'fake_owner_id', image_id)
+
 
 class TestSqlAlchemyVisibility(base.TestVisibility, base.VisibilityTests):
 
@@ -60,7 +80,8 @@ class TestSqlAlchemyMembershipVisibility(base.TestMembershipVisibility,
 
 
 class TestSqlAlchemyDBDataIntegrity(base.TestDriver):
-    """ Test class for checking the data integrity in the database.
+    """Test class for checking the data integrity in the database.
+
     Helpful in testing scenarios specific to the sqlalchemy api.
     """
 
@@ -74,28 +95,34 @@ class TestSqlAlchemyDBDataIntegrity(base.TestDriver):
 
         def fake_paginate_query(query, model, limit,
                                 sort_keys, marker, sort_dir):
-            self.assertEquals(sort_keys, ['created_at', 'id'])
+            self.assertEqual(sort_keys, ['created_at', 'id'])
             return original_method(query, model, limit,
                                    sort_keys, marker, sort_dir)
 
         self.stubs.Set(self.db_api, '_paginate_query',
                        fake_paginate_query)
-        images = self.db_api.image_get_all(self.context,
-                                           sort_key='created_at')
+        self.db_api.image_get_all(self.context, sort_key='created_at')
 
     def test_paginate_non_redundant_sort_keys(self):
         original_method = self.db_api._paginate_query
 
         def fake_paginate_query(query, model, limit,
                                 sort_keys, marker, sort_dir):
-            self.assertEquals(sort_keys, ['name', 'created_at', 'id'])
+            self.assertEqual(sort_keys, ['name', 'created_at', 'id'])
             return original_method(query, model, limit,
                                    sort_keys, marker, sort_dir)
 
         self.stubs.Set(self.db_api, '_paginate_query',
                        fake_paginate_query)
-        images = self.db_api.image_get_all(self.context,
-                                           sort_key='name')
+        self.db_api.image_get_all(self.context, sort_key='name')
+
+
+class TestSqlAlchemyTask(base.TaskTests):
+
+    def setUp(self):
+        db_tests.load(get_db, reset_db)
+        super(TestSqlAlchemyTask, self).setUp()
+        self.addCleanup(db_tests.reset)
 
 
 class TestSqlAlchemyQuota(base.DriverQuotaTests):
