@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack, LLC
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -23,24 +21,26 @@ to the backend store.
 """
 
 import hashlib
-import json
 import os
 import shutil
 import sys
 import time
 
 import httplib2
+from six.moves import xrange
 
+from glance.openstack.common import jsonutils
+from glance.openstack.common import units
 from glance.tests import functional
-from glance.tests.utils import (skip_if_disabled,
-                                execute,
-                                xattr_writes_supported,
-                                minimal_headers)
+from glance.tests.utils import execute
+from glance.tests.utils import minimal_headers
+from glance.tests.utils import skip_if_disabled
+from glance.tests.utils import xattr_writes_supported
 
-from glance.tests.functional.store_utils import (setup_http,
-                                                 get_http_uri)
+from glance.tests.functional.store_utils import get_http_uri
+from glance.tests.functional.store_utils import setup_http
 
-FIVE_KB = 5 * 1024
+FIVE_KB = 5 * units.Ki
 
 
 class BaseCacheMiddlewareTest(object):
@@ -62,7 +62,7 @@ class BaseCacheMiddlewareTest(object):
         response, content = http.request(path, 'POST', headers=headers,
                                          body=image_data)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['checksum'],
                          hashlib.md5(image_data).hexdigest())
         self.assertEqual(data['image']['size'], FIVE_KB)
@@ -135,9 +135,9 @@ class BaseCacheMiddlewareTest(object):
         }
         response, content = http.request(path, 'POST',
                                          headers=headers,
-                                         body=json.dumps(image_entity))
+                                         body=jsonutils.dumps(image_entity))
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         image_id = data['id']
 
         path = "http://%s:%d/v2/images/%s/file" % ("0.0.0.0", self.api_port,
@@ -196,7 +196,7 @@ class BaseCacheMiddlewareTest(object):
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['size'], FIVE_KB)
 
         image_id = data['image']['id']
@@ -234,7 +234,7 @@ class BaseCacheMiddlewareTest(object):
         response, content = http.request(path, 'POST', headers=headers,
                                          body=image_data)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['checksum'],
                          hashlib.md5(image_data).hexdigest())
         self.assertEqual(data['image']['size'], FIVE_KB)
@@ -292,9 +292,9 @@ class BaseCacheMiddlewareTest(object):
         }
         response, content = http.request(path, 'POST',
                                          headers=headers,
-                                         body=json.dumps(image_entity))
+                                         body=jsonutils.dumps(image_entity))
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         image_id = data['id']
 
         path = "http://%s:%d/v2/images/%s/file" % ("0.0.0.0", self.api_port,
@@ -342,7 +342,7 @@ class BaseCacheManageMiddlewareTest(object):
         http = httplib2.Http()
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertTrue('images' in data)
         self.assertEqual(0, len(data['images']))
 
@@ -359,7 +359,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'POST', headers=headers,
                                          body=image_data)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['checksum'],
                          hashlib.md5(image_data).hexdigest())
         self.assertEqual(data['image']['size'], FIVE_KB)
@@ -376,7 +376,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertTrue('cached_images' in data)
         self.assertEqual(data['cached_images'], [])
 
@@ -406,7 +406,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertTrue('cached_images' in data)
 
         cached_images = data['cached_images']
@@ -474,7 +474,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertTrue('cached_images' in data)
 
         # Verify the last_modified/last_accessed values are valid floats
@@ -506,7 +506,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertTrue('cached_images' in data)
 
         cached_images = data['cached_images']
@@ -529,7 +529,7 @@ class BaseCacheManageMiddlewareTest(object):
         ids = {}
 
         # Add a bunch of images...
-        for x in xrange(0, 4):
+        for x in xrange(4):
             ids[x] = self.add_image("Image%s" % str(x))
 
         # Verify no images in cached_images because no image has been hit
@@ -537,7 +537,7 @@ class BaseCacheManageMiddlewareTest(object):
         self.verify_no_cached_images()
 
         # Grab the images, essentially caching them...
-        for x in xrange(0, 4):
+        for x in xrange(4):
             path = "http://%s:%d/v1/images/%s" % ("127.0.0.1", self.api_port,
                                                   ids[x])
             http = httplib2.Http()
@@ -551,7 +551,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertTrue('cached_images' in data)
 
         cached_images = data['cached_images']
@@ -573,7 +573,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertTrue('cached_images' in data)
 
         cached_images = data['cached_images']
@@ -591,7 +591,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertTrue('cached_images' in data)
 
         cached_images = data['cached_images']
@@ -613,7 +613,7 @@ class BaseCacheManageMiddlewareTest(object):
         NUM_IMAGES = 4
 
         # Add and then queue some images
-        for x in xrange(0, NUM_IMAGES):
+        for x in xrange(NUM_IMAGES):
             ids[x] = self.add_image("Image%s" % str(x))
             path = "http://%s:%d/v1/queued_images/%s" % ("127.0.0.1",
                                                          self.api_port, ids[x])
@@ -627,7 +627,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'DELETE')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         num_deleted = data['num_deleted']
         self.assertEqual(NUM_IMAGES, num_deleted)
 
@@ -637,7 +637,7 @@ class BaseCacheManageMiddlewareTest(object):
         response, content = http.request(path, 'DELETE')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         num_deleted = data['num_deleted']
         self.assertEqual(0, num_deleted)
 
@@ -677,7 +677,7 @@ log_file = %(log_file)s
         ids = {}
 
         # Add a bunch of images...
-        for x in xrange(0, 4):
+        for x in xrange(4):
             ids[x] = self.add_image("Image%s" % str(x))
 
         # Queue the first image, verify no images still in cache after queueing
@@ -705,7 +705,7 @@ log_file = %(log_file)s
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
 
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertTrue('cached_images' in data)
 
         cached_images = data['cached_images']
@@ -732,7 +732,7 @@ class TestImageCacheXattr(functional.FunctionalTest,
 
         if not getattr(self, 'inited', False):
             try:
-                import xattr
+                import xattr  # noqa
             except ImportError:
                 self.inited = True
                 self.disabled = True
@@ -778,7 +778,7 @@ class TestImageCacheManageXattr(functional.FunctionalTest,
 
         if not getattr(self, 'inited', False):
             try:
-                import xattr
+                import xattr  # noqa
             except ImportError:
                 self.inited = True
                 self.disabled = True
@@ -824,7 +824,7 @@ class TestImageCacheSqlite(functional.FunctionalTest,
 
         if not getattr(self, 'inited', False):
             try:
-                import sqlite3
+                import sqlite3  # noqa
             except ImportError:
                 self.inited = True
                 self.disabled = True
@@ -863,7 +863,7 @@ class TestImageCacheManageSqlite(functional.FunctionalTest,
 
         if not getattr(self, 'inited', False):
             try:
-                import sqlite3
+                import sqlite3  # noqa
             except ImportError:
                 self.inited = True
                 self.disabled = True
