@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011-2012 OpenStack, LLC
+# Copyright 2011-2012 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,15 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
 import os
 import sys
 import time
 
 import httplib2
+from six.moves import xrange
 import swiftclient
 
 from glance.common import crypt
+from glance.openstack.common import jsonutils
+from glance.openstack.common import units
 from glance.store.swift import StoreLocation
 from glance.tests import functional
 from glance.tests.functional.store.test_swift import parse_config
@@ -32,7 +32,7 @@ from glance.tests.functional.store.test_swift import swift_connect
 from glance.tests.utils import execute
 
 
-TEST_IMAGE_DATA = '*' * 5 * 1024
+TEST_IMAGE_DATA = '*' * 5 * units.Ki
 TEST_IMAGE_META = {
     'name': 'test_image',
     'is_public': False,
@@ -47,7 +47,7 @@ class TestScrubber(functional.FunctionalTest):
 
     def test_delayed_delete(self):
         """
-        test that images don't get deleted immediatly and that the scrubber
+        test that images don't get deleted immediately and that the scrubber
         scrubs them
         """
         self.cleanup()
@@ -65,7 +65,7 @@ class TestScrubber(functional.FunctionalTest):
         response, content = http.request(path, 'POST', body='XXX',
                                          headers=headers)
         self.assertEqual(response.status, 201)
-        image = json.loads(content)['image']
+        image = jsonutils.loads(content)['image']
         self.assertEqual('active', image['status'])
         image_id = image['id']
 
@@ -103,7 +103,7 @@ class TestScrubber(functional.FunctionalTest):
         response, content = http.request(path, 'POST', body='XXX',
                                          headers=headers)
         self.assertEqual(response.status, 201)
-        image = json.loads(content)['image']
+        image = jsonutils.loads(content)['image']
         self.assertEqual('active', image['status'])
         image_id = image['id']
 
@@ -162,7 +162,7 @@ class TestScrubber(functional.FunctionalTest):
                                          headers=headers)
         # ensure the request was successful and the image is active
         self.assertEqual(response.status, 201)
-        image = json.loads(content)['image']
+        image = jsonutils.loads(content)['image']
         self.assertEqual('active', image['status'])
         image_id = image['id']
 
@@ -223,7 +223,7 @@ class TestScrubber(functional.FunctionalTest):
         response, content = http.request(path, 'POST', body='XXX',
                                          headers=headers)
         self.assertEqual(response.status, 201)
-        image = json.loads(content)['image']
+        image = jsonutils.loads(content)['image']
         self.assertEqual('active', image['status'])
         image_id = image['id']
 
@@ -248,7 +248,7 @@ class TestScrubber(functional.FunctionalTest):
         self.assertTrue(marker_uri is not None)
 
         decrypted_uri = crypt.urlsafe_decrypt(
-                self.api_server.metadata_encryption_key, marker_uri)
+            self.api_server.metadata_encryption_key, marker_uri)
         loc = StoreLocation({})
         loc.parse_uri(decrypted_uri)
 
@@ -289,7 +289,7 @@ class TestScrubber(functional.FunctionalTest):
         response, content = http.request(path, 'POST', body='XXX',
                                          headers=headers)
         self.assertEqual(response.status, 201)
-        image = json.loads(content)['image']
+        image = jsonutils.loads(content)['image']
         self.assertEqual('active', image['status'])
         image_id = image['id']
 
@@ -315,7 +315,7 @@ class TestScrubber(functional.FunctionalTest):
             swift.head_object(swift_config['swift_store_container'], image_id)
             self.fail('image should have been deleted from swift')
         except swiftclient.ClientException as e:
-            self.assertEquals(e.http_status, 404)
+            self.assertEqual(e.http_status, 404)
 
         # wait for the scrub time on the image to pass
         time.sleep(self.api_server.scrub_time)
@@ -359,7 +359,7 @@ class TestScrubber(functional.FunctionalTest):
         response, content = http.request(path, 'POST', body='XXX',
                                          headers=headers)
         self.assertEqual(response.status, 201)
-        image = json.loads(content)['image']
+        image = jsonutils.loads(content)['image']
         self.assertEqual('active', image['status'])
         image_id = image['id']
 
@@ -403,7 +403,7 @@ class TestScrubber(functional.FunctionalTest):
         """
         NOTE(jkoelker) The build servers sometimes take longer than 15 seconds
         to scrub. Give it up to 5 min, checking checking every 15 seconds.
-        When/if it flips to deleted, bail immediatly.
+        When/if it flips to deleted, bail immediately.
         """
         http = httplib2.Http()
         wait_for = 300    # seconds
