@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 Red Hat, Inc
 # All Rights Reserved.
 #
@@ -19,7 +17,9 @@
 
 
 from glance.tests import functional
-from glance.tests.utils import execute, depends_on_exe, skip_if_disabled
+from glance.tests.utils import depends_on_exe
+from glance.tests.utils import execute
+from glance.tests.utils import skip_if_disabled
 
 
 class TestSqlite(functional.FunctionalTest):
