@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack, LLC
+# Copyright 2011 OpenStack Foundation
 # Copyright 2012 Red Hat, Inc
 # All Rights Reserved.
 #
@@ -19,52 +17,30 @@
 """
 Tests copying images to a Glance API server which uses a filesystem-
 based storage backend.
-
-The from_swift testcase requires that a real Swift account is available.
-It looks in a file GLANCE_TEST_SWIFT_CONF environ variable for the
-credentials to use.
-
-Note that this test clears the entire container from the Swift account
-for use by the test case, so make sure you supply credentials for
-test accounts only.
-
-The from_s3 testcase requires that a real S3 account is available.
-It looks in a file specified in the GLANCE_TEST_S3_CONF environ variable
-for the credentials to use.
-
-Note that this test clears the entire bucket from the S3 account
-for use by the test case, so make sure you supply credentials for
-test accounts only.
-
-In either case, if a connection to the external store cannot be
-established, then the relevant test case is skipped.
 """
 
 import hashlib
 import httplib2
-import json
 import tempfile
 import time
 
-from glance.tests import functional
-from glance.tests.functional.store_utils import (setup_swift,
-                                                 teardown_swift,
-                                                 get_swift_uri,
-                                                 setup_s3,
-                                                 teardown_s3,
-                                                 get_s3_uri,
-                                                 setup_http,
-                                                 get_http_uri)
-from glance.tests.utils import skip_if_disabled, requires
+from six.moves import xrange
 
-FIVE_KB = 5 * 1024
+from glance.openstack.common import jsonutils
+from glance.openstack.common import units
+from glance.tests import functional
+from glance.tests.functional.store_utils import get_http_uri
+from glance.tests.functional.store_utils import setup_http
+from glance.tests.utils import skip_if_disabled
+
+FIVE_KB = 5 * units.Ki
 
 
 class TestCopyToFile(functional.FunctionalTest):
 
     """
-    Functional tests for copying images from the Swift, S3 & HTTP storage
-    backends to file
+    Functional tests for copying images from the HTTP storage
+    backend to file
     """
 
     def _do_test_copy_from(self, from_store, get_uri):
@@ -74,6 +50,7 @@ class TestCopyToFile(functional.FunctionalTest):
         self.cleanup()
 
         self.start_servers(**self.__dict__.copy())
+        setup_http(self)
 
         # POST /images with public image to be stored in from_store,
         # to stand in for the 'external' image
@@ -89,7 +66,7 @@ class TestCopyToFile(functional.FunctionalTest):
         response, content = http.request(path, 'POST', headers=headers,
                                          body=image_data)
         self.assertEqual(response.status, 201, content)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         original_image_id = data['image']['id']
 
@@ -105,7 +82,7 @@ class TestCopyToFile(functional.FunctionalTest):
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201, content)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         copy_image_id = data['image']['id']
         self.assertNotEqual(copy_image_id, original_image_id)
@@ -168,21 +145,12 @@ class TestCopyToFile(functional.FunctionalTest):
 
         self.stop_servers()
 
-    @requires(setup_swift, teardown_swift)
     @skip_if_disabled
-    def test_copy_from_swift(self):
+    def test_copy_from_http_store(self):
         """
-        Ensure we can copy from an external image in Swift.
+        Ensure we can copy from an external image in HTTP store.
         """
-        self._do_test_copy_from('swift', get_swift_uri)
-
-    @requires(setup_s3, teardown_s3)
-    @skip_if_disabled
-    def test_copy_from_s3(self):
-        """
-        Ensure we can copy from an external image in S3.
-        """
-        self._do_test_copy_from('s3', get_s3_uri)
+        self._do_test_copy_from('file', get_http_uri)
 
     @skip_if_disabled
     def _do_test_copy_from_http(self, exists):
@@ -210,7 +178,7 @@ class TestCopyToFile(functional.FunctionalTest):
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201, content)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         copy_image_id = data['image']['id']
         self.assertEqual(data['image']['status'], 'queued', content)
