@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack, LLC
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -33,17 +31,20 @@ skipped.
 import datetime
 import hashlib
 import httplib2
-import json
 import os
 import tempfile
+import uuid
 
+from glance.openstack.common import jsonutils
 from glance.openstack.common import timeutils
-from glance.openstack.common import uuidutils
-from glance.tests import functional
-from glance.tests.utils import skip_if_disabled, minimal_headers
+from glance.openstack.common import units
 
-FIVE_KB = 5 * 1024
-FIVE_GB = 5 * 1024 * 1024 * 1024
+from glance.tests import functional
+from glance.tests.utils import minimal_headers
+from glance.tests.utils import skip_if_disabled
+
+FIVE_KB = 5 * units.Ki
+FIVE_GB = 5 * units.Gi
 TEST_VAR_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             '../..', 'var'))
 
@@ -155,7 +156,7 @@ class TestSSL(functional.FunctionalTest):
                                           headers=headers,
                                           body=image_data)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['checksum'],
                          hashlib.md5(image_data).hexdigest())
         self.assertEqual(data['image']['size'], FIVE_KB)
@@ -226,7 +227,7 @@ class TestSSL(functional.FunctionalTest):
              "name": "Image1",
              "checksum": "c2e5db72bd7fd153f53ede5da5a06de3",
              "size": 5120}]}
-        self.assertEqual(json.loads(content), expected_result)
+        self.assertEqual(jsonutils.loads(content), expected_result)
 
         # 6. GET /images/detail
         # Verify image and all its metadata
@@ -247,7 +248,7 @@ class TestSSL(functional.FunctionalTest):
             "properties": {},
             "size": 5120}
 
-        image = json.loads(content)
+        image = jsonutils.loads(content)
 
         for expected_key, expected_value in expected_image.items():
             self.assertEqual(expected_value,
@@ -266,7 +267,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'PUT', headers=headers)
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['properties']['arch'], "x86_64")
         self.assertEqual(data['image']['properties']['distro'], "Ubuntu")
 
@@ -289,7 +290,7 @@ class TestSSL(functional.FunctionalTest):
             "properties": {'distro': 'Ubuntu', 'arch': 'x86_64'},
             "size": 5120}
 
-        image = json.loads(content)
+        image = jsonutils.loads(content)
 
         for expected_key, expected_value in expected_image.items():
             self.assertEqual(expected_value,
@@ -310,7 +311,7 @@ class TestSSL(functional.FunctionalTest):
         path = "https://%s:%d/v1/images/detail" % ("127.0.0.1", self.api_port)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)['images'][0]
+        data = jsonutils.loads(content)['images'][0]
         self.assertEqual(len(data['properties']), 1)
         self.assertEqual(data['properties']['arch'], "x86_64")
 
@@ -322,12 +323,12 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'PUT', headers=headers)
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         path = "https://%s:%d/v1/images/detail" % ("127.0.0.1", self.api_port)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)['images'][0]
+        data = jsonutils.loads(content)['images'][0]
         self.assertEqual(len(data['properties']), 2)
         self.assertEqual(data['properties']['arch'], "x86_64")
         self.assertEqual(data['properties']['distro'], "Ubuntu")
@@ -377,8 +378,8 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
-        self.assertEqual(data['image']['checksum'], None)
+        data = jsonutils.loads(content)
+        self.assertIsNone(data['image']['checksum'])
         self.assertEqual(data['image']['size'], 0)
         self.assertEqual(data['image']['container_format'], 'ovf')
         self.assertEqual(data['image']['disk_format'], 'raw')
@@ -393,9 +394,9 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['images'][0]['id'], image_id)
-        self.assertEqual(data['images'][0]['checksum'], None)
+        self.assertIsNone(data['images'][0]['checksum'])
         self.assertEqual(data['images'][0]['size'], 0)
         self.assertEqual(data['images'][0]['container_format'], 'ovf')
         self.assertEqual(data['images'][0]['disk_format'], 'raw')
@@ -424,7 +425,7 @@ class TestSSL(functional.FunctionalTest):
                                           headers=headers,
                                           body=image_data)
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['checksum'],
                          hashlib.md5(image_data).hexdigest())
         self.assertEqual(data['image']['size'], FIVE_KB)
@@ -447,7 +448,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['images'][0]['checksum'],
                          hashlib.md5(image_data).hexdigest())
         self.assertEqual(data['images'][0]['id'], image_id)
@@ -474,30 +475,30 @@ class TestSSL(functional.FunctionalTest):
             "status": "CURRENT",
             "links": [{
                 "rel": "self",
-                "href": "https://127.0.0.1:%d/v2/" % self.api_port}]}, {
-            "id": "v2.1",
-            "status": "SUPPORTED",
-            "links": [{
-                "rel": "self",
-                "href": "https://127.0.0.1:%d/v2/" % self.api_port}]}, {
-            "id": "v2.0",
-            "status": "SUPPORTED",
-            "links": [{
-                "rel": "self",
-                "href": "https://127.0.0.1:%d/v2/" % self.api_port}]}, {
-            "id": "v1.1",
-            "status": "CURRENT",
-            "links": [{
-                "rel": "self",
-                "href": "https://127.0.0.1:%d/v1/" % self.api_port}]}, {
-            "id": "v1.0",
-            "status": "SUPPORTED",
-            "links": [{
-                "rel": "self",
-                "href": "https://127.0.0.1:%d/v1/" % self.api_port}]}]}
-        versions_json = json.dumps(versions)
+                "href": "https://127.0.0.1:%d/v2/" % self.api_port}]},
+            {"id": "v2.1",
+             "status": "SUPPORTED",
+             "links": [{
+                 "rel": "self",
+                 "href": "https://127.0.0.1:%d/v2/" % self.api_port}]},
+            {"id": "v2.0",
+             "status": "SUPPORTED",
+             "links": [{
+                 "rel": "self",
+                 "href": "https://127.0.0.1:%d/v2/" % self.api_port}]},
+            {"id": "v1.1",
+             "status": "CURRENT",
+             "links": [{
+                 "rel": "self",
+                 "href": "https://127.0.0.1:%d/v1/" % self.api_port}]},
+            {"id": "v1.0",
+             "status": "SUPPORTED",
+             "links": [{
+                 "rel": "self",
+                 "href": "https://127.0.0.1:%d/v1/" % self.api_port}]}]}
+        versions_json = jsonutils.dumps(versions)
         images = {'images': []}
-        images_json = json.dumps(images)
+        images_json = jsonutils.dumps(images)
 
         # 0. GET / with no Accept: header
         # Verify version choices returned.
@@ -743,7 +744,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['properties']['pants'], "are on")
         self.assertEqual(data['image']['is_public'], True)
 
@@ -759,7 +760,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['properties']['pants'], "are on")
         self.assertEqual(data['image']['is_public'], True)
 
@@ -775,7 +776,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['properties']['pants'], "are off")
         self.assertEqual(data['image']['is_public'], True)
 
@@ -790,7 +791,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(data['image']['is_public'], False)
 
         # 2. GET /images
@@ -798,7 +799,7 @@ class TestSSL(functional.FunctionalTest):
         path = "https://%s:%d/v1/images" % ("127.0.0.1", self.api_port)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 3)
 
         # 3. GET /images with name filter
@@ -808,7 +809,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 2)
         for image in data['images']:
             self.assertEqual(image['name'], "My Image!")
@@ -820,7 +821,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 3)
         for image in data['images']:
             self.assertEqual(image['status'], "queued")
@@ -830,7 +831,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 0)
 
         # 5. GET /images with container_format filter
@@ -840,7 +841,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 2)
         for image in data['images']:
             self.assertEqual(image['container_format'], "ovf")
@@ -852,7 +853,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 1)
         for image in data['images']:
             self.assertEqual(image['disk_format'], "vdi")
@@ -864,7 +865,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 2)
         for image in data['images']:
             self.assertTrue(image['size'] <= 20)
@@ -876,7 +877,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 2)
         for image in data['images']:
             self.assertTrue(image['size'] >= 20)
@@ -889,7 +890,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 4)
 
         # 10. Get /images with is_public=False filter
@@ -900,7 +901,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 1)
         for image in data['images']:
             self.assertEqual(image['name'], "My Private Image")
@@ -913,7 +914,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 3)
         for image in data['images']:
             self.assertNotEqual(image['name'], "My Private Image")
@@ -925,7 +926,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 2)
         for image in data['images']:
             self.assertEqual(image['properties']['pants'], "are on")
@@ -935,10 +936,10 @@ class TestSSL(functional.FunctionalTest):
         # Make sure you quote the url when using more than one param!
         params = "name=My%20Image!&property-pants=are%20on"
         path = "https://%s:%d/v1/images/detail?%s" % (
-                "127.0.0.1", self.api_port, params)
+            "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 1)
         for image in data['images']:
             self.assertEqual(image['properties']['pants'], "are on")
@@ -952,7 +953,7 @@ class TestSSL(functional.FunctionalTest):
                                                self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 3)
 
         # 15. GET /images with future changes-since filter
@@ -963,7 +964,7 @@ class TestSSL(functional.FunctionalTest):
                                                self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 0)
 
         self.stop_servers()
@@ -990,7 +991,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         image_ids = [data['image']['id']]
 
@@ -999,7 +1000,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         image_ids.append(data['image']['id'])
 
@@ -1008,7 +1009,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         image_ids.append(data['image']['id'])
 
@@ -1019,7 +1020,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 2)
         self.assertEqual(data['images'][0]['id'], image_ids[2])
         self.assertEqual(data['images'][1]['id'], image_ids[1])
@@ -1031,7 +1032,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 2)
         self.assertEqual(data['images'][0]['id'], image_ids[1])
         self.assertEqual(data['images'][1]['id'], image_ids[0])
@@ -1043,7 +1044,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 1)
         self.assertEqual(data['images'][0]['id'], image_ids[0])
 
@@ -1054,7 +1055,7 @@ class TestSSL(functional.FunctionalTest):
                "127.0.0.1", self.api_port, params)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 1)
         self.assertEqual(data['images'][0]['id'], image_ids[1])
 
@@ -1088,7 +1089,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         image_ids = [data['image']['id']]
 
@@ -1103,7 +1104,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         image_ids.append(data['image']['id'])
 
@@ -1118,7 +1119,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         image_ids.append(data['image']['id'])
 
@@ -1128,7 +1129,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 3)
         self.assertEqual(data['images'][0]['id'], image_ids[2])
         self.assertEqual(data['images'][1]['id'], image_ids[1])
@@ -1141,7 +1142,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 3)
         self.assertEqual(data['images'][0]['id'], image_ids[1])
         self.assertEqual(data['images'][1]['id'], image_ids[0])
@@ -1154,7 +1155,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 3)
         self.assertEqual(data['images'][0]['id'], image_ids[0])
         self.assertEqual(data['images'][1]['id'], image_ids[2])
@@ -1166,7 +1167,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 2)
         self.assertEqual(data['images'][0]['id'], image_ids[2])
         self.assertEqual(data['images'][1]['id'], image_ids[1])
@@ -1178,7 +1179,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'GET')
         self.assertEqual(response.status, 200)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
         self.assertEqual(len(data['images']), 0)
 
         self.stop_servers()
@@ -1211,7 +1212,7 @@ class TestSSL(functional.FunctionalTest):
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'POST', headers=headers)
         self.assertEqual(response.status, 201)
-        data = json.loads(content)
+        data = jsonutils.loads(content)
 
         image_id = data['image']['id']
 
@@ -1258,7 +1259,7 @@ class TestSSL(functional.FunctionalTest):
         # 1. DELETE /images/1
         # Verify 404 returned
         path = "https://%s:%d/v1/images/%s" % ("127.0.0.1", self.api_port,
-                                               uuidutils.generate_uuid())
+                                               str(uuid.uuid4()))
         https = httplib2.Http(disable_ssl_certificate_validation=True)
         response, content = https.request(path, 'DELETE')
         self.assertEqual(response.status, 404)
