@@ -15,14 +15,14 @@ import os.path
 import tempfile
 
 import fixtures
-import glance_store
-from oslo_config import cfg
-from oslo_db import options
+from oslo.config import cfg
 
 import glance.common.client
 from glance.common import config
+from glance.db import migration
 import glance.db.sqlalchemy.api
 import glance.registry.client.v1.client
+import glance.store
 from glance import tests as glance_tests
 from glance.tests import utils as test_utils
 
@@ -111,17 +111,15 @@ paste.filter_factory = glance.tests.utils:FakeAuthMiddleware.factory
 """
 
 CONF = cfg.CONF
+CONF.import_opt('filesystem_store_datadir', 'glance.store.filesystem')
+CONF.import_opt('backend', 'glance.openstack.common.db.api', group='database')
 
 
 class ApiTest(test_utils.BaseTestCase):
     def setUp(self):
         super(ApiTest, self).setUp()
-        self.init()
-
-    def init(self):
         self.test_dir = self.useFixture(fixtures.TempDir()).path
         self._configure_logging()
-        self._configure_policy()
         self._setup_database()
         self._setup_stores()
         self._setup_property_protection()
@@ -144,10 +142,6 @@ class ApiTest(test_utils.BaseTestCase):
         self.property_file = os.path.join(self.test_dir,
                                           'property-protections.conf')
 
-    def _configure_policy(self):
-        policy_file = self._copy_data_file('policy.json', self.test_dir)
-        self.config(policy_file=policy_file, group='oslo_policy')
-
     def _configure_logging(self):
         self.config(default_log_levels=[
             'amqplib=WARN',
@@ -160,7 +154,7 @@ class ApiTest(test_utils.BaseTestCase):
 
     def _setup_database(self):
         sql_connection = 'sqlite:////%s/tests.sqlite' % self.test_dir
-        options.set_defaults(CONF, connection=sql_connection)
+        self.config(connection=sql_connection, group='database')
         glance.db.sqlalchemy.api.clear_db_env()
         glance_db_env = 'GLANCE_DB_TEST_SQLITE_FILE'
         if glance_db_env in os.environ:
@@ -170,7 +164,7 @@ class ApiTest(test_utils.BaseTestCase):
             test_utils.execute('cp %s %s/tests.sqlite'
                                % (db_location, self.test_dir))
         else:
-            test_utils.db_sync()
+            migration.db_sync()
 
             # copy the clean db to a temp location so that it
             # can be reused for future tests
@@ -192,17 +186,13 @@ class ApiTest(test_utils.BaseTestCase):
             atexit.register(_delete_cached_db)
 
     def _setup_stores(self):
-        glance_store.register_opts(CONF)
-
         image_dir = os.path.join(self.test_dir, "images")
-        self.config(group='glance_store',
-                    filesystem_store_datadir=image_dir)
-
-        glance_store.create_stores()
+        self.config(filesystem_store_datadir=image_dir)
+        glance.store.create_stores()
 
     def _load_paste_app(self, name, flavor, conf):
         conf_file_path = os.path.join(self.test_dir, '%s-paste.ini' % name)
-        with open(conf_file_path, 'w') as conf_file:
+        with open(conf_file_path, 'wb') as conf_file:
             conf_file.write(conf)
             conf_file.flush()
         return config.load_paste_app(name, flavor=flavor,
