@@ -47,7 +47,7 @@ class TestProtectedImageRepoProxy(utils.BaseTestCase):
     def setUp(self):
         super(TestProtectedImageRepoProxy, self).setUp()
         self.set_property_protections()
-        self.policy = policy.Enforcer(suppress_deprecation_warnings=True)
+        self.policy = policy.Enforcer()
         self.property_rules = property_utils.PropertyRules(self.policy)
         self.image_factory = glance.domain.ImageFactory()
         extra_props = {'spl_create_prop': 'c',
@@ -72,28 +72,28 @@ class TestProtectedImageRepoProxy(utils.BaseTestCase):
         image_id = '1'
         result_image = self.image_repo.get(image_id)
         result_extra_props = result_image.extra_properties
-        self.assertEqual('c', result_extra_props['spl_create_prop'])
-        self.assertEqual('r', result_extra_props['spl_read_prop'])
-        self.assertEqual('u', result_extra_props['spl_update_prop'])
-        self.assertEqual('d', result_extra_props['spl_delete_prop'])
-        self.assertNotIn('forbidden', result_extra_props.keys())
+        self.assertEqual(result_extra_props['spl_create_prop'], 'c')
+        self.assertEqual(result_extra_props['spl_read_prop'], 'r')
+        self.assertEqual(result_extra_props['spl_update_prop'], 'u')
+        self.assertEqual(result_extra_props['spl_delete_prop'], 'd')
+        self.assertFalse('forbidden' in result_extra_props.keys())
 
     def test_list_image(self):
         result_images = self.image_repo.list()
-        self.assertEqual(3, len(result_images))
+        self.assertEqual(len(result_images), 3)
         result_extra_props = result_images[0].extra_properties
-        self.assertEqual('c', result_extra_props['spl_create_prop'])
-        self.assertEqual('r', result_extra_props['spl_read_prop'])
-        self.assertEqual('u', result_extra_props['spl_update_prop'])
-        self.assertEqual('d', result_extra_props['spl_delete_prop'])
-        self.assertNotIn('forbidden', result_extra_props.keys())
+        self.assertEqual(result_extra_props['spl_create_prop'], 'c')
+        self.assertEqual(result_extra_props['spl_read_prop'], 'r')
+        self.assertEqual(result_extra_props['spl_update_prop'], 'u')
+        self.assertEqual(result_extra_props['spl_delete_prop'], 'd')
+        self.assertFalse('forbidden' in result_extra_props.keys())
 
         result_extra_props = result_images[1].extra_properties
-        self.assertEqual({}, result_extra_props)
+        self.assertEqual(result_extra_props, {})
 
         result_extra_props = result_images[2].extra_properties
-        self.assertEqual('r', result_extra_props['spl_read_prop'])
-        self.assertNotIn('forbidden', result_extra_props.keys())
+        self.assertEqual(result_extra_props['spl_read_prop'], 'r')
+        self.assertFalse('forbidden' in result_extra_props.keys())
 
 
 class TestProtectedImageProxy(utils.BaseTestCase):
@@ -101,7 +101,7 @@ class TestProtectedImageProxy(utils.BaseTestCase):
     def setUp(self):
         super(TestProtectedImageProxy, self).setUp()
         self.set_property_protections()
-        self.policy = policy.Enforcer(suppress_deprecation_warnings=True)
+        self.policy = policy.Enforcer()
         self.property_rules = property_utils.PropertyRules(self.policy)
 
     class ImageStub(object):
@@ -115,8 +115,8 @@ class TestProtectedImageProxy(utils.BaseTestCase):
         result_image = property_protections.ProtectedImageProxy(
             image, context, self.property_rules)
         result_extra_props = result_image.extra_properties
-        self.assertEqual('read', result_extra_props['spl_read_prop'])
-        self.assertNotIn('spl_fake_prop', result_extra_props.keys())
+        self.assertEqual(result_extra_props['spl_read_prop'], 'read')
+        self.assertFalse('spl_fake_prop' in result_extra_props.keys())
 
 
 class TestExtraPropertiesProxy(utils.BaseTestCase):
@@ -124,7 +124,7 @@ class TestExtraPropertiesProxy(utils.BaseTestCase):
     def setUp(self):
         super(TestExtraPropertiesProxy, self).setUp()
         self.set_property_protections()
-        self.policy = policy.Enforcer(suppress_deprecation_warnings=True)
+        self.policy = policy.Enforcer()
         self.property_rules = property_utils.PropertyRules(self.policy)
 
     def test_read_extra_property_as_admin_role(self):
@@ -133,7 +133,7 @@ class TestExtraPropertiesProxy(utils.BaseTestCase):
         extra_prop_proxy = property_protections.ExtraPropertiesProxy(
             context, extra_properties, self.property_rules)
         test_result = extra_prop_proxy['foo']
-        self.assertEqual('bar', test_result)
+        self.assertEqual(test_result, 'bar')
 
     def test_read_extra_property_as_unpermitted_role(self):
         extra_properties = {'foo': 'bar', 'ping': 'pong'}
@@ -148,7 +148,7 @@ class TestExtraPropertiesProxy(utils.BaseTestCase):
         extra_prop_proxy = property_protections.ExtraPropertiesProxy(
             context, extra_properties, self.property_rules)
         extra_prop_proxy['foo'] = 'par'
-        self.assertEqual('par', extra_prop_proxy['foo'])
+        self.assertEqual(extra_prop_proxy['foo'], 'par')
 
     def test_update_extra_property_as_unpermitted_role_after_read(self):
         extra_properties = {'spl_read_prop': 'bar'}
@@ -168,21 +168,13 @@ class TestExtraPropertiesProxy(utils.BaseTestCase):
                           extra_prop_proxy.__setitem__, 'spl_create_prop',
                           'par')
 
-    def test_update_empty_extra_property(self):
-        extra_properties = {'foo': ''}
-        context = glance.context.RequestContext(roles=['admin'])
-        extra_prop_proxy = property_protections.ExtraPropertiesProxy(
-            context, extra_properties, self.property_rules)
-        extra_prop_proxy['foo'] = 'bar'
-        self.assertEqual('bar', extra_prop_proxy['foo'])
-
     def test_create_extra_property_admin(self):
         extra_properties = {}
         context = glance.context.RequestContext(roles=['admin'])
         extra_prop_proxy = property_protections.ExtraPropertiesProxy(
             context, extra_properties, self.property_rules)
         extra_prop_proxy['boo'] = 'doo'
-        self.assertEqual('doo', extra_prop_proxy['boo'])
+        self.assertEqual(extra_prop_proxy['boo'], 'doo')
 
     def test_create_reserved_extra_property(self):
         extra_properties = {}
@@ -214,7 +206,7 @@ class TestExtraPropertiesProxy(utils.BaseTestCase):
         extra_prop_proxy = property_protections.ExtraPropertiesProxy(
             context, extra_properties, self.property_rules)
         # Ensure property has been created and can be read
-        self.assertEqual('r', extra_prop_proxy['spl_read_prop'])
+        self.assertEqual(extra_prop_proxy['spl_read_prop'], 'r')
         self.assertRaises(exception.ReservedProperty,
                           extra_prop_proxy.__delitem__, 'spl_read_prop')
 
@@ -226,20 +218,12 @@ class TestExtraPropertiesProxy(utils.BaseTestCase):
         self.assertRaises(KeyError,
                           extra_prop_proxy.__delitem__, 'spl_read_prop')
 
-    def test_delete_empty_extra_property(self):
-        extra_properties = {'foo': ''}
-        context = glance.context.RequestContext(roles=['admin'])
-        extra_prop_proxy = property_protections.ExtraPropertiesProxy(
-            context, extra_properties, self.property_rules)
-        del extra_prop_proxy['foo']
-        self.assertNotIn('foo', extra_prop_proxy)
-
 
 class TestProtectedImageFactoryProxy(utils.BaseTestCase):
     def setUp(self):
         super(TestProtectedImageFactoryProxy, self).setUp()
         self.set_property_protections()
-        self.policy = policy.Enforcer(suppress_deprecation_warnings=True)
+        self.policy = policy.Enforcer()
         self.property_rules = property_utils.PropertyRules(self.policy)
         self.factory = glance.domain.ImageFactory()
 
@@ -252,7 +236,7 @@ class TestProtectedImageFactoryProxy(utils.BaseTestCase):
         extra_props = {}
         image = self.image_factory.new_image(extra_properties=extra_props)
         expected_extra_props = {}
-        self.assertEqual(expected_extra_props, image.extra_properties)
+        self.assertEqual(image.extra_properties, expected_extra_props)
 
     def test_create_image_extra_prop(self):
         self.context = glance.context.RequestContext(tenant=TENANT1,
@@ -263,7 +247,7 @@ class TestProtectedImageFactoryProxy(utils.BaseTestCase):
         extra_props = {'spl_create_prop': 'c'}
         image = self.image_factory.new_image(extra_properties=extra_props)
         expected_extra_props = {'spl_create_prop': 'c'}
-        self.assertEqual(expected_extra_props, image.extra_properties)
+        self.assertEqual(image.extra_properties, expected_extra_props)
 
     def test_create_image_extra_prop_reserved_property(self):
         self.context = glance.context.RequestContext(tenant=TENANT1,
@@ -286,7 +270,7 @@ class TestProtectedImageFactoryProxy(utils.BaseTestCase):
         extra_props = {'foo': 'bar', 'spl_create_prop': 'c'}
         image = self.image_factory.new_image(extra_properties=extra_props)
         expected_extra_props = {'foo': 'bar', 'spl_create_prop': 'c'}
-        self.assertEqual(expected_extra_props, image.extra_properties)
+        self.assertEqual(image.extra_properties, expected_extra_props)
 
     def test_create_image_extra_prop_invalid_role(self):
         self.context = glance.context.RequestContext(tenant=TENANT1,
