@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation.
 # All Rights Reserved.
 #
@@ -15,21 +13,24 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import json
 import os
 import shutil
 
 import fixtures
 from oslo.config import cfg
-import stubout
 
+from glance.openstack.common import jsonutils
 from glance import store
 from glance.store import location
+from glance.store import sheepdog
+from glance.store import vmware_datastore
 from glance.tests import stubs
 from glance.tests import utils as test_utils
 
 CONF = cfg.CONF
 CONF.import_opt('filesystem_store_datadir', 'glance.store.filesystem')
+CONF.import_opt('connection', 'glance.openstack.common.db.sqlalchemy.session',
+                group='database')
 
 
 class StoreClearingUnitTest(test_utils.BaseTestCase):
@@ -38,9 +39,19 @@ class StoreClearingUnitTest(test_utils.BaseTestCase):
         super(StoreClearingUnitTest, self).setUp()
         # Ensure stores + locations cleared
         location.SCHEME_TO_CLS_MAP = {}
-        store.create_stores()
+
+        self._create_stores()
         self.addCleanup(setattr, location, 'SCHEME_TO_CLS_MAP', dict())
 
+    def _create_stores(self):
+        """Create known stores. Mock out sheepdog's subprocess dependency
+        on collie.
+        """
+        self.stubs.Set(sheepdog.Store, 'configure_add', lambda x: None)
+        self.stubs.Set(vmware_datastore.Store, 'configure', lambda x: None)
+        self.stubs.Set(vmware_datastore.Store, 'configure_add', lambda x: None)
+        store.create_stores()
+
 
 class IsolatedUnitTest(StoreClearingUnitTest):
 
@@ -48,20 +59,23 @@ class IsolatedUnitTest(StoreClearingUnitTest):
     Unit test case that establishes a mock environment within
     a testing directory (in isolation)
     """
+    registry = None
 
     def setUp(self):
         super(IsolatedUnitTest, self).setUp()
         self.test_dir = self.useFixture(fixtures.TempDir()).path
-        self.stubs = stubout.StubOutForTesting()
         policy_file = self._copy_data_file('policy.json', self.test_dir)
-        self.config(sql_connection='sqlite://',
-                    verbose=False,
+        self.config(connection='sqlite://',
+                    group='database')
+        self.config(verbose=False,
                     debug=False,
                     default_store='filesystem',
                     filesystem_store_datadir=os.path.join(self.test_dir),
-                    policy_file=policy_file)
-        stubs.stub_out_registry_and_store_server(self.stubs, self.test_dir)
-        self.addCleanup(self.stubs.UnsetAll)
+                    policy_file=policy_file,
+                    lock_path=os.path.join(self.test_dir))
+        stubs.stub_out_registry_and_store_server(self.stubs,
+                                                 self.test_dir,
+                                                 registry=self.registry)
 
     def _copy_data_file(self, file_name, dst_dir):
         src_file_name = os.path.join('glance/tests/etc', file_name)
@@ -71,5 +85,5 @@ class IsolatedUnitTest(StoreClearingUnitTest):
 
     def set_policy_rules(self, rules):
         fap = open(CONF.policy_file, 'w')
-        fap.write(json.dumps(rules))
+        fap.write(jsonutils.dumps(rules))
         fap.close()
