@@ -17,7 +17,7 @@ import os.path
 import shutil
 
 import fixtures
-import oslo_middleware
+import stubout
 
 from glance.api.middleware import context
 from glance.common import config
@@ -28,6 +28,8 @@ class TestPasteApp(test_utils.BaseTestCase):
 
     def setUp(self):
         super(TestPasteApp, self).setUp()
+        self.stubs = stubout.StubOutForTesting()
+        self.addCleanup(self.stubs.UnsetAll)
 
     def _do_test_load_paste_app(self,
                                 expected_app_type,
@@ -37,13 +39,13 @@ class TestPasteApp(test_utils.BaseTestCase):
                                 paste_append=None):
 
         def _writeto(path, str):
-            with open(path, 'w') as f:
+            with open(path, 'wb') as f:
                 f.write(str or '')
                 f.flush()
 
         def _appendto(orig, copy, str):
             shutil.copy(orig, copy)
-            with open(copy, 'a') as f:
+            with open(copy, 'ab') as f:
                 f.write(str or '')
                 f.flush()
 
@@ -61,15 +63,15 @@ class TestPasteApp(test_utils.BaseTestCase):
         paste_to = temp_file.replace('.conf', '-paste.ini')
         if not paste_config_file and make_paste_file:
             paste_from = os.path.join(os.getcwd(),
-                                      'etc/glance-api-paste.ini')
+                                      'etc/glance-registry-paste.ini')
             _appendto(paste_from, paste_to, paste_append)
 
-        app = config.load_paste_app('glance-api')
+        app = config.load_paste_app('glance-registry')
 
-        self.assertIsInstance(app['/'], expected_app_type)
+        self.assertIsInstance(app, expected_app_type)
 
     def test_load_paste_app(self):
-        expected_middleware = oslo_middleware.CORS
+        expected_middleware = context.UnauthenticatedContextMiddleware
         self._do_test_load_paste_app(expected_middleware)
 
     def test_load_paste_app_paste_config_not_found(self):
@@ -78,12 +80,8 @@ class TestPasteApp(test_utils.BaseTestCase):
                           expected_middleware, make_paste_file=False)
 
     def test_load_paste_app_with_paste_flavor(self):
-        pipeline = ('[composite:glance-api-incomplete]\n'
-                    'paste.composite_factory = glance.api:root_app_factory\n'
-                    '/: api-incomplete\n'
-                    '/healthcheck: healthcheck\n'
-                    '[pipeline:api-incomplete]\n'
-                    'pipeline = context rootapp')
+        pipeline = ('[pipeline:glance-registry-incomplete]\n'
+                    'pipeline = context registryapp')
         expected_middleware = context.ContextMiddleware
         self._do_test_load_paste_app(expected_middleware,
                                      paste_flavor='incomplete',
@@ -91,29 +89,10 @@ class TestPasteApp(test_utils.BaseTestCase):
 
     def test_load_paste_app_with_paste_config_file(self):
         paste_config_file = os.path.join(os.getcwd(),
-                                         'etc/glance-api-paste.ini')
-        expected_middleware = oslo_middleware.CORS
+                                         'etc/glance-registry-paste.ini')
+        expected_middleware = context.UnauthenticatedContextMiddleware
         self._do_test_load_paste_app(expected_middleware,
                                      paste_config_file=paste_config_file)
 
-    def test_load_paste_app_with_paste_config_file_but_not_exist(self):
-        paste_config_file = os.path.abspath("glance-api-paste.ini")
-        expected_middleware = oslo_middleware.Healthcheck
-        self.assertRaises(RuntimeError, self._do_test_load_paste_app,
-                          expected_middleware,
-                          paste_config_file=paste_config_file)
-
     def test_get_path_non_exist(self):
         self.assertRaises(RuntimeError, config._get_deployment_config_file)
-
-
-class TestDefaultConfig(test_utils.BaseTestCase):
-
-    def setUp(self):
-        super(TestDefaultConfig, self).setUp()
-        self.CONF = config.cfg.CONF
-        self.CONF.import_group('profiler', 'glance.common.wsgi')
-
-    def test_osprofiler_disabled(self):
-        self.assertFalse(self.CONF.profiler.enabled)
-        self.assertFalse(self.CONF.profiler.trace_sqlalchemy)
