@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -15,6 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import six
+
 from glance.common import exception
 from glance.tests import utils as test_utils
 
@@ -26,7 +26,7 @@ class GlanceExceptionTestCase(test_utils.BaseTestCase):
             message = "default message"
 
         exc = FakeGlanceException()
-        self.assertEquals(unicode(exc), 'default message')
+        self.assertEqual(unicode(exc), 'default message')
 
     def test_specified_error_msg(self):
         self.assertTrue('test' in unicode(exception.GlanceException('test')))
@@ -36,9 +36,13 @@ class GlanceExceptionTestCase(test_utils.BaseTestCase):
             message = "default message: %(code)s"
 
         exc = FakeGlanceException(code=500)
-        self.assertEquals(unicode(exc), "default message: 500")
+        self.assertEqual(unicode(exc), "default message: 500")
 
     def test_specified_error_msg_with_kwargs(self):
         self.assertTrue('test: 500' in
                         unicode(exception.GlanceException('test: %(code)s',
                                                           code=500)))
+
+    def test_non_unicode_error_msg(self):
+        exc = exception.GlanceException(str('test'))
+        self.assertIsInstance(six.text_type(exc), six.text_type)
