@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 # -*- coding: utf-8 -*-
 
 # Copyright 2013 Red Hat, Inc.
@@ -15,7 +14,6 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-import json
 import datetime
 
 import mox
@@ -23,9 +21,10 @@ from oslo.config import cfg
 import routes
 import webob
 
+from glance.common import exception
 from glance.common import rpc
 from glance.common import wsgi
-from glance.common import exception
+from glance.openstack.common import jsonutils
 from glance.tests.unit import base
 from glance.tests import utils as test_utils
 
@@ -123,22 +122,22 @@ class TestRPCController(base.IsolatedUnitTest):
         api = create_api()
         req = webob.Request.blank('/rpc')
         req.method = 'POST'
-        req.body = json.dumps([
+        req.body = jsonutils.dumps([
             {
                 "command": "get_images",
                 "kwargs": {"keyword": 1}
             }
         ])
         res = req.get_response(api)
-        returned = json.loads(res.body)
-        self.assertTrue(isinstance(returned, list))
-        self.assertEquals(returned[0], 1)
+        returned = jsonutils.loads(res.body)
+        self.assertIsInstance(returned, list)
+        self.assertEqual(returned[0], 1)
 
     def test_request_exc(self):
         api = create_api()
         req = webob.Request.blank('/rpc')
         req.method = 'POST'
-        req.body = json.dumps([
+        req.body = jsonutils.dumps([
             {
                 "command": "get_all_images",
                 "kwargs": {"keyword": 1}
@@ -148,7 +147,7 @@ class TestRPCController(base.IsolatedUnitTest):
         # Sending non-accepted keyword
         # to get_all_images method
         res = req.get_response(api)
-        returned = json.loads(res.body)
+        returned = jsonutils.loads(res.body)
         self.assertIn("_error", returned[0])
 
     def test_rpc_errors(self):
@@ -158,29 +157,29 @@ class TestRPCController(base.IsolatedUnitTest):
         req.content_type = 'application/json'
 
         # Body is not a list, it should fail
-        req.body = json.dumps({})
+        req.body = jsonutils.dumps({})
         res = req.get_response(api)
-        self.assertEquals(res.status_int, 400)
+        self.assertEqual(res.status_int, 400)
 
         # cmd is not dict, it should fail.
-        req.body = json.dumps([None])
+        req.body = jsonutils.dumps([None])
         res = req.get_response(api)
-        self.assertEquals(res.status_int, 400)
+        self.assertEqual(res.status_int, 400)
 
         # No command key, it should fail.
-        req.body = json.dumps([{}])
+        req.body = jsonutils.dumps([{}])
         res = req.get_response(api)
-        self.assertEquals(res.status_int, 400)
+        self.assertEqual(res.status_int, 400)
 
         # kwargs not dict, it should fail.
-        req.body = json.dumps([{"command": "test", "kwargs": 200}])
+        req.body = jsonutils.dumps([{"command": "test", "kwargs": 200}])
         res = req.get_response(api)
-        self.assertEquals(res.status_int, 400)
+        self.assertEqual(res.status_int, 400)
 
         # Command does not exist, it should fail.
-        req.body = json.dumps([{"command": "test"}])
+        req.body = jsonutils.dumps([{"command": "test"}])
         res = req.get_response(api)
-        self.assertEquals(res.status_int, 404)
+        self.assertEqual(res.status_int, 404)
 
     def test_rpc_exception_propagation(self):
         api = create_api()
@@ -188,20 +187,20 @@ class TestRPCController(base.IsolatedUnitTest):
         req.method = 'POST'
         req.content_type = 'application/json'
 
-        req.body = json.dumps([{"command": "raise_value_error"}])
+        req.body = jsonutils.dumps([{"command": "raise_value_error"}])
         res = req.get_response(api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        returned = json.loads(res.body)[0]
-        self.assertEquals(returned['_error']['cls'], 'exceptions.ValueError')
+        returned = jsonutils.loads(res.body)[0]
+        self.assertEqual(returned['_error']['cls'], 'exceptions.ValueError')
 
-        req.body = json.dumps([{"command": "raise_weird_error"}])
+        req.body = jsonutils.dumps([{"command": "raise_weird_error"}])
         res = req.get_response(api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        returned = json.loads(res.body)[0]
-        self.assertEquals(returned['_error']['cls'],
-                          'glance.common.exception.RPCError')
+        returned = jsonutils.loads(res.body)[0]
+        self.assertEqual(returned['_error']['cls'],
+                         'glance.common.exception.RPCError')
 
 
 class TestRPCClient(base.IsolatedUnitTest):
@@ -231,7 +230,7 @@ class TestRPCClient(base.IsolatedUnitTest):
                     {"command": "get_all_images"}]
 
         res = self.client.bulk_request(commands)
-        self.assertEquals(len(res), 2)
+        self.assertEqual(len(res), 2)
         self.assertTrue(res[0])
         self.assertFalse(res[1])
 
@@ -240,7 +239,7 @@ class TestRPCClient(base.IsolatedUnitTest):
             self.client.raise_value_error()
             self.fail("Exception not raised")
         except ValueError as exc:
-            self.assertEquals(str(exc), "Yep, Just like that!")
+            self.assertEqual(str(exc), "Yep, Just like that!")
 
     def test_rpc_exception(self):
         try:
@@ -251,8 +250,8 @@ class TestRPCClient(base.IsolatedUnitTest):
 
     def test_non_str_or_dict_response(self):
         rst = self.client.count_images(images=[1, 2, 3, 4])
-        self.assertEquals(rst, 4)
-        self.assertTrue(isinstance(rst, int))
+        self.assertEqual(rst, 4)
+        self.assertIsInstance(rst, int)
 
 
 class TestRPCJSONSerializer(test_utils.BaseTestCase):
