@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack, LLC
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,9 +14,10 @@
 #    under the License.
 
 import os
-import StringIO
 import tempfile
+import uuid
 
+import six
 import webob
 
 from glance.common import exception
@@ -39,7 +38,7 @@ class TestUtils(test_utils.BaseTestCase):
             for chunk in utils.CooperativeReader(tmp_fd):
                 bytes_read += len(chunk)
 
-        self.assertEquals(bytes_read, BYTES)
+        self.assertEqual(bytes_read, BYTES)
 
         bytes_read = 0
         with tempfile.TemporaryFile('w+') as tmp_fd:
@@ -51,7 +50,7 @@ class TestUtils(test_utils.BaseTestCase):
                 bytes_read += 1
                 byte = reader.read(1)
 
-        self.assertEquals(bytes_read, BYTES)
+        self.assertEqual(bytes_read, BYTES)
 
     def test_cooperative_reader_of_iterator(self):
         """Ensure cooperative reader supports iterator backends too"""
@@ -79,21 +78,21 @@ class TestUtils(test_utils.BaseTestCase):
         """Ensure limiting reader class accesses all bytes of file"""
         BYTES = 1024
         bytes_read = 0
-        data = StringIO.StringIO("*" * BYTES)
+        data = six.StringIO("*" * BYTES)
         for chunk in utils.LimitingReader(data, BYTES):
             bytes_read += len(chunk)
 
-        self.assertEquals(bytes_read, BYTES)
+        self.assertEqual(bytes_read, BYTES)
 
         bytes_read = 0
-        data = StringIO.StringIO("*" * BYTES)
+        data = six.StringIO("*" * BYTES)
         reader = utils.LimitingReader(data, BYTES)
         byte = reader.read(1)
         while len(byte) != 0:
             bytes_read += 1
             byte = reader.read(1)
 
-        self.assertEquals(bytes_read, BYTES)
+        self.assertEqual(bytes_read, BYTES)
 
     def test_limiting_reader_fails(self):
         """Ensure limiting reader class throws exceptions if limit exceeded"""
@@ -101,7 +100,7 @@ class TestUtils(test_utils.BaseTestCase):
 
         def _consume_all_iter():
             bytes_read = 0
-            data = StringIO.StringIO("*" * BYTES)
+            data = six.StringIO("*" * BYTES)
             for chunk in utils.LimitingReader(data, BYTES - 1):
                 bytes_read += len(chunk)
 
@@ -109,7 +108,7 @@ class TestUtils(test_utils.BaseTestCase):
 
         def _consume_all_read():
             bytes_read = 0
-            data = StringIO.StringIO("*" * BYTES)
+            data = six.StringIO("*" * BYTES)
             reader = utils.LimitingReader(data, BYTES - 1)
             byte = reader.read(1)
             while len(byte) != 0:
@@ -217,3 +216,15 @@ class TestUtils(test_utils.BaseTestCase):
                 self.assertRaises(RuntimeError,
                                   utils.validate_key_cert,
                                   keyf.name, keyf.name)
+
+
+class UUIDTestCase(test_utils.BaseTestCase):
+
+    def test_is_uuid_like(self):
+        self.assertTrue(utils.is_uuid_like(str(uuid.uuid4())))
+
+    def test_id_is_uuid_like(self):
+        self.assertFalse(utils.is_uuid_like(1234567))
+
+    def test_name_is_uuid_like(self):
+        self.assertFalse(utils.is_uuid_like('zhongyueluo'))
