@@ -1,6 +1,5 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2010-2011 OpenStack, LLC
+# Copyright 2010-2011 OpenStack Foundation
+# Copyright 2014 IBM Corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,38 +14,60 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import socket
-import time
-
 import datetime
+import socket
+
+from babel import localedata
 import eventlet.patcher
-import httplib2
+import fixtures
+import gettext
+import mock
 import webob
 
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
-from glance.openstack.common import jsonutils
+from glance.openstack.common import gettextutils
 from glance.tests import utils as test_utils
 
 
 class RequestTest(test_utils.BaseTestCase):
 
+    def _set_expected_languages(self, all_locales=[], avail_locales=None):
+        # Override localedata.locale_identifiers to return some locales.
+        def returns_some_locales(*args, **kwargs):
+            return all_locales
+
+        self.stubs.Set(localedata, 'locale_identifiers', returns_some_locales)
+
+        # Override gettext.find to return other than None for some languages.
+        def fake_gettext_find(lang_id, *args, **kwargs):
+            found_ret = '/glance/%s/LC_MESSAGES/glance.mo' % lang_id
+            if avail_locales is None:
+                # All locales are available.
+                return found_ret
+            languages = kwargs['languages']
+            if languages[0] in avail_locales:
+                return found_ret
+            return None
+
+        self.stubs.Set(gettext, 'find', fake_gettext_find)
+
     def test_content_type_missing(self):
         request = wsgi.Request.blank('/tests/123')
         self.assertRaises(exception.InvalidContentType,
-                          request.get_content_type, ('application/xml'))
+                          request.get_content_type, ('application/xml',))
 
     def test_content_type_unsupported(self):
         request = wsgi.Request.blank('/tests/123')
         request.headers["Content-Type"] = "text/html"
         self.assertRaises(exception.InvalidContentType,
-                          request.get_content_type, ('application/xml'))
+                          request.get_content_type, ('application/xml',))
 
     def test_content_type_with_charset(self):
         request = wsgi.Request.blank('/tests/123')
         request.headers["Content-Type"] = "application/json; charset=UTF-8"
-        result = request.get_content_type(('application/json'))
+        result = request.get_content_type(('application/json',))
         self.assertEqual(result, "application/json")
 
     def test_content_type_from_accept_xml(self):
@@ -80,6 +101,50 @@ class RequestTest(test_utils.BaseTestCase):
         result = request.best_match_content_type()
         self.assertEqual(result, "application/json")
 
+    def test_language_accept_default(self):
+        request = wsgi.Request.blank('/tests/123')
+        request.headers["Accept-Language"] = "zz-ZZ,zz;q=0.8"
+        result = request.best_match_language()
+        self.assertIsNone(result)
+
+    def test_language_accept_none(self):
+        request = wsgi.Request.blank('/tests/123')
+        result = request.best_match_language()
+        self.assertIsNone(result)
+
+    def test_best_match_language_expected(self):
+        # If Accept-Language is a supported language, best_match_language()
+        # returns it.
+        self._set_expected_languages(all_locales=['it'])
+
+        req = wsgi.Request.blank('/', headers={'Accept-Language': 'it'})
+        self.assertEqual('it', req.best_match_language())
+
+    def test_request_match_language_unexpected(self):
+        # If Accept-Language is a language we do not support,
+        # best_match_language() returns None.
+        self._set_expected_languages(all_locales=['it'])
+
+        req = wsgi.Request.blank('/', headers={'Accept-Language': 'zh'})
+        self.assertIsNone(req.best_match_language())
+
+    @mock.patch.object(webob.acceptparse.AcceptLanguage, 'best_match')
+    def test_best_match_language_unknown(self, mock_best_match):
+        # Test that we are actually invoking language negotiation by webop
+        request = wsgi.Request.blank('/')
+        accepted = 'unknown-lang'
+        request.headers = {'Accept-Language': accepted}
+
+        mock_best_match.return_value = None
+
+        self.assertIsNone(request.best_match_language())
+
+        # If Accept-Language is missing or empty, match should be None
+        request.headers = {'Accept-Language': ''}
+        self.assertIsNone(request.best_match_language())
+        request.headers.pop('Accept-Language')
+        self.assertIsNone(request.best_match_language())
+
 
 class ResourceTest(test_utils.BaseTestCase):
 
@@ -174,6 +239,42 @@ class ResourceTest(test_utils.BaseTestCase):
         self.assertIsInstance(response, webob.exc.HTTPForbidden)
         self.assertEqual(response.status_code, 403)
 
+    @mock.patch.object(wsgi, 'translate_exception')
+    def test_resource_call_error_handle_localized(self,
+                                                  mock_translate_exception):
+        class Controller(object):
+            def delete(self, req, identity):
+                raise webob.exc.HTTPBadRequest(explanation='Not Found')
+
+        actions = {'action': 'delete', 'identity': 12}
+        env = {'wsgiorg.routing_args': [None, actions]}
+        request = wsgi.Request.blank('/tests/123', environ=env)
+        message_es = 'No Encontrado'
+
+        resource = wsgi.Resource(Controller(),
+                                 wsgi.JSONRequestDeserializer(),
+                                 None)
+        translated_exc = webob.exc.HTTPBadRequest(message_es)
+        mock_translate_exception.return_value = translated_exc
+
+        e = self.assertRaises(webob.exc.HTTPBadRequest,
+                              resource, request)
+        self.assertEqual(message_es, str(e))
+
+    @mock.patch.object(webob.acceptparse.AcceptLanguage, 'best_match')
+    @mock.patch.object(gettextutils, 'translate')
+    def test_translate_exception(self, mock_translate, mock_best_match):
+
+        mock_translate.return_value = 'No Encontrado'
+        mock_best_match.return_value = 'de'
+
+        req = wsgi.Request.blank('/tests/123')
+        req.headers["Accept-Language"] = "de"
+
+        e = webob.exc.HTTPNotFound(explanation='Not Found')
+        e = wsgi.translate_exception(req, e)
+        self.assertEqual('No Encontrado', e.explanation)
+
 
 class JSONResponseSerializerTest(test_utils.BaseTestCase):
 
@@ -191,11 +292,15 @@ class JSONResponseSerializerTest(test_utils.BaseTestCase):
 
     def test_to_json_with_more_deep_format(self):
         fixture = {"is_public": True, "name": [{"name1": "test"}]}
-        expected = {"is_public": True, "name": [{"name1": "test"}]}
+        expected = '{"is_public": true, "name": [{"name1": "test"}]}'
         actual = wsgi.JSONResponseSerializer().to_json(fixture)
-        actual = jsonutils.loads(actual)
-        for k in expected:
-            self.assertEqual(expected[k], actual[k])
+        self.assertEqual(actual, expected)
+
+    def test_to_json_with_set(self):
+        fixture = set(["foo"])
+        expected = '["foo"]'
+        actual = wsgi.JSONResponseSerializer().to_json(fixture)
+        self.assertEqual(actual, expected)
 
     def test_default(self):
         fixture = {"key": "value"}
@@ -277,9 +382,9 @@ class JSONRequestDeserializerTest(test_utils.BaseTestCase):
 
 class ServerTest(test_utils.BaseTestCase):
     def test_create_pool(self):
-        """ Ensure the wsgi thread pool is an eventlet.greenpool.GreenPool. """
+        """Ensure the wsgi thread pool is an eventlet.greenpool.GreenPool."""
         actual = wsgi.Server(threads=1).create_pool()
-        self.assertTrue(isinstance(actual, eventlet.greenpool.GreenPool))
+        self.assertIsInstance(actual, eventlet.greenpool.GreenPool)
 
 
 class TestHelpers(test_utils.BaseTestCase):
@@ -300,7 +405,7 @@ class TestHelpers(test_utils.BaseTestCase):
                    'properties': {'distro': 'Ubuntu 10.04 LTS'}}
         headers = utils.image_meta_to_http_headers(fixture)
         for k, v in headers.iteritems():
-            self.assert_(isinstance(v, unicode), "%s is not unicode" % v)
+            self.assertIsInstance(v, unicode)
 
     def test_data_passed_properly_through_headers(self):
         """
@@ -326,3 +431,75 @@ class TestHelpers(test_utils.BaseTestCase):
                 self.assertEqual(v, result[k])
             else:
                 self.assertFalse(k in result)
+
+
+class GetSocketTestCase(test_utils.BaseTestCase):
+
+    def setUp(self):
+        super(GetSocketTestCase, self).setUp()
+        self.useFixture(fixtures.MonkeyPatch(
+            "glance.common.wsgi.get_bind_addr",
+            lambda x: ('192.168.0.13', 1234)))
+        addr_info_list = [(2, 1, 6, '', ('192.168.0.13', 80)),
+                          (2, 2, 17, '', ('192.168.0.13', 80)),
+                          (2, 3, 0, '', ('192.168.0.13', 80))]
+        self.useFixture(fixtures.MonkeyPatch(
+            "glance.common.wsgi.socket.getaddrinfo",
+            lambda *x: addr_info_list))
+        self.useFixture(fixtures.MonkeyPatch(
+            "glance.common.wsgi.time.time",
+            mock.Mock(side_effect=[0, 1, 5, 10, 20, 35])))
+        self.useFixture(fixtures.MonkeyPatch(
+            "glance.common.wsgi.utils.validate_key_cert",
+            lambda *x: None))
+        wsgi.CONF.cert_file = '/etc/ssl/cert'
+        wsgi.CONF.key_file = '/etc/ssl/key'
+        wsgi.CONF.ca_file = '/etc/ssl/ca_cert'
+        wsgi.CONF.tcp_keepidle = 600
+
+    def test_correct_get_socket(self):
+        mock_socket = mock.Mock()
+        self.useFixture(fixtures.MonkeyPatch(
+            'glance.common.wsgi.ssl.wrap_socket',
+            mock_socket))
+        self.useFixture(fixtures.MonkeyPatch(
+            'glance.common.wsgi.eventlet.listen',
+            lambda *x, **y: None))
+        wsgi.get_socket(1234)
+        self.assertTrue(mock.call().setsockopt(
+            socket.SOL_SOCKET,
+            socket.SO_REUSEADDR,
+            1) in mock_socket.mock_calls)
+        self.assertTrue(mock.call().setsockopt(
+            socket.SOL_SOCKET,
+            socket.SO_KEEPALIVE,
+            1) in mock_socket.mock_calls)
+        if hasattr(socket, 'TCP_KEEPIDLE'):
+            self.assertTrue(mock.call().setsockopt(
+                socket.IPPROTO_TCP,
+                socket.TCP_KEEPIDLE,
+                wsgi.CONF.tcp_keepidle) in mock_socket.mock_calls)
+
+    def test_get_socket_without_all_ssl_reqs(self):
+        wsgi.CONF.key_file = None
+        self.assertRaises(RuntimeError, wsgi.get_socket, 1234)
+
+    def test_get_socket_with_bind_problems(self):
+        self.useFixture(fixtures.MonkeyPatch(
+            'glance.common.wsgi.eventlet.listen',
+            mock.Mock(side_effect=(
+                [wsgi.socket.error(socket.errno.EADDRINUSE)] * 3 + [None]))))
+        self.useFixture(fixtures.MonkeyPatch(
+            'glance.common.wsgi.ssl.wrap_socket',
+            lambda *x, **y: None))
+
+        self.assertRaises(RuntimeError, wsgi.get_socket, 1234)
+
+    def test_get_socket_with_unexpected_socket_errno(self):
+        self.useFixture(fixtures.MonkeyPatch(
+            'glance.common.wsgi.eventlet.listen',
+            mock.Mock(side_effect=wsgi.socket.error(socket.errno.ENOMEM))))
+        self.useFixture(fixtures.MonkeyPatch(
+            'glance.common.wsgi.ssl.wrap_socket',
+            lambda *x, **y: None))
+        self.assertRaises(wsgi.socket.error, wsgi.get_socket, 1234)
