@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2010-2011 OpenStack, LLC
+# Copyright 2010-2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -52,27 +50,6 @@ class TestContext(utils.BaseTestCase):
 
         self.assertEqual(self.db_api.is_image_visible(ctx, img), exp_res)
 
-    def do_sharable(self, exp_res, img_owner, membership=None, **kwargs):
-        """
-        Perform a context sharability test.  Creates a (fake) image
-        with the specified owner and is_public attributes, then
-        creates a context with the given keyword arguments and expects
-        exp_res as the result of an is_image_sharable() call on the
-        context.  If membership is not None, its value will be passed
-        in as the 'membership' keyword argument of
-        is_image_sharable().
-        """
-
-        img = _fake_image(img_owner, True)
-        ctx = context.RequestContext(**kwargs)
-
-        sharable_args = {}
-        if membership is not None:
-            sharable_args['membership'] = membership
-
-        output = self.db_api.is_image_sharable(ctx, img, **sharable_args)
-        self.assertEqual(exp_res, output)
-
     def test_empty_public(self):
         """
         Tests that an empty context (with is_admin set to True) can
@@ -101,15 +78,6 @@ class TestContext(utils.BaseTestCase):
         """
         self.do_visible(True, 'pattieblack', False, is_admin=True)
 
-    def test_empty_shared(self):
-        """
-        Tests that an empty context (with is_admin set to False) can
-        not share an image, with or without membership.
-        """
-        self.do_sharable(False, 'pattieblack', None, is_admin=False)
-        self.do_sharable(False, 'pattieblack', _fake_membership(True),
-                         is_admin=False)
-
     def test_anon_public(self):
         """
         Tests that an anonymous context (with is_admin set to False)
@@ -138,14 +106,6 @@ class TestContext(utils.BaseTestCase):
         """
         self.do_visible(False, 'pattieblack', False)
 
-    def test_anon_shared(self):
-        """
-        Tests that an empty context (with is_admin set to True) can
-        not share an image, with or without membership.
-        """
-        self.do_sharable(False, 'pattieblack', None)
-        self.do_sharable(False, 'pattieblack', _fake_membership(True))
-
     def test_auth_public(self):
         """
         Tests that an authenticated context (with is_admin set to
@@ -192,49 +152,6 @@ class TestContext(utils.BaseTestCase):
         """
         self.do_visible(True, 'pattieblack', False, tenant='pattieblack')
 
-    def test_auth_sharable(self):
-        """
-        Tests that an authenticated context (with is_admin set to
-        False) cannot share an image it neither owns nor is shared
-        with it.
-        """
-        self.do_sharable(False, 'pattieblack', None, tenant='froggy')
-
-    def test_auth_sharable_admin(self):
-        """
-        Tests that an authenticated context (with is_admin set to
-        True) can share an image it neither owns nor is shared with
-        it.
-        """
-        self.do_sharable(True, 'pattieblack', None, tenant='froggy',
-                         is_admin=True)
-
-    def test_auth_sharable_owned(self):
-        """
-        Tests that an authenticated context (with is_admin set to
-        False) can share an image it owns, even if it is not shared
-        with it.
-        """
-        self.do_sharable(True, 'pattieblack', None, tenant='pattieblack')
-
-    def test_auth_sharable_cannot_share(self):
-        """
-        Tests that an authenticated context (with is_admin set to
-        False) cannot share an image it does not own even if it is
-        shared with it, but with can_share = False.
-        """
-        self.do_sharable(False, 'pattieblack', _fake_membership(False),
-                         tenant='froggy')
-
-    def test_auth_sharable_can_share(self):
-        """
-        Tests that an authenticated context (with is_admin set to
-        False) can share an image it does not own if it is shared with
-        it with can_share = True.
-        """
-        self.do_sharable(True, 'pattieblack', _fake_membership(True),
-                         tenant='froggy')
-
     def test_request_id(self):
         contexts = [context.RequestContext().request_id for _ in range(5)]
         # Check for uniqueness -- set() will normalize its argument
@@ -250,3 +167,12 @@ class TestContext(utils.BaseTestCase):
         ctx = context.RequestContext()
         self.assertTrue(hasattr(local.store, 'context'))
         self.assertEqual(ctx, local.store.context)
+
+    def test_user_identity(self):
+        ctx = context.RequestContext(user="user",
+                                     tenant="tenant",
+                                     domain="domain",
+                                     user_domain="user-domain",
+                                     project_domain="project-domain")
+        self.assertEqual('user tenant domain user-domain project-domain',
+                         ctx.to_dict()["user_identity"])
