@@ -40,10 +40,10 @@ class TestContextMiddleware(base.IsolatedUnitTest):
     def test_header_parsing(self):
         req = self._build_request()
         self._build_middleware().process_request(req)
-        self.assertEqual('token1', req.context.auth_token)
-        self.assertEqual('user1', req.context.user_id)
-        self.assertEqual('tenant1', req.context.project_id)
-        self.assertEqual(['role1', 'role2'], req.context.roles)
+        self.assertEqual(req.context.auth_tok, 'token1')
+        self.assertEqual(req.context.user, 'user1')
+        self.assertEqual(req.context.tenant, 'tenant1')
+        self.assertEqual(req.context.roles, ['role1', 'role2'])
 
     def test_is_admin_flag(self):
         # is_admin check should look for 'admin' role by default
@@ -56,11 +56,11 @@ class TestContextMiddleware(base.IsolatedUnitTest):
         self._build_middleware().process_request(req)
         self.assertFalse(req.context.is_admin)
 
-        # the admin_role config option was removed in Wallaby
-        from oslo_config.cfg import NoSuchOptError
-        self.assertRaises(NoSuchOptError,
-                          self.config,
-                          admin_role='role1')
+        # if we change the admin_role attribute, we should be able to use it
+        req = self._build_request()
+        self.config(admin_role='role1')
+        self._build_middleware().process_request(req)
+        self.assertTrue(req.context.is_admin)
 
     def test_roles_case_insensitive(self):
         # accept role from request
@@ -68,21 +68,34 @@ class TestContextMiddleware(base.IsolatedUnitTest):
         self._build_middleware().process_request(req)
         self.assertTrue(req.context.is_admin)
 
+        # accept role from config
+        req = self._build_request(roles=['role1'])
+        self.config(admin_role='rOLe1')
+        self._build_middleware().process_request(req)
+        self.assertTrue(req.context.is_admin)
+
     def test_roles_stripping(self):
         # stripping extra spaces in request
         req = self._build_request(roles=['\trole1'])
+        self.config(admin_role='role1')
         self._build_middleware().process_request(req)
-        self.assertIn('role1', req.context.roles)
+        self.assertTrue(req.context.is_admin)
+
+        # stripping extra spaces in config
+        req = self._build_request(roles=['\trole1\n'])
+        self.config(admin_role=' role1\t')
+        self._build_middleware().process_request(req)
+        self.assertTrue(req.context.is_admin)
 
     def test_anonymous_access_enabled(self):
         req = self._build_request(identity_status='Nope')
         self.config(allow_anonymous_access=True)
         middleware = self._build_middleware()
         middleware.process_request(req)
-        self.assertIsNone(req.context.auth_token)
-        self.assertIsNone(req.context.user_id)
-        self.assertIsNone(req.context.project_id)
-        self.assertEqual([], req.context.roles)
+        self.assertIsNone(req.context.auth_tok)
+        self.assertIsNone(req.context.user)
+        self.assertIsNone(req.context.tenant)
+        self.assertEqual(req.context.roles, [])
         self.assertFalse(req.context.is_admin)
         self.assertTrue(req.context.read_only)
 
@@ -105,47 +118,24 @@ class TestContextMiddleware(base.IsolatedUnitTest):
         self.assertRaises(webob.exc.HTTPInternalServerError,
                           middleware.process_request, req)
 
-    def test_response(self):
-        req = self._build_request()
-        req.context = glance.context.RequestContext()
-        request_id = req.context.request_id
-
-        resp = webob.Response()
-        resp.request = req
-        self._build_middleware().process_response(resp)
-        self.assertEqual(request_id, resp.headers['x-openstack-request-id'])
-        resp_req_id = resp.headers['x-openstack-request-id']
-        # Validate that request-id do not starts with 'req-req-'
-        if isinstance(resp_req_id, bytes):
-            resp_req_id = resp_req_id.decode('utf-8')
-        self.assertFalse(resp_req_id.startswith('req-req-'))
-        self.assertTrue(resp_req_id.startswith('req-'))
-
 
 class TestUnauthenticatedContextMiddleware(base.IsolatedUnitTest):
     def test_request(self):
         middleware = context.UnauthenticatedContextMiddleware(None)
         req = webob.Request.blank('/')
         middleware.process_request(req)
-        self.assertIsNone(req.context.auth_token)
-        self.assertIsNone(req.context.user_id)
-        self.assertIsNone(req.context.project_id)
-        self.assertEqual([], req.context.roles)
+        self.assertIsNone(req.context.auth_tok)
+        self.assertIsNone(req.context.user)
+        self.assertIsNone(req.context.tenant)
+        self.assertEqual(req.context.roles, [])
         self.assertTrue(req.context.is_admin)
 
     def test_response(self):
         middleware = context.UnauthenticatedContextMiddleware(None)
         req = webob.Request.blank('/')
         req.context = glance.context.RequestContext()
-        request_id = req.context.request_id
-
         resp = webob.Response()
         resp.request = req
         middleware.process_response(resp)
-        self.assertEqual(request_id, resp.headers['x-openstack-request-id'])
-        resp_req_id = resp.headers['x-openstack-request-id']
-        if isinstance(resp_req_id, bytes):
-            resp_req_id = resp_req_id.decode('utf-8')
-        # Validate that request-id do not starts with 'req-req-'
-        self.assertFalse(resp_req_id.startswith('req-req-'))
-        self.assertTrue(resp_req_id.startswith('req-'))
+        self.assertEqual(resp.headers['x-openstack-request-id'],
+                         'req-%s' % req.context.request_id)
