@@ -1,4 +1,5 @@
 # Copyright 2013 OpenStack Foundation.
+# Copyright 2013 IBM Corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,6 +14,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import mock
+from six.moves import xrange
+
 from glance.domain import proxy
 import glance.tests.utils as test_utils
 
@@ -88,17 +92,17 @@ class TestProxyRepoWrapping(test_utils.BaseTestCase):
         self.fake_repo.result = base_result
         method = getattr(self.proxy_repo, name)
         proxy_result = method(*args, **kwargs)
-        self.assertTrue(isinstance(proxy_result, FakeProxy))
-        self.assertEquals(proxy_result.base, base_result)
-        self.assertEquals(len(proxy_result.args), 0)
-        self.assertEquals(proxy_result.kwargs, {'a': 1})
+        self.assertIsInstance(proxy_result, FakeProxy)
+        self.assertEqual(proxy_result.base, base_result)
+        self.assertEqual(len(proxy_result.args), 0)
+        self.assertEqual(proxy_result.kwargs, {'a': 1})
         self.assertEqual(self.fake_repo.args, args)
         self.assertEqual(self.fake_repo.kwargs, kwargs)
 
     def test_get(self):
         self.fake_repo.result = 'snarf'
         result = self.proxy_repo.get('some-id')
-        self.assertTrue(isinstance(result, FakeProxy))
+        self.assertIsInstance(result, FakeProxy)
         self.assertEqual(self.fake_repo.args, ('some-id',))
         self.assertEqual(self.fake_repo.kwargs, {})
         self.assertEqual(result.base, 'snarf')
@@ -112,7 +116,7 @@ class TestProxyRepoWrapping(test_utils.BaseTestCase):
         self.assertEqual(self.fake_repo.kwargs, {'prefix': 's'})
         self.assertEqual(len(results), 2)
         for i in xrange(2):
-            self.assertTrue(isinstance(results[i], FakeProxy))
+            self.assertIsInstance(results[i], FakeProxy)
             self.assertEqual(results[i].base, self.fake_repo.result[i])
             self.assertEqual(results[i].args, tuple())
             self.assertEqual(results[i].kwargs, {'a': 1})
@@ -129,7 +133,7 @@ class TestProxyRepoWrapping(test_utils.BaseTestCase):
         if result is None:
             self.assertTrue(proxy_result is None)
         else:
-            self.assertTrue(isinstance(proxy_result, FakeProxy))
+            self.assertIsInstance(proxy_result, FakeProxy)
             self.assertEqual(proxy_result.base, result)
             self.assertEqual(proxy_result.args, tuple())
             self.assertEqual(proxy_result.kwargs, {'a': 1})
@@ -181,7 +185,7 @@ class TestImageFactory(test_utils.BaseTestCase):
                                            proxy_kwargs={'dog': 'bark'})
         self.factory.result = 'stark'
         image = proxy_factory.new_image(a=1, b='two')
-        self.assertTrue(isinstance(image, FakeProxy))
+        self.assertIsInstance(image, FakeProxy)
         self.assertEqual(image.base, 'stark')
         self.assertEqual(self.factory.kwargs, {'a': 1, 'b': 'two'})
 
@@ -213,11 +217,11 @@ class TestImageMembershipFactory(test_utils.BaseTestCase):
 
     def test_proxy_wrapped_membership(self):
         proxy_factory = proxy.ImageMembershipFactory(
-                self.factory, member_proxy_class=FakeProxy,
-                member_proxy_kwargs={'a': 1})
+            self.factory, member_proxy_class=FakeProxy,
+            member_proxy_kwargs={'a': 1})
         self.factory.result = 'tyrion'
         membership = proxy_factory.new_image_member('jaime', 'cersei')
-        self.assertTrue(isinstance(membership, FakeProxy))
+        self.assertIsInstance(membership, FakeProxy)
         self.assertEqual(membership.base, 'tyrion')
         self.assertEqual(membership.kwargs, {'a': 1})
         self.assertEqual(self.factory.image, 'jaime')
@@ -225,7 +229,7 @@ class TestImageMembershipFactory(test_utils.BaseTestCase):
 
     def test_proxy_wrapped_image(self):
         proxy_factory = proxy.ImageMembershipFactory(
-                self.factory, image_proxy_class=FakeProxy)
+            self.factory, image_proxy_class=FakeProxy)
         self.factory.result = 'tyrion'
         image = FakeProxy('jaime')
         membership = proxy_factory.new_image_member(image, 'cersei')
@@ -238,15 +242,15 @@ class TestImageMembershipFactory(test_utils.BaseTestCase):
             pass
 
         proxy_factory = proxy.ImageMembershipFactory(
-                self.factory,
-                member_proxy_class=FakeProxy,
-                member_proxy_kwargs={'b': 2},
-                image_proxy_class=FakeProxy2)
+            self.factory,
+            member_proxy_class=FakeProxy,
+            member_proxy_kwargs={'b': 2},
+            image_proxy_class=FakeProxy2)
 
         self.factory.result = 'tyrion'
         image = FakeProxy2('jaime')
         membership = proxy_factory.new_image_member(image, 'cersei')
-        self.assertTrue(isinstance(membership, FakeProxy))
+        self.assertIsInstance(membership, FakeProxy)
         self.assertEqual(membership.base, 'tyrion')
         self.assertEqual(membership.kwargs, {'b': 2})
         self.assertEqual(self.factory.image, 'jaime')
@@ -277,5 +281,68 @@ class TestImage(test_utils.BaseTestCase):
                                   member_repo_proxy_kwargs={'a': 10})
         self.image.result = 'corn'
         member_repo = proxy_image.get_member_repo()
-        self.assertTrue(isinstance(member_repo, FakeProxy))
+        self.assertIsInstance(member_repo, FakeProxy)
         self.assertEqual(member_repo.base, 'corn')
+
+
+class TestTaskFactory(test_utils.BaseTestCase):
+    def setUp(self):
+        super(TestTaskFactory, self).setUp()
+        self.factory = mock.Mock()
+        self.fake_type = 'import'
+        self.fake_owner = "owner"
+
+    def test_proxy_plain(self):
+        proxy_factory = proxy.TaskFactory(self.factory)
+
+        proxy_factory.new_task(
+            type=self.fake_type,
+            owner=self.fake_owner
+        )
+
+        self.factory.new_task.assert_called_once_with(
+            type=self.fake_type,
+            owner=self.fake_owner
+        )
+
+        proxy_factory.new_task_details("task_01", "input")
+
+        self.factory.new_task_details.assert_called_once_with(
+            "task_01",
+            "input",
+            None, None
+        )
+
+    def test_proxy_wrapping(self):
+        proxy_factory = proxy.TaskFactory(
+            self.factory,
+            task_proxy_class=FakeProxy,
+            task_proxy_kwargs={'dog': 'bark'},
+            task_details_proxy_class=FakeProxy,
+            task_details_proxy_kwargs={'dog': 'bark'})
+
+        self.factory.new_task.return_value = 'fake_task'
+        self.factory.new_task_details.return_value = 'fake_task_detail'
+
+        task = proxy_factory.new_task(
+            type=self.fake_type,
+            owner=self.fake_owner
+        )
+
+        self.factory.new_task.assert_called_once_with(
+            type=self.fake_type,
+            owner=self.fake_owner
+        )
+        self.assertIsInstance(task, FakeProxy)
+        self.assertEqual(task.base, 'fake_task')
+
+        task_details = proxy_factory.new_task_details('task_01', "input")
+
+        self.factory.new_task_details.assert_called_once_with(
+            'task_01',
+            "input",
+            None, None
+        )
+
+        self.assertIsInstance(task_details, FakeProxy)
+        self.assertEqual(task_details.base, 'fake_task_detail')
