@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 #    Copyright 2012 Michael Still and Canonical Inc
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,17 +13,16 @@
 #    under the License.
 
 import copy
-import json
 import os
-import StringIO
 import sys
 import UserDict
 import uuid
 
 import fixtures
+import six
 
 from glance.cmd import replicator as glance_replicator
-from glance.tests.unit import utils as unit_test_utils
+from glance.openstack.common import jsonutils
 from glance.tests import utils as test_utils
 
 
@@ -76,7 +73,7 @@ class FakeHTTPConnection(object):
                       out_code, out_body, out_headers):
         if not url.startswith('/'):
             url = '/' + url
-        url = unit_test_utils.sort_url_by_qs_keys(url)
+
         hkeys = in_headers.keys()
         hkeys.sort()
         hashable = (method, url, in_body, ' '.join(hkeys))
@@ -89,7 +86,7 @@ class FakeHTTPConnection(object):
 
     def request(self, method, url, body, headers):
         self.count += 1
-        url = unit_test_utils.sort_url_by_qs_keys(url)
+
         hkeys = headers.keys()
         hkeys.sort()
         hashable = (method, url, body, ' '.join(hkeys))
@@ -109,7 +106,7 @@ class FakeHTTPConnection(object):
     def getresponse(self):
         class FakeResponse(object):
             def __init__(self, (code, body, headers)):
-                self.body = StringIO.StringIO(body)
+                self.body = six.StringIO(body)
                 self.headers = headers
                 self.status = code
 
@@ -146,16 +143,16 @@ class ImageServiceTestCase(test_utils.BaseTestCase):
         resp = {'images': [IMG_RESPONSE_ACTIVE, IMG_RESPONSE_QUEUED]}
         c.conn.prime_request('GET', 'v1/images/detail?is_public=None', '',
                              {'x-auth-token': 'noauth'},
-                             200, json.dumps(resp), {})
+                             200, jsonutils.dumps(resp), {})
         c.conn.prime_request('GET',
                              ('v1/images/detail?marker=%s&is_public=None'
                               % IMG_RESPONSE_QUEUED['id']),
                              '', {'x-auth-token': 'noauth'},
-                             200, json.dumps({'images': []}), {})
+                             200, jsonutils.dumps({'images': []}), {})
 
         imgs = list(c.get_images())
-        self.assertEquals(len(imgs), 2)
-        self.assertEquals(c.conn.count, 2)
+        self.assertEqual(len(imgs), 2)
+        self.assertEqual(c.conn.count, 2)
 
     def test_rest_get_image(self):
         c = glance_replicator.ImageService(FakeHTTPConnection(), 'noauth')
@@ -167,7 +164,7 @@ class ImageServiceTestCase(test_utils.BaseTestCase):
                              200, image_contents, IMG_RESPONSE_ACTIVE)
 
         body = c.get_image(IMG_RESPONSE_ACTIVE['id'])
-        self.assertEquals(body.read(), image_contents)
+        self.assertEqual(body.read(), image_contents)
 
     def test_rest_header_list_to_dict(self):
         i = [('x-image-meta-banana', 42),
@@ -225,8 +222,8 @@ class ImageServiceTestCase(test_utils.BaseTestCase):
                              200, '', IMG_RESPONSE_ACTIVE)
 
         headers, body = c.add_image(IMG_RESPONSE_ACTIVE, image_body)
-        self.assertEquals(headers, IMG_RESPONSE_ACTIVE)
-        self.assertEquals(c.conn.count, 1)
+        self.assertEqual(headers, IMG_RESPONSE_ACTIVE)
+        self.assertEqual(c.conn.count, 1)
 
     def test_rest_add_image_meta(self):
         c = glance_replicator.ImageService(FakeHTTPConnection(), 'noauth')
@@ -244,7 +241,7 @@ class ImageServiceTestCase(test_utils.BaseTestCase):
 class FakeHttpResponse(object):
     def __init__(self, headers, data):
         self.headers = headers
-        self.data = StringIO.StringIO(data)
+        self.data = six.StringIO(data)
 
     def getheaders(self):
         return self.headers
@@ -342,7 +339,7 @@ class ReplicationCommandsTestCase(test_utils.BaseTestCase):
 
         stdout = sys.stdout
         orig_img_service = glance_replicator.get_image_service
-        sys.stdout = StringIO.StringIO()
+        sys.stdout = six.StringIO()
         try:
             glance_replicator.get_image_service = get_image_service
             glance_replicator.replication_size(options, args)
@@ -387,7 +384,7 @@ class ReplicationCommandsTestCase(test_utils.BaseTestCase):
             self.assertTrue(os.path.exists('%s.img' % imgfile))
 
             with open(imgfile) as f:
-                d = json.loads(f.read())
+                d = jsonutils.loads(f.read())
                 self.assertTrue('status' in d)
                 self.assertTrue('id' in d)
                 self.assertTrue('size' in d)
@@ -398,7 +395,7 @@ class ReplicationCommandsTestCase(test_utils.BaseTestCase):
             self.assertFalse(os.path.exists('%s.img' % imgfile))
 
             with open(imgfile) as f:
-                d = json.loads(f.read())
+                d = jsonutils.loads(f.read())
                 self.assertTrue('status' in d)
                 self.assertTrue('id' in d)
                 self.assertTrue('size' in d)
@@ -419,7 +416,7 @@ class ReplicationCommandsTestCase(test_utils.BaseTestCase):
         def write_image(img, data):
             imgfile = os.path.join(tempdir, img['id'])
             with open(imgfile, 'w') as f:
-                f.write(json.dumps(img))
+                f.write(jsonutils.dumps(img))
 
             if data:
                 with open('%s.img' % imgfile, 'w') as f:
@@ -452,7 +449,7 @@ class ReplicationCommandsTestCase(test_utils.BaseTestCase):
         # A file which should be ignored
         badfile = os.path.join(tempdir, 'kjdfhf')
         with open(badfile, 'w') as f:
-            f.write(json.dumps([1, 2, 3, 4, 5]))
+            f.write(jsonutils.dumps([1, 2, 3, 4, 5]))
 
         # Finally, we're ready to test
         options = UserDict.UserDict()
@@ -551,7 +548,7 @@ class ReplicationUtilitiesTestCase(test_utils.BaseTestCase):
 
         d = {'image': {'status': 'active'}}
         glance_replicator._check_upload_response_headers({},
-                                                         json.dumps(d))
+                                                         jsonutils.dumps(d))
 
         self.assertRaises(
             glance_replicator.UploadException,
@@ -560,9 +557,9 @@ class ReplicationUtilitiesTestCase(test_utils.BaseTestCase):
     def test_image_present(self):
         client = FakeImageService(None, 'noauth')
         self.assertTrue(glance_replicator._image_present(
-                client, '5dcddce0-cba5-4f18-9cf4-9853c7b207a6'))
+            client, '5dcddce0-cba5-4f18-9cf4-9853c7b207a6'))
         self.assertFalse(glance_replicator._image_present(
-                client, uuid.uuid4()))
+            client, uuid.uuid4()))
 
     def test_dict_diff(self):
         a = {'a': 1, 'b': 2, 'c': 3}
