@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2010-2011 OpenStack, LLC
+# Copyright 2010-2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,18 +13,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from six.moves import xrange
 import stubout
 
 from glance.common import exception
 from glance import context
 from glance.db.sqlalchemy import api as db_api
-from glance.registry import configure_registry_client
-from glance.store import (delete_from_backend,
-                          safe_delete_from_backend)
-from glance.store.http import Store, MAX_REDIRECTS
+from glance.registry.client.v1.api import configure_registry_client
+from glance.store import delete_from_backend
+from glance.store.http import MAX_REDIRECTS
+from glance.store.http import Store
 from glance.store.location import get_location_from_uri
+from glance.store import safe_delete_from_backend
+from glance.tests import stubs as test_stubs
 from glance.tests.unit import base
-from glance.tests import utils, stubs as test_stubs
+from glance.tests import utils
 
 
 # The response stack is used to return designated responses in order;
@@ -184,6 +185,6 @@ class TestHttpStore(base.StoreClearingUnitTest):
         ctx = context.RequestContext()
         stub_out_registry_image_update(self.stubs)
         try:
-            safe_delete_from_backend(uri, ctx, 'image_id')
+            safe_delete_from_backend(ctx, uri, 'image_id')
         except exception.StoreDeleteNotSupported:
             self.fail('StoreDeleteNotSupported should be swallowed')
