@@ -1,10 +1,8 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011 OpenStack, LLC
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
-#    not use this s3 except in compliance with the License. You may obtain
+#    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
 #
 #         http://www.apache.org/licenses/LICENSE-2.0
@@ -18,30 +16,34 @@
 """Tests the S3 backend store"""
 
 import hashlib
-import StringIO
+import uuid
 
 import boto.s3.connection
+import six
 import stubout
 
 from glance.common import exception
-from glance.openstack.common import uuidutils
+from glance.openstack.common import units
+
 from glance.store.location import get_location_from_uri
 import glance.store.s3
-from glance.store.s3 import Store, get_s3_location
+from glance.store.s3 import get_s3_location
+from glance.store.s3 import Store
 from glance.store import UnsupportedBackend
 from glance.tests.unit import base
 
 
-FAKE_UUID = uuidutils.generate_uuid()
+FAKE_UUID = str(uuid.uuid4())
 
-FIVE_KB = (5 * 1024)
+FIVE_KB = 5 * units.Ki
 S3_CONF = {'verbose': True,
            'debug': True,
            'default_store': 's3',
            's3_store_access_key': 'user',
            's3_store_secret_key': 'key',
            's3_store_host': 'localhost:8080',
-           's3_store_bucket': 'glance'}
+           's3_store_bucket': 'glance',
+           'known_stores': ['glance.store.s3.Store']}
 
 
 # We stub out as little as possible to ensure that the code paths
@@ -79,7 +81,7 @@ def stub_out_s3(stubs):
             return checksum_hex, None
 
         def set_contents_from_file(self, fp, replace=False, **kwargs):
-            self.data = StringIO.StringIO()
+            self.data = six.StringIO()
             for bytes in fp:
                 self.data.write(bytes)
             self.size = self.data.len
@@ -121,7 +123,7 @@ def stub_out_s3(stubs):
     fixture_buckets = {'glance': FakeBucket('glance')}
     b = fixture_buckets['glance']
     k = b.new_key(FAKE_UUID)
-    k.set_contents_from_file(StringIO.StringIO("*" * FIVE_KB))
+    k.set_contents_from_file(six.StringIO("*" * FIVE_KB))
 
     def fake_connection_constructor(self, *args, **kwargs):
         host = kwargs.get('host')
@@ -188,8 +190,7 @@ class TestStore(base.StoreClearingUnitTest):
 
         def fake_S3Connection_init(*args, **kwargs):
             expected_cls = boto.s3.connection.OrdinaryCallingFormat
-            self.assertTrue(isinstance(kwargs.get('calling_format'),
-                                       expected_cls))
+            self.assertIsInstance(kwargs.get('calling_format'), expected_cls)
 
         self.stubs.Set(boto.s3.connection.S3Connection, '__init__',
                        fake_S3Connection_init)
@@ -203,8 +204,7 @@ class TestStore(base.StoreClearingUnitTest):
 
         def fake_S3Connection_init(*args, **kwargs):
             expected_cls = boto.s3.connection.SubdomainCallingFormat
-            self.assertTrue(isinstance(kwargs.get('calling_format'),
-                                       expected_cls))
+            self.assertIsInstance(kwargs.get('calling_format'), expected_cls)
 
         self.stubs.Set(boto.s3.connection.S3Connection, '__init__',
                        fake_S3Connection_init)
@@ -228,7 +228,7 @@ class TestStore(base.StoreClearingUnitTest):
 
     def test_add(self):
         """Test that we can add an image via the s3 backend"""
-        expected_image_id = uuidutils.generate_uuid()
+        expected_image_id = str(uuid.uuid4())
         expected_s3_size = FIVE_KB
         expected_s3_contents = "*" * expected_s3_size
         expected_checksum = hashlib.md5(expected_s3_contents).hexdigest()
@@ -238,25 +238,25 @@ class TestStore(base.StoreClearingUnitTest):
             S3_CONF['s3_store_host'],
             S3_CONF['s3_store_bucket'],
             expected_image_id)
-        image_s3 = StringIO.StringIO(expected_s3_contents)
+        image_s3 = six.StringIO(expected_s3_contents)
 
-        location, size, checksum = self.store.add(expected_image_id,
-                                                  image_s3,
-                                                  expected_s3_size)
+        location, size, checksum, _ = self.store.add(expected_image_id,
+                                                     image_s3,
+                                                     expected_s3_size)
 
-        self.assertEquals(expected_location, location)
-        self.assertEquals(expected_s3_size, size)
-        self.assertEquals(expected_checksum, checksum)
+        self.assertEqual(expected_location, location)
+        self.assertEqual(expected_s3_size, size)
+        self.assertEqual(expected_checksum, checksum)
 
         loc = get_location_from_uri(expected_location)
         (new_image_s3, new_image_size) = self.store.get(loc)
-        new_image_contents = StringIO.StringIO()
+        new_image_contents = six.StringIO()
         for chunk in new_image_s3:
             new_image_contents.write(chunk)
         new_image_s3_size = new_image_contents.len
 
-        self.assertEquals(expected_s3_contents, new_image_contents.getvalue())
-        self.assertEquals(expected_s3_size, new_image_s3_size)
+        self.assertEqual(expected_s3_contents, new_image_contents.getvalue())
+        self.assertEqual(expected_s3_size, new_image_s3_size)
 
     def test_add_host_variations(self):
         """
@@ -274,7 +274,7 @@ class TestStore(base.StoreClearingUnitTest):
                       'localhost',
                       'localhost:8080/v1']
         for variation in variations:
-            expected_image_id = uuidutils.generate_uuid()
+            expected_image_id = str(uuid.uuid4())
             expected_s3_size = FIVE_KB
             expected_s3_contents = "*" * expected_s3_size
             expected_checksum = hashlib.md5(expected_s3_contents).hexdigest()
@@ -286,32 +286,32 @@ class TestStore(base.StoreClearingUnitTest):
                 new_conf['s3_store_host'],
                 new_conf['s3_store_bucket'],
                 expected_image_id)
-            image_s3 = StringIO.StringIO(expected_s3_contents)
+            image_s3 = six.StringIO(expected_s3_contents)
 
             self.config(**new_conf)
             self.store = Store()
-            location, size, checksum = self.store.add(expected_image_id,
-                                                      image_s3,
-                                                      expected_s3_size)
+            location, size, checksum, _ = self.store.add(expected_image_id,
+                                                         image_s3,
+                                                         expected_s3_size)
 
-            self.assertEquals(expected_location, location)
-            self.assertEquals(expected_s3_size, size)
-            self.assertEquals(expected_checksum, checksum)
+            self.assertEqual(expected_location, location)
+            self.assertEqual(expected_s3_size, size)
+            self.assertEqual(expected_checksum, checksum)
 
             loc = get_location_from_uri(expected_location)
             (new_image_s3, new_image_size) = self.store.get(loc)
             new_image_contents = new_image_s3.getvalue()
             new_image_s3_size = len(new_image_s3)
 
-            self.assertEquals(expected_s3_contents, new_image_contents)
-            self.assertEquals(expected_s3_size, new_image_s3_size)
+            self.assertEqual(expected_s3_contents, new_image_contents)
+            self.assertEqual(expected_s3_size, new_image_s3_size)
 
     def test_add_already_existing(self):
         """
         Tests that adding an image with an existing identifier
         raises an appropriate exception
         """
-        image_s3 = StringIO.StringIO("nevergonnamakeit")
+        image_s3 = six.StringIO("nevergonnamakeit")
         self.assertRaises(exception.Duplicate,
                           self.store.add,
                           FAKE_UUID, image_s3, 0)
@@ -366,12 +366,12 @@ class TestStore(base.StoreClearingUnitTest):
         self.assertRaises(exception.NotFound, self.store.delete, loc)
 
     def _do_test_get_s3_location(self, host, loc):
-        self.assertEquals(get_s3_location(host), loc)
-        self.assertEquals(get_s3_location(host + ':80'), loc)
-        self.assertEquals(get_s3_location('http://' + host), loc)
-        self.assertEquals(get_s3_location('http://' + host + ':80'), loc)
-        self.assertEquals(get_s3_location('https://' + host), loc)
-        self.assertEquals(get_s3_location('https://' + host + ':80'), loc)
+        self.assertEqual(get_s3_location(host), loc)
+        self.assertEqual(get_s3_location(host + ':80'), loc)
+        self.assertEqual(get_s3_location('http://' + host), loc)
+        self.assertEqual(get_s3_location('http://' + host + ':80'), loc)
+        self.assertEqual(get_s3_location('https://' + host), loc)
+        self.assertEqual(get_s3_location('https://' + host + ':80'), loc)
 
     def test_get_s3_good_location(self):
         """
@@ -402,14 +402,14 @@ class TestStore(base.StoreClearingUnitTest):
 
     def test_calling_format_path(self):
         self.config(s3_store_bucket_url_format='path')
-        self.assertTrue(isinstance(glance.store.s3.get_calling_format(),
-                                   boto.s3.connection.OrdinaryCallingFormat))
+        self.assertIsInstance(glance.store.s3.get_calling_format(),
+                              boto.s3.connection.OrdinaryCallingFormat)
 
     def test_calling_format_subdomain(self):
         self.config(s3_store_bucket_url_format='subdomain')
-        self.assertTrue(isinstance(glance.store.s3.get_calling_format(),
-                                   boto.s3.connection.SubdomainCallingFormat))
+        self.assertIsInstance(glance.store.s3.get_calling_format(),
+                              boto.s3.connection.SubdomainCallingFormat)
 
     def test_calling_format_default(self):
-        self.assertTrue(isinstance(glance.store.s3.get_calling_format(),
-                                   boto.s3.connection.SubdomainCallingFormat))
+        self.assertIsInstance(glance.store.s3.get_calling_format(),
+                              boto.s3.connection.SubdomainCallingFormat)
