@@ -13,8 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import StringIO
-
+import six
 import stubout
 
 from glance.common import exception
@@ -55,7 +54,7 @@ class TestStore(base.StoreClearingUnitTest):
         self.assertRaises(exception.ImageSizeLimitExceeded,
                           self.store.add,
                           'fake_image_id',
-                          utils.LimitingReader(StringIO.StringIO('xx'), 1),
+                          utils.LimitingReader(six.StringIO('xx'), 1),
                           2)
         self.assertEqual([['list', '-r'], ['create'], ['delete']],
                          called_commands)
