@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011-2013 OpenStack, LLC
+# Copyright 2011-2013 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,10 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from glance.common import exception
+from glance import store
 from glance.store import base as store_base
 from glance.tests.unit import base as test_base
 
 
+class FakeUnconfigurableStoreDriver(store_base.Store):
+    def configure(self):
+        raise exception.BadStoreConfiguration("Unconfigurable store driver.")
+
+
 class TestStoreBase(test_base.StoreClearingUnitTest):
 
     def setUp(self):
@@ -32,16 +37,23 @@ class TestStoreBase(test_base.StoreClearingUnitTest):
 
         exc = Exception('error message')
         ret = store_base._exception_to_unicode(exc)
-        self.assertEqual(unicode, type(ret))
+        self.assertIsInstance(ret, unicode)
         self.assertEqual(ret, 'error message')
 
         exc = Exception('\xa5 error message')
         ret = store_base._exception_to_unicode(exc)
-        self.assertEqual(unicode, type(ret))
+        self.assertIsInstance(ret, unicode)
         self.assertEqual(ret, ' error message')
 
         exc = FakeException('\xa5 error message')
         ret = store_base._exception_to_unicode(exc)
-        self.assertEqual(unicode, type(ret))
+        self.assertIsInstance(ret, unicode)
         self.assertEqual(ret, _("Caught '%(exception)s' exception.") %
                          {'exception': 'FakeException'})
+
+    def test_create_store_exclude_unconfigurable_drivers(self):
+        self.config(known_stores=[
+            "glance.tests.unit.test_store_base.FakeUnconfigurableStoreDriver",
+            "glance.store.filesystem.Store"])
+        count = store.create_stores()
+        self.assertEqual(9, count)
