@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
-# Copyright 2011-2013 OpenStack, LLC
+# Copyright 2011-2013 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,6 +12,7 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import mock
 
 from glance.common import exception
 from glance import context
@@ -23,6 +22,7 @@ import glance.store.http
 import glance.store.location as location
 import glance.store.s3
 import glance.store.swift
+import glance.store.vmware_datastore
 from glance.tests.unit import base
 
 
@@ -30,6 +30,20 @@ class TestStoreLocation(base.StoreClearingUnitTest):
 
     def setUp(self):
         self.config(default_store='file')
+
+        # NOTE(flaper87): Each store should test
+        # this in their test suite.
+        self.config(known_stores=[
+            "glance.store.filesystem.Store",
+            "glance.store.http.Store",
+            "glance.store.rbd.Store",
+            "glance.store.s3.Store",
+            "glance.store.swift.Store",
+            "glance.store.sheepdog.Store",
+            "glance.store.cinder.Store",
+            "glance.store.gridfs.Store",
+            "glance.store.vmware_datastore.Store",
+        ])
         super(TestStoreLocation, self).setUp()
 
     def test_get_location_from_uri_back_to_uri(self):
@@ -52,8 +66,9 @@ class TestStoreLocation(base.StoreClearingUnitTest):
             'rbd://imagename',
             'rbd://fsid/pool/image/snap',
             'rbd://%2F/%2F/%2F/%2F',
-            'sheepdog://imagename',
+            'sheepdog://244e75f1-9c69-4167-9db7-1aa7d1973f6c',
             'cinder://12345678-9012-3455-6789-012345678901',
+            'vsphere://ip/folder/openstack_glance/2332298?dcPath=dc&dsName=ds',
         ]
 
         for uri in good_store_uris:
@@ -85,7 +100,7 @@ class TestStoreLocation(base.StoreClearingUnitTest):
         self.assertEqual(uri, loc.get_uri())
 
         bad_uri = 'fil://'
-        self.assertRaises(Exception, loc.parse_uri, bad_uri)
+        self.assertRaises(AssertionError, loc.parse_uri, bad_uri)
 
         bad_uri = 'file://'
         self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
@@ -132,7 +147,7 @@ class TestStoreLocation(base.StoreClearingUnitTest):
         self.assertEqual(uri, loc.get_uri())
 
         bad_uri = 'htt://'
-        self.assertRaises(Exception, loc.parse_uri, bad_uri)
+        self.assertRaises(AssertionError, loc.parse_uri, bad_uri)
 
         bad_uri = 'http://'
         self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
@@ -153,7 +168,7 @@ class TestStoreLocation(base.StoreClearingUnitTest):
         self.assertEqual("https://example.com", loc.swift_url)
         self.assertEqual("images", loc.container)
         self.assertEqual("1", loc.obj)
-        self.assertEqual(None, loc.user)
+        self.assertIsNone(loc.user)
         self.assertEqual(uri, loc.get_uri())
 
         uri = 'swift+https://user:pass@authurl.com/images/1'
@@ -202,12 +217,12 @@ class TestStoreLocation(base.StoreClearingUnitTest):
         self.assertEqual("http://storeurl.com/v1", loc.swift_url)
         self.assertEqual("container", loc.container)
         self.assertEqual("12345", loc.obj)
-        self.assertEqual(None, loc.user)
-        self.assertEqual(None, loc.key)
+        self.assertIsNone(loc.user)
+        self.assertIsNone(loc.key)
         self.assertEqual(uri, loc.get_uri())
 
         bad_uri = 'swif://'
-        self.assertRaises(Exception, loc.parse_uri, bad_uri)
+        self.assertRaises(AssertionError, loc.parse_uri, bad_uri)
 
         bad_uri = 'swift://'
         self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
@@ -230,7 +245,7 @@ class TestStoreLocation(base.StoreClearingUnitTest):
         self.assertEqual("example.com", loc.s3serviceurl)
         self.assertEqual("images", loc.bucket)
         self.assertEqual("1", loc.key)
-        self.assertEqual(None, loc.accesskey)
+        self.assertIsNone(loc.accesskey)
         self.assertEqual(uri, loc.get_uri())
 
         uri = 's3+https://accesskey:pass@s3serviceurl.com/images/1'
@@ -267,7 +282,7 @@ class TestStoreLocation(base.StoreClearingUnitTest):
         self.assertEqual(uri, loc.get_uri())
 
         bad_uri = 's://'
-        self.assertRaises(Exception, loc.parse_uri, bad_uri)
+        self.assertRaises(AssertionError, loc.parse_uri, bad_uri)
 
         bad_uri = 's3://'
         self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
@@ -287,18 +302,18 @@ class TestStoreLocation(base.StoreClearingUnitTest):
         loc.parse_uri(uri)
 
         self.assertEqual('imagename', loc.image)
-        self.assertEqual(None, loc.fsid)
-        self.assertEqual(None, loc.pool)
-        self.assertEqual(None, loc.snapshot)
+        self.assertIsNone(loc.fsid)
+        self.assertIsNone(loc.pool)
+        self.assertIsNone(loc.snapshot)
 
         uri = u'rbd://imagename'
         loc = glance.store.rbd.StoreLocation({})
         loc.parse_uri(uri)
 
         self.assertEqual('imagename', loc.image)
-        self.assertEqual(None, loc.fsid)
-        self.assertEqual(None, loc.pool)
-        self.assertEqual(None, loc.snapshot)
+        self.assertIsNone(loc.fsid)
+        self.assertIsNone(loc.pool)
+        self.assertIsNone(loc.snapshot)
 
         uri = 'rbd://fsid/pool/image/snap'
         loc = glance.store.rbd.StoreLocation({})
@@ -367,15 +382,59 @@ class TestStoreLocation(base.StoreClearingUnitTest):
         """
         Test the specific StoreLocation for the Sheepdog store
         """
-        uri = 'sheepdog://imagename'
+        uri = 'sheepdog://244e75f1-9c69-4167-9db7-1aa7d1973f6c'
         loc = glance.store.sheepdog.StoreLocation({})
         loc.parse_uri(uri)
-        self.assertEqual('imagename', loc.image)
+        self.assertEqual('244e75f1-9c69-4167-9db7-1aa7d1973f6c', loc.image)
 
-        bad_uri = 'sheepdog:/image'
+        bad_uri = 'sheepdog:/244e75f1-9c69-4167-9db7-1aa7d1973f6c'
         self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
 
-        bad_uri = 'http://image'
+        bad_uri = 'http://244e75f1-9c69-4167-9db7-1aa7d1973f6c'
+        self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
+
+        bad_uri = 'image; name'
+        self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
+
+    def test_vmware_store_location(self):
+        """
+        Test the specific StoreLocation for the VMware store
+        """
+        ds_url_prefix = glance.store.vmware_datastore.DS_URL_PREFIX
+        image_dir = glance.store.vmware_datastore.DEFAULT_STORE_IMAGE_DIR
+        uri = ('vsphere://127.0.0.1%s%s/29038321?dcPath=my-dc&dsName=my-ds' %
+               (ds_url_prefix, image_dir))
+        loc = glance.store.vmware_datastore.StoreLocation({})
+        loc.parse_uri(uri)
+
+        self.assertEqual("vsphere", loc.scheme)
+        self.assertEqual("127.0.0.1", loc.server_host)
+        self.assertEqual("%s%s/29038321" %
+                         (ds_url_prefix, image_dir), loc.path)
+        self.assertEqual("dcPath=my-dc&dsName=my-ds", loc.query)
+        self.assertEqual(uri, loc.get_uri())
+
+        bad_uri = 'vphere://'
+        self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
+
+        bad_uri = ('vspheer://127.0.0.1%s%s/29038321?dcPath=my-dc&dsName=my-ds'
+                   % (ds_url_prefix, image_dir))
+        self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
+
+        bad_uri = ('http://127.0.0.1%s%s/29038321?dcPath=my-dc&dsName=my-ds'
+                   % (ds_url_prefix, image_dir))
+        self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
+
+        bad_uri = ('vsphere:/127.0.0.1%s%s/29038321?dcPath=my-dc&dsName=my-ds'
+                   % (ds_url_prefix, image_dir))
+        self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
+
+        bad_uri = ('vsphere://127.0.0.1%s%s/29038321?dcPath=my-dc&dsName=my-ds'
+                   % (ds_url_prefix, "/folder_not_in_configuration"))
+        self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
+
+        bad_uri = ('vsphere://127.0.0.1%s%s/29038321?dcPath=my-dc&dsName=my-ds'
+                   % ("/wrong_folder_path", image_dir))
         self.assertRaises(exception.BadStoreUri, loc.parse_uri, bad_uri)
 
     def test_cinder_store_good_location(self):
@@ -413,7 +472,8 @@ class TestStoreLocation(base.StoreClearingUnitTest):
             'https': glance.store.http.Store,
             'rbd': glance.store.rbd.Store,
             'sheepdog': glance.store.sheepdog.Store,
-            'cinder': glance.store.cinder.Store}
+            'cinder': glance.store.cinder.Store,
+            'vsphere': glance.store.vmware_datastore.Store}
 
         ctx = context.RequestContext()
         for scheme, store in good_results.items():
@@ -427,3 +487,31 @@ class TestStoreLocation(base.StoreClearingUnitTest):
                               glance.store.get_store_from_scheme,
                               ctx,
                               store)
+
+    def test_add_location_for_image_without_size(self):
+        class FakeImageProxy():
+            size = None
+            context = None
+            store_api = mock.Mock()
+
+        def fake_get_size_from_backend(context, uri):
+            return 1
+
+        self.stubs.Set(glance.store, 'get_size_from_backend',
+                       fake_get_size_from_backend)
+        with mock.patch('glance.store._check_image_location'):
+            loc1 = {'url': 'file:///fake1.img.tar.gz', 'metadata': {}}
+            loc2 = {'url': 'file:///fake2.img.tar.gz', 'metadata': {}}
+
+            # Test for insert location
+            image1 = FakeImageProxy()
+            locations = glance.store.StoreLocations(image1, [])
+            locations.insert(0, loc2)
+            self.assertEqual(image1.size, 1)
+
+            # Test for set_attr of _locations_proxy
+            image2 = FakeImageProxy()
+            locations = glance.store.StoreLocations(image2, [loc1])
+            locations[0] = loc2
+            self.assertTrue(loc2 in locations)
+            self.assertEqual(image2.size, 1)
