@@ -13,9 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import urllib
-import urlparse
-
 from oslo.config import cfg
 
 from glance.common import exception
@@ -39,27 +36,7 @@ USER1 = '54492ba0-f4df-4e4e-be62-27f4d76b29cf'
 USER2 = '0b3b3006-cb76-4517-ae32-51397e22c754'
 USER3 = '2hss8dkl-d8jh-88yd-uhs9-879sdjsd8skd'
 
-BASE_URI = 'swift+http://storeurl.com/container'
-
-
-def sort_url_by_qs_keys(url):
-    #NOTE(kragniz): this only sorts the keys of the query string of a url.
-    # For example, an input of '/v2/tasks?sort_key=id&sort_dir=asc&limit=10'
-    # returns '/v2/tasks?limit=10&sort_dir=asc&sort_key=id'. This is to prevent
-    # non-deterministic ordering of the query string causing problems with unit
-    # tests.
-
-    parsed = urlparse.urlparse(url)
-    queries = urlparse.parse_qsl(parsed.query, True)
-    sorted_query = sorted(queries, key=lambda x: x[0])
-
-    encoded_sorted_query = urllib.urlencode(sorted_query, True)
-
-    url_parts = (parsed.scheme, parsed.netloc, parsed.path,
-                 parsed.params, encoded_sorted_query,
-                 parsed.fragment)
-
-    return urlparse.urlunparse(url_parts)
+BASE_URI = 'http://storeurl.com/container'
 
 
 def get_fake_request(path='', method='POST', is_admin=False, user=USER1,
@@ -78,6 +55,10 @@ def get_fake_request(path='', method='POST', is_admin=False, user=USER1,
     return req
 
 
+def fake_get_size_from_backend(context, uri):
+    return 1
+
+
 class FakeDB(object):
 
     def __init__(self):
@@ -104,12 +85,7 @@ class FakeDB(object):
 
     @staticmethod
     def reset():
-        simple_db.DATA = {
-            'images': {},
-            'members': [],
-            'tags': {},
-            'locations': [],
-        }
+        simple_db.reset()
 
     def __getattr__(self, key):
         return getattr(simple_db, key)
@@ -172,10 +148,7 @@ class FakeStoreAPI(object):
             if image_id in location:
                 raise exception.Duplicate()
         if not size:
-            # 'data' is a string wrapped in a LimitingReader|CooperativeReader
-            # pipeline, so peek under the hood of those objects to get at the
-            # string itself.
-            size = len(data.data.fd)
+            size = len(data.fd)
         if (current_store_size + size) > store_max_size:
             raise exception.StorageFull()
         if context.user == USER2:
@@ -207,26 +180,70 @@ class FakeNotifier(object):
     def __init__(self, *_args, **kwargs):
         self.log = []
 
-    def warn(self, event_type, payload):
+    def _notify(self, event_type, payload, level):
         log = {}
-        log['notification_type'] = "WARN"
+        log['notification_type'] = level
         log['event_type'] = event_type
         log['payload'] = payload
         self.log.append(log)
 
+    def warn(self, event_type, payload):
+        self._notify(event_type, payload, 'WARN')
+
     def info(self, event_type, payload):
-        log = {}
-        log['notification_type'] = "INFO"
-        log['event_type'] = event_type
-        log['payload'] = payload
-        self.log.append(log)
+        self._notify(event_type, payload, 'INFO')
 
     def error(self, event_type, payload):
-        log = {}
-        log['notification_type'] = "ERROR"
-        log['event_type'] = event_type
-        log['payload'] = payload
-        self.log.append(log)
+        self._notify(event_type, payload, 'ERROR')
+
+    def debug(self, event_type, payload):
+        self._notify(event_type, payload, 'DEBUG')
+
+    def critical(self, event_type, payload):
+        self._notify(event_type, payload, 'CRITICAL')
 
     def get_logs(self):
         return self.log
+
+
+class FakeGateway(object):
+    def __init__(self, image_factory=None, image_member_factory=None,
+                 image_repo=None, task_factory=None, task_repo=None):
+        self.image_factory = image_factory
+        self.image_member_factory = image_member_factory
+        self.image_repo = image_repo
+        self.task_factory = task_factory
+        self.task_repo = task_repo
+
+    def get_image_factory(self, context):
+        return self.image_factory
+
+    def get_image_member_factory(self, context):
+        return self.image_member_factory
+
+    def get_repo(self, context):
+        return self.image_repo
+
+    def get_task_factory(self, context):
+        return self.task_factory
+
+    def get_task_repo(self, context):
+        return self.task_repo
+
+
+class FakeTask(object):
+    def __init__(self, task_id, type=None, status=None):
+        self.task_id = task_id
+        self.type = type
+        self.message = None
+        self.input = None
+        self._status = status
+        self._executor = None
+
+    def success(self, result):
+        self.result = result
+        self._status = 'success'
+
+    def fail(self, message):
+        self.message = message
+        self._status = 'failure'
