@@ -13,13 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import StringIO
+import mock
+import uuid
 
+import six
 import webob
 
 import glance.api.v2.image_data
 from glance.common import exception
-from glance.openstack.common import uuidutils
+
 from glance.tests.unit import base
 import glance.tests.unit.utils as unit_test_utils
 import glance.tests.utils as test_utils
@@ -35,12 +37,13 @@ class Raise(object):
 
 class FakeImage(object):
     def __init__(self, image_id=None, data=None, checksum=None, size=0,
-                 locations=None, container_format='bear', disk_format='rawr',
-                 status=None):
+                 virtual_size=0, locations=None, container_format='bear',
+                 disk_format='rawr', status=None):
         self.image_id = image_id
         self.data = data
         self.checksum = checksum
         self.size = size
+        self.virtual_size = virtual_size
         self.locations = locations
         self.container_format = container_format
         self.disk_format = disk_format
@@ -96,7 +99,7 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.image_repo = FakeImageRepo()
         self.gateway = FakeGateway(self.image_repo)
         self.controller = glance.api.v2.image_data.ImageDataController(
-                gateway=self.gateway)
+            gateway=self.gateway)
 
     def test_download(self):
         request = unit_test_utils.get_fake_request()
@@ -108,20 +111,20 @@ class TestImagesController(base.StoreClearingUnitTest):
     def test_download_no_location(self):
         request = unit_test_utils.get_fake_request()
         self.image_repo.result = FakeImage('abcd')
-        self.assertRaises(webob.exc.HTTPNotFound, self.controller.download,
+        self.assertRaises(webob.exc.HTTPNoContent, self.controller.download,
                           request, unit_test_utils.UUID2)
 
     def test_download_non_existent_image(self):
         request = unit_test_utils.get_fake_request()
         self.image_repo.result = exception.NotFound()
         self.assertRaises(webob.exc.HTTPNotFound, self.controller.download,
-                          request, uuidutils.generate_uuid())
+                          request, str(uuid.uuid4()))
 
     def test_download_forbidden(self):
         request = unit_test_utils.get_fake_request()
         self.image_repo.result = exception.Forbidden()
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.download,
-                          request, uuidutils.generate_uuid())
+                          request, str(uuid.uuid4()))
 
     def test_download_get_image_location_forbidden(self):
         class ImageLocations(object):
@@ -133,7 +136,7 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.image_repo.result = image
         image.locations = ImageLocations()
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.download,
-                          request, uuidutils.generate_uuid())
+                          request, str(uuid.uuid4()))
 
     def test_upload(self):
         request = unit_test_utils.get_fake_request()
@@ -166,7 +169,7 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.image_repo.result = image
         self.controller.upload(request, unit_test_utils.UUID2, 'YYYY', None)
         self.assertEqual(image.data, 'YYYY')
-        self.assertEqual(image.size, None)
+        self.assertIsNone(image.size)
 
     def test_upload_invalid(self):
         request = unit_test_utils.get_fake_request()
@@ -176,7 +179,7 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.upload,
                           request, unit_test_utils.UUID1, 'YYYY', 4)
 
-    def test_upload_non_existent_image_during_save(self):
+    def test_upload_non_existent_image_during_save_initiates_deletion(self):
         def fake_save(self):
             raise exception.NotFound()
 
@@ -184,19 +187,23 @@ class TestImagesController(base.StoreClearingUnitTest):
         image = FakeImage('abcd', locations=['http://example.com/image'])
         self.image_repo.result = image
         self.image_repo.save = fake_save
+        image.delete = mock.Mock()
         self.assertRaises(webob.exc.HTTPGone, self.controller.upload,
-                          request, uuidutils.generate_uuid(), 'ABC', 3)
+                          request, str(uuid.uuid4()), 'ABC', 3)
+        self.assertTrue(image.delete.called)
 
     def test_upload_non_existent_image_before_save(self):
         request = unit_test_utils.get_fake_request()
         self.image_repo.result = exception.NotFound()
         self.assertRaises(webob.exc.HTTPNotFound, self.controller.upload,
-                          request, uuidutils.generate_uuid(), 'ABC', 3)
+                          request, str(uuid.uuid4()), 'ABC', 3)
 
     def test_upload_data_exists(self):
         request = unit_test_utils.get_fake_request()
         image = FakeImage()
-        image.set_data = Raise(exception.Duplicate)
+        exc = exception.InvalidImageStatusTransition(cur_status='active',
+                                                     new_status='queued')
+        image.set_data = Raise(exc)
         self.image_repo.result = image
         self.assertRaises(webob.exc.HTTPConflict, self.controller.upload,
                           request, unit_test_utils.UUID1, 'YYYY', 4)
@@ -210,6 +217,15 @@ class TestImagesController(base.StoreClearingUnitTest):
                           self.controller.upload,
                           request, unit_test_utils.UUID2, 'YYYYYYY', 7)
 
+    def test_image_size_limit_exceeded(self):
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage()
+        image.set_data = Raise(exception.ImageSizeLimitExceeded)
+        self.image_repo.result = image
+        self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
+                          self.controller.upload,
+                          request, unit_test_utils.UUID1, 'YYYYYYY', 7)
+
     def test_upload_storage_forbidden(self):
         request = unit_test_utils.get_fake_request(user=unit_test_utils.USER2)
         image = FakeImage()
@@ -235,6 +251,7 @@ class TestImagesController(base.StoreClearingUnitTest):
         prepare_payload = output['meta'].copy()
         prepare_payload['checksum'] = None
         prepare_payload['size'] = None
+        prepare_payload['virtual_size'] = None
         prepare_payload['location'] = None
         prepare_payload['status'] = 'queued'
         del prepare_payload['updated_at']
@@ -277,6 +294,16 @@ class TestImagesController(base.StoreClearingUnitTest):
         self.assertEqual(len(output_log), 3)
         self.assertEqual(output_log[2], activate_log)
 
+    def test_restore_image_when_upload_failed(self):
+        request = unit_test_utils.get_fake_request()
+        image = FakeImage('fake')
+        image.set_data = Raise(exception.StorageWriteDenied)
+        self.image_repo.result = image
+        self.assertRaises(webob.exc.HTTPServiceUnavailable,
+                          self.controller.upload,
+                          request, unit_test_utils.UUID2, 'ZZZ', 3)
+        self.assertEqual(self.image_repo.saved_image.status, 'queued')
+
 
 class TestImageDataDeserializer(test_utils.BaseTestCase):
 
@@ -300,7 +327,7 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
         request.headers['Content-Type'] = 'application/octet-stream'
         # If we use body_file, webob assumes we want to do a chunked upload,
         # ignoring the Content-Length header
-        request.body_file = StringIO.StringIO('YYY')
+        request.body_file = six.StringIO('YYY')
         output = self.deserializer.upload(request)
         data = output.pop('data')
         self.assertEqual(data.read(), 'YYY')
@@ -310,7 +337,7 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
     def test_upload_chunked_with_content_length(self):
         request = unit_test_utils.get_fake_request()
         request.headers['Content-Type'] = 'application/octet-stream'
-        request.body_file = StringIO.StringIO('YYY')
+        request.body_file = six.StringIO('YYY')
         # The deserializer shouldn't care if the Content-Length is
         # set when the user is attempting to send chunked data.
         request.headers['Content-Length'] = 3
@@ -341,6 +368,12 @@ class TestImageDataDeserializer(test_utils.BaseTestCase):
         self.assertRaises(webob.exc.HTTPUnsupportedMediaType,
                           self.deserializer.upload, request)
 
+        request = unit_test_utils.get_fake_request()
+        request.headers['Content-Type'] = 'application/octet-st'
+        request.body = 'YYYYY'
+        self.assertRaises(webob.exc.HTTPUnsupportedMediaType,
+                          self.deserializer.upload, request)
+
 
 class TestImageDataSerializer(test_utils.BaseTestCase):
 
