@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 # -*- coding: utf-8 -*-
 
 # Copyright 2013 Red Hat, Inc.
@@ -17,7 +16,7 @@
 #    under the License.
 
 import datetime
-import json
+import uuid
 
 from oslo.config import cfg
 import routes
@@ -25,12 +24,12 @@ import webob
 
 import glance.api.common
 import glance.common.config
-from glance.common import exception
 import glance.context
 from glance.db.sqlalchemy import api as db_api
 from glance.db.sqlalchemy import models as db_models
+from glance.openstack.common import jsonutils
 from glance.openstack.common import timeutils
-from glance.openstack.common import uuidutils
+
 from glance.registry.api import v2 as rserver
 import glance.store.filesystem
 from glance.tests.unit import base
@@ -38,7 +37,7 @@ from glance.tests import utils as test_utils
 
 CONF = cfg.CONF
 
-_gen_uuid = uuidutils.generate_uuid
+_gen_uuid = lambda: str(uuid.uuid4())
 
 UUID1 = _gen_uuid()
 UUID2 = _gen_uuid()
@@ -92,7 +91,6 @@ class TestRegistryRPC(base.IsolatedUnitTest):
              'properties': {}}]
 
         self.context = glance.context.RequestContext(is_admin=True)
-        db_api.setup_db_env()
         db_api.get_engine()
         self.destroy_fixtures()
         self.create_fixtures()
@@ -112,8 +110,8 @@ class TestRegistryRPC(base.IsolatedUnitTest):
 
     def destroy_fixtures(self):
         # Easiest to just drop the models and re-create them...
-        db_models.unregister_models(db_api._ENGINE)
-        db_models.register_models(db_api._ENGINE)
+        db_models.unregister_models(db_api.get_engine())
+        db_models.register_models(db_api.get_engine())
 
     def test_show(self):
         """
@@ -132,13 +130,13 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get',
             'kwargs': {'image_id': UUID2},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        res_dict = json.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
         image = res_dict
         for k, v in fixture.iteritems():
-            self.assertEquals(v, image[k])
+            self.assertEqual(v, image[k])
 
     def test_show_unknown(self):
         """
@@ -151,11 +149,11 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get',
             'kwargs': {'image_id': _gen_uuid()},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        res_dict = json.loads(res.body)[0]
-        self.assertEquals(res_dict["_error"]["cls"],
-                          'glance.common.exception.NotFound')
+        res_dict = jsonutils.loads(res.body)[0]
+        self.assertEqual(res_dict["_error"]["cls"],
+                         'glance.common.exception.NotFound')
 
     def test_get_index(self):
         """
@@ -173,16 +171,16 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'filters': fixture},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        images = json.loads(res.body)[0]
-        self.assertEquals(len(images), 1)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 1)
 
         for k, v in fixture.iteritems():
-            self.assertEquals(v, images[0][k])
+            self.assertEqual(v, images[0][k])
 
     def test_get_index_marker(self):
         """
@@ -241,17 +239,17 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'marker': UUID4, "is_public": True},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        images = json.loads(res.body)[0]
+        images = jsonutils.loads(res.body)[0]
         # should be sorted by created_at desc, id desc
         # page should start after marker 4
-        self.assertEquals(len(images), 2)
-        self.assertEquals(images[0]['id'], UUID5)
-        self.assertEquals(images[1]['id'], UUID2)
+        self.assertEqual(len(images), 2)
+        self.assertEqual(images[0]['id'], UUID5)
+        self.assertEqual(images[1]['id'], UUID2)
 
     def test_get_index_marker_and_name_asc(self):
         """Test marker and null name ascending
@@ -279,12 +277,12 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'marker': UUID3, 'sort_key': 'name',
                        'sort_dir': 'asc'},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        images = json.loads(res.body)[0]
-        self.assertEquals(len(images), 2)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 2)
 
     def test_get_index_marker_and_name_desc(self):
         """Test marker and null name descending
@@ -312,12 +310,12 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'marker': UUID3, 'sort_key': 'name',
                        'sort_dir': 'desc'},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        images = json.loads(res.body)[0]
-        self.assertEquals(len(images), 0)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 0)
 
     def test_get_index_marker_and_disk_format_asc(self):
         """Test marker and null disk format ascending
@@ -345,12 +343,12 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'marker': UUID3, 'sort_key': 'disk_format',
                        'sort_dir': 'asc'},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        images = json.loads(res.body)[0]
-        self.assertEquals(len(images), 2)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 2)
 
     def test_get_index_marker_and_disk_format_desc(self):
         """Test marker and null disk format descending
@@ -378,12 +376,12 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'marker': UUID3, 'sort_key': 'disk_format',
                        'sort_dir': 'desc'},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        images = json.loads(res.body)[0]
-        self.assertEquals(len(images), 0)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 0)
 
     def test_get_index_marker_and_container_format_asc(self):
         """Test marker and null container format ascending
@@ -411,12 +409,12 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'marker': UUID3, 'sort_key': 'container_format',
                        'sort_dir': 'asc'},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        images = json.loads(res.body)[0]
-        self.assertEquals(len(images), 2)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 2)
 
     def test_get_index_marker_and_container_format_desc(self):
         """Test marker and null container format descending
@@ -444,12 +442,12 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'marker': UUID3, 'sort_key': 'container_format',
                        'sort_dir': 'desc'},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        images = json.loads(res.body)[0]
-        self.assertEquals(len(images), 0)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 0)
 
     def test_get_index_unknown_marker(self):
         """
@@ -462,9 +460,9 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'marker': _gen_uuid()},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        result = json.loads(res.body)[0]
+        result = jsonutils.loads(res.body)[0]
 
         self.assertIn("_error", result)
         self.assertIn("NotFound", result["_error"]["cls"])
@@ -511,16 +509,16 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'limit': 1},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        res_dict = json.loads(res.body)[0]
-        self.assertEquals(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
 
         images = res_dict
-        self.assertEquals(len(images), 1)
+        self.assertEqual(len(images), 1)
 
         # expect list to be sorted by created_at desc
-        self.assertTrue(images[0]['id'], UUID4)
+        self.assertEqual(images[0]['id'], UUID4)
 
     def test_get_index_limit_marker(self):
         """
@@ -563,13 +561,13 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'marker': UUID3, 'limit': 1},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        res_dict = json.loads(res.body)[0]
-        self.assertEquals(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
 
         images = res_dict
-        self.assertEquals(len(images), 1)
+        self.assertEqual(len(images), 1)
 
         # expect list to be sorted by created_at desc
         self.assertEqual(images[0]['id'], UUID2)
@@ -608,17 +606,127 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'filters': {'name': 'new name! #123'}},
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        res_dict = json.loads(res.body)[0]
-        self.assertEquals(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
 
         images = res_dict
-        self.assertEquals(len(images), 2)
+        self.assertEqual(len(images), 2)
 
         for image in images:
             self.assertEqual('new name! #123', image['name'])
 
+    def test_get_index_filter_on_user_defined_properties(self):
+        """
+        Tests that the registry API returns list of
+        public images that have a specific user-defined properties.
+        """
+        properties = {'distro': 'ubuntu', 'arch': 'i386', 'type': 'kernel'}
+        extra_id = _gen_uuid()
+        extra_fixture = {'id': extra_id,
+                         'status': 'active',
+                         'is_public': True,
+                         'disk_format': 'vhd',
+                         'container_format': 'ovf',
+                         'name': 'image-extra-1',
+                         'size': 19, 'properties': properties,
+                         'checksum': None}
+        db_api.image_create(self.context, extra_fixture)
+
+        # testing with a common property.
+        req = webob.Request.blank('/rpc')
+        req.method = "POST"
+        cmd = [{
+            'command': 'image_get_all',
+            'kwargs': {'filters': {'type': 'kernel'}},
+        }]
+        req.body = jsonutils.dumps(cmd)
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 2)
+        self.assertEqual(images[0]['id'], extra_id)
+        self.assertEqual(images[1]['id'], UUID1)
+
+        # testing with a non-existent value for a common property.
+        cmd = [{
+            'command': 'image_get_all',
+            'kwargs': {'filters': {'type': 'random'}},
+        }]
+        req.body = jsonutils.dumps(cmd)
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 0)
+
+        # testing with a non-existent value for a common property.
+        cmd = [{
+            'command': 'image_get_all',
+            'kwargs': {'filters': {'type': 'random'}},
+        }]
+        req.body = jsonutils.dumps(cmd)
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 0)
+
+        # testing with a non-existent property.
+        cmd = [{
+            'command': 'image_get_all',
+            'kwargs': {'filters': {'poo': 'random'}},
+        }]
+        req.body = jsonutils.dumps(cmd)
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 0)
+
+        # testing with multiple existing properties.
+        cmd = [{
+            'command': 'image_get_all',
+            'kwargs': {'filters': {'type': 'kernel', 'distro': 'ubuntu'}},
+        }]
+        req.body = jsonutils.dumps(cmd)
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 1)
+        self.assertEqual(images[0]['id'], extra_id)
+
+        # testing with multiple existing properties but non-existent values.
+        cmd = [{
+            'command': 'image_get_all',
+            'kwargs': {'filters': {'type': 'random', 'distro': 'random'}},
+        }]
+        req.body = jsonutils.dumps(cmd)
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 0)
+
+        # testing with multiple non-existing properties.
+        cmd = [{
+            'command': 'image_get_all',
+            'kwargs': {'filters': {'typo': 'random', 'poo': 'random'}},
+        }]
+        req.body = jsonutils.dumps(cmd)
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 0)
+
+        # testing with one existing property and the other non-existing.
+        cmd = [{
+            'command': 'image_get_all',
+            'kwargs': {'filters': {'type': 'kernel', 'poo': 'random'}},
+        }]
+        req.body = jsonutils.dumps(cmd)
+        res = req.get_response(self.api)
+        self.assertEqual(res.status_int, 200)
+        images = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(images), 0)
+
     def test_get_index_sort_default_created_at_desc(self):
         """
         Tests that the registry API returns list of
@@ -675,21 +783,21 @@ class TestRegistryRPC(base.IsolatedUnitTest):
         cmd = [{
             'command': 'image_get_all',
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        res_dict = json.loads(res.body)[0]
-        self.assertEquals(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
 
         images = res_dict
         # (flaper87)registry's v1 forced is_public to True
         # when no value was specified. This is not
         # the default behaviour anymore.
-        self.assertEquals(len(images), 5)
-        self.assertEquals(images[0]['id'], UUID3)
-        self.assertEquals(images[1]['id'], UUID4)
-        self.assertEquals(images[2]['id'], UUID5)
-        self.assertEquals(images[3]['id'], UUID2)
-        self.assertEquals(images[4]['id'], UUID1)
+        self.assertEqual(len(images), 5)
+        self.assertEqual(images[0]['id'], UUID3)
+        self.assertEqual(images[1]['id'], UUID4)
+        self.assertEqual(images[2]['id'], UUID5)
+        self.assertEqual(images[3]['id'], UUID2)
+        self.assertEqual(images[4]['id'], UUID1)
 
     def test_get_index_sort_name_asc(self):
         """
@@ -738,18 +846,18 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'sort_key': 'name', 'sort_dir': 'asc'}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        res_dict = json.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
 
         images = res_dict
-        self.assertEquals(len(images), 5)
-        self.assertEquals(images[0]['id'], UUID5)
-        self.assertEquals(images[1]['id'], UUID3)
-        self.assertEquals(images[2]['id'], UUID1)
-        self.assertEquals(images[3]['id'], UUID2)
-        self.assertEquals(images[4]['id'], UUID4)
+        self.assertEqual(len(images), 5)
+        self.assertEqual(images[0]['id'], UUID5)
+        self.assertEqual(images[1]['id'], UUID3)
+        self.assertEqual(images[2]['id'], UUID1)
+        self.assertEqual(images[3]['id'], UUID2)
+        self.assertEqual(images[4]['id'], UUID4)
 
     def test_get_index_sort_status_desc(self):
         """
@@ -791,17 +899,17 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'sort_key': 'status', 'sort_dir': 'asc'}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        res_dict = json.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
 
         images = res_dict
-        self.assertEquals(len(images), 4)
-        self.assertEquals(images[0]['id'], UUID1)
-        self.assertEquals(images[1]['id'], UUID2)
-        self.assertEquals(images[2]['id'], UUID4)
-        self.assertEquals(images[3]['id'], UUID3)
+        self.assertEqual(len(images), 4)
+        self.assertEqual(images[0]['id'], UUID1)
+        self.assertEqual(images[1]['id'], UUID2)
+        self.assertEqual(images[2]['id'], UUID4)
+        self.assertEqual(images[3]['id'], UUID3)
 
     def test_get_index_sort_disk_format_asc(self):
         """
@@ -843,17 +951,17 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'sort_key': 'disk_format', 'sort_dir': 'asc'}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        res_dict = json.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
 
         images = res_dict
-        self.assertEquals(len(images), 4)
-        self.assertEquals(images[0]['id'], UUID1)
-        self.assertEquals(images[1]['id'], UUID3)
-        self.assertEquals(images[2]['id'], UUID4)
-        self.assertEquals(images[3]['id'], UUID2)
+        self.assertEqual(len(images), 4)
+        self.assertEqual(images[0]['id'], UUID1)
+        self.assertEqual(images[1]['id'], UUID3)
+        self.assertEqual(images[2]['id'], UUID4)
+        self.assertEqual(images[3]['id'], UUID2)
 
     def test_get_index_sort_container_format_desc(self):
         """
@@ -896,17 +1004,17 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'sort_key': 'container_format',
                        'sort_dir': 'desc'}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        res_dict = json.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
 
         images = res_dict
-        self.assertEquals(len(images), 4)
-        self.assertEquals(images[0]['id'], UUID2)
-        self.assertEquals(images[1]['id'], UUID4)
-        self.assertEquals(images[2]['id'], UUID3)
-        self.assertEquals(images[3]['id'], UUID1)
+        self.assertEqual(len(images), 4)
+        self.assertEqual(images[0]['id'], UUID2)
+        self.assertEqual(images[1]['id'], UUID4)
+        self.assertEqual(images[2]['id'], UUID3)
+        self.assertEqual(images[3]['id'], UUID1)
 
     def test_get_index_sort_size_asc(self):
         """
@@ -944,17 +1052,17 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'sort_key': 'size',
                        'sort_dir': 'asc'}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        res_dict = json.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
 
         images = res_dict
-        self.assertEquals(len(images), 4)
-        self.assertEquals(images[0]['id'], UUID4)
-        self.assertEquals(images[1]['id'], UUID1)
-        self.assertEquals(images[2]['id'], UUID2)
-        self.assertEquals(images[3]['id'], UUID3)
+        self.assertEqual(len(images), 4)
+        self.assertEqual(images[0]['id'], UUID4)
+        self.assertEqual(images[1]['id'], UUID1)
+        self.assertEqual(images[2]['id'], UUID2)
+        self.assertEqual(images[3]['id'], UUID3)
 
     def test_get_index_sort_created_at_asc(self):
         """
@@ -999,17 +1107,17 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'sort_key': 'created_at',
                        'sort_dir': 'asc'}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        res_dict = json.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
 
         images = res_dict
-        self.assertEquals(len(images), 4)
-        self.assertEquals(images[0]['id'], UUID1)
-        self.assertEquals(images[1]['id'], UUID2)
-        self.assertEquals(images[2]['id'], UUID4)
-        self.assertEquals(images[3]['id'], UUID3)
+        self.assertEqual(len(images), 4)
+        self.assertEqual(images[0]['id'], UUID1)
+        self.assertEqual(images[1]['id'], UUID2)
+        self.assertEqual(images[2]['id'], UUID4)
+        self.assertEqual(images[3]['id'], UUID3)
 
     def test_get_index_sort_updated_at_desc(self):
         """
@@ -1054,17 +1162,17 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'sort_key': 'updated_at',
                        'sort_dir': 'desc'}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
-        res_dict = json.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
 
         images = res_dict
-        self.assertEquals(len(images), 4)
-        self.assertEquals(images[0]['id'], UUID3)
-        self.assertEquals(images[1]['id'], UUID4)
-        self.assertEquals(images[2]['id'], UUID2)
-        self.assertEquals(images[3]['id'], UUID1)
+        self.assertEqual(len(images), 4)
+        self.assertEqual(images[0]['id'], UUID3)
+        self.assertEqual(images[1]['id'], UUID4)
+        self.assertEqual(images[2]['id'], UUID2)
+        self.assertEqual(images[3]['id'], UUID1)
 
     def test_create_image(self):
         """Tests that the registry API creates the image"""
@@ -1080,18 +1188,18 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_create',
             'kwargs': {'values': fixture}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
 
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        res_dict = json.loads(res.body)[0]
+        res_dict = jsonutils.loads(res.body)[0]
 
         for k, v in fixture.iteritems():
-            self.assertEquals(v, res_dict[k])
+            self.assertEqual(v, res_dict[k])
 
         # Test status was updated properly
-        self.assertEquals('active', res_dict['status'])
+        self.assertEqual('active', res_dict['status'])
 
     def test_create_image_with_min_disk(self):
         """Tests that the registry API creates the image"""
@@ -1108,14 +1216,14 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_create',
             'kwargs': {'values': fixture}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        res_dict = json.loads(res.body)[0]
+        res_dict = jsonutils.loads(res.body)[0]
 
-        self.assertEquals(5, res_dict['min_disk'])
+        self.assertEqual(5, res_dict['min_disk'])
 
     def test_create_image_with_min_ram(self):
         """Tests that the registry API creates the image"""
@@ -1132,14 +1240,14 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_create',
             'kwargs': {'values': fixture}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        res_dict = json.loads(res.body)[0]
+        res_dict = jsonutils.loads(res.body)[0]
 
-        self.assertEquals(256, res_dict['min_ram'])
+        self.assertEqual(256, res_dict['min_ram'])
 
     def test_create_image_with_min_ram_default(self):
         """Tests that the registry API creates the image"""
@@ -1155,14 +1263,14 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_create',
             'kwargs': {'values': fixture}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        res_dict = json.loads(res.body)[0]
+        res_dict = jsonutils.loads(res.body)[0]
 
-        self.assertEquals(0, res_dict['min_ram'])
+        self.assertEqual(0, res_dict['min_ram'])
 
     def test_create_image_with_min_disk_default(self):
         """Tests that the registry API creates the image"""
@@ -1178,14 +1286,14 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_create',
             'kwargs': {'values': fixture}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        res_dict = json.loads(res.body)[0]
+        res_dict = jsonutils.loads(res.body)[0]
 
-        self.assertEquals(0, res_dict['min_disk'])
+        self.assertEqual(0, res_dict['min_disk'])
 
     def test_update_image(self):
         """Tests that the registry API updates the image"""
@@ -1201,18 +1309,18 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'kwargs': {'values': fixture,
                        'image_id': UUID2}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        res_dict = json.loads(res.body)[0]
+        res_dict = jsonutils.loads(res.body)[0]
 
-        self.assertNotEquals(res_dict['created_at'],
-                             res_dict['updated_at'])
+        self.assertNotEqual(res_dict['created_at'],
+                            res_dict['updated_at'])
 
         for k, v in fixture.iteritems():
-            self.assertEquals(v, res_dict[k])
+            self.assertEqual(v, res_dict[k])
 
     def test_delete_image(self):
         """Tests that the registry API deletes the image"""
@@ -1224,10 +1332,10 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_get_all',
             'kwargs': {'filters': {'deleted': False}}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        res_dict = json.loads(res.body)[0]
-        self.assertEquals(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
 
         orig_num_images = len(res_dict)
 
@@ -1236,23 +1344,23 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_destroy',
             'kwargs': {'image_id': UUID2}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
         # Verify one less image
         cmd = [{
             'command': 'image_get_all',
             'kwargs': {'filters': {'deleted': False}}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
-        res_dict = json.loads(res.body)[0]
-        self.assertEquals(res.status_int, 200)
+        res_dict = jsonutils.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
 
         new_num_images = len(res_dict)
-        self.assertEquals(new_num_images, orig_num_images - 1)
+        self.assertEqual(new_num_images, orig_num_images - 1)
 
     def test_delete_image_response(self):
         """Tests that the registry API delete returns the image metadata"""
@@ -1264,13 +1372,13 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_destroy',
             'kwargs': {'image_id': image['id']}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
         res = req.get_response(self.api)
 
-        self.assertEquals(res.status_int, 200)
-        deleted_image = json.loads(res.body)[0]
+        self.assertEqual(res.status_int, 200)
+        deleted_image = jsonutils.loads(res.body)[0]
 
-        self.assertEquals(image['id'], deleted_image['id'])
+        self.assertEqual(image['id'], deleted_image['id'])
         self.assertTrue(deleted_image['deleted'])
         self.assertTrue(deleted_image['deleted_at'])
 
@@ -1284,10 +1392,10 @@ class TestRegistryRPC(base.IsolatedUnitTest):
             'command': 'image_member_find',
             'kwargs': {'image_id': UUID2}
         }]
-        req.body = json.dumps(cmd)
+        req.body = jsonutils.dumps(cmd)
 
         res = req.get_response(self.api)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
-        memb_list = json.loads(res.body)[0]
-        self.assertEquals(len(memb_list), 0)
+        memb_list = jsonutils.loads(res.body)[0]
+        self.assertEqual(len(memb_list), 0)
