@@ -15,18 +15,14 @@
 #    under the License.
 
 import datetime
-import http.client as http
-from unittest import mock
 import uuid
 
-from oslo_config import cfg
-from oslo_serialization import jsonutils
-from oslo_utils import timeutils
 import webob
 
 import glance.api.v2.tasks
 import glance.domain
-import glance.gateway
+from glance.openstack.common import jsonutils
+from glance.openstack.common import timeutils
 from glance.tests.unit import base
 import glance.tests.unit.utils as unit_test_utils
 import glance.tests.utils as test_utils
@@ -54,11 +50,8 @@ def _db_fixture(task_id, **kwargs):
         'input': {},
         'result': None,
         'owner': None,
-        'image_id': 'fake_image_id',
-        'user_id': 'fake_user',
-        'request_id': 'fake_request_id',
         'message': None,
-        'expires_at': default_datetime + datetime.timedelta(days=365),
+        'expires_at': None,
         'created_at': default_datetime,
         'updated_at': default_datetime,
         'deleted_at': None,
@@ -74,30 +67,24 @@ def _domain_fixture(task_id, **kwargs):
         'task_id': task_id,
         'status': kwargs.get('status', 'pending'),
         'task_type': kwargs.get('type', 'import'),
-        'owner': kwargs.get('owner'),
-        'expires_at': kwargs.get('expires_at'),
+        'owner': kwargs.get('owner', None),
+        'expires_at': kwargs.get('expires_at', None),
         'created_at': kwargs.get('created_at', default_datetime),
         'updated_at': kwargs.get('updated_at', default_datetime),
-        'task_input': kwargs.get('task_input', {}),
-        'message': kwargs.get('message'),
-        'result': kwargs.get('result'),
-        'image_id': kwargs.get('image_id'),
-        'user_id': kwargs.get('user_id'),
-        'request_id': kwargs.get('request_id'),
     }
     task = glance.domain.Task(**task_properties)
-    return task
-
-
-CONF = cfg.CONF
-CONF.import_opt('task_time_to_live', 'glance.common.config', group='task')
+    task_details = glance.domain.TaskDetails(task_id,
+                                             kwargs.get('input', {}),
+                                             kwargs.get('message', None),
+                                             kwargs.get('result', None))
+    return {'task': task, 'task_details': task_details}
 
 
 class TestTasksController(test_utils.BaseTestCase):
 
     def setUp(self):
         super(TestTasksController, self).setUp()
-        self.db = unit_test_utils.FakeDB(initialize=False)
+        self.db = unit_test_utils.FakeDB()
         self.policy = unit_test_utils.FakePolicyEnforcer()
         self.notifier = unit_test_utils.FakeNotifier()
         self.store = unit_test_utils.FakeStoreAPI()
@@ -106,10 +93,9 @@ class TestTasksController(test_utils.BaseTestCase):
                                                               self.policy,
                                                               self.notifier,
                                                               self.store)
-        self.gateway = glance.gateway.Gateway(self.db, self.store,
-                                              self.notifier, self.policy)
 
     def _create_tasks(self):
+        self.db.reset()
         now = timeutils.utcnow()
         times = [now + datetime.timedelta(seconds=5 * i) for i in range(4)]
         self.tasks = [
@@ -132,7 +118,7 @@ class TestTasksController(test_utils.BaseTestCase):
         self.assertEqual(1, len(output['tasks']))
         actual = set([task.task_id for task in output['tasks']])
         expected = set([UUID1])
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
 
     def test_index_admin(self):
         request = unit_test_utils.get_fake_request(is_admin=True)
@@ -157,7 +143,7 @@ class TestTasksController(test_utils.BaseTestCase):
         self.assertEqual(2, len(output['tasks']))
         actual = set([task.task_id for task in output['tasks']])
         expected = set([UUID2, UUID1])
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
         self.assertEqual(UUID1, output['next_marker'])
 
     def test_index_no_next_marker(self):
@@ -167,8 +153,8 @@ class TestTasksController(test_utils.BaseTestCase):
         self.assertEqual(0, len(output['tasks']))
         actual = set([task.task_id for task in output['tasks']])
         expected = set([])
-        self.assertEqual(expected, actual)
-        self.assertNotIn('next_marker', output)
+        self.assertEqual(actual, expected)
+        self.assertTrue('next_marker' not in output)
 
     def test_index_with_id_filter(self):
         request = unit_test_utils.get_fake_request('/tasks?id=%s' % UUID1)
@@ -176,7 +162,7 @@ class TestTasksController(test_utils.BaseTestCase):
         self.assertEqual(1, len(output['tasks']))
         actual = set([task.task_id for task in output['tasks']])
         expected = set([UUID1])
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
 
     def test_index_with_filters_return_many(self):
         path = '/tasks?status=pending'
@@ -185,7 +171,7 @@ class TestTasksController(test_utils.BaseTestCase):
         self.assertEqual(4, len(output['tasks']))
         actual = set([task.task_id for task in output['tasks']])
         expected = set([UUID1, UUID2, UUID3, UUID4])
-        self.assertEqual(sorted(expected), sorted(actual))
+        self.assertEqual(sorted(actual), sorted(expected))
 
     def test_index_with_many_filters(self):
         url = '/tasks?status=pending&type=import'
@@ -199,7 +185,7 @@ class TestTasksController(test_utils.BaseTestCase):
         self.assertEqual(1, len(output['tasks']))
         actual = set([task.task_id for task in output['tasks']])
         expected = set([UUID1])
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
 
     def test_index_with_marker(self):
         self.config(limit_param_default=1, api_limit_max=3)
@@ -208,7 +194,7 @@ class TestTasksController(test_utils.BaseTestCase):
         output = self.controller.index(request, marker=UUID3)
         actual = set([task.task_id for task in output['tasks']])
         self.assertEqual(1, len(actual))
-        self.assertIn(UUID2, actual)
+        self.assertTrue(UUID2 in actual)
 
     def test_index_with_limit(self):
         path = '/tasks'
@@ -225,7 +211,7 @@ class TestTasksController(test_utils.BaseTestCase):
         output = self.controller.index(request, limit=4)
         actual = set([task.task_id for task in output['tasks']])
         self.assertEqual(3, len(actual))
-        self.assertNotIn(output['next_marker'], output)
+        self.assertTrue(output['next_marker'] not in output)
 
     def test_index_default_limit(self):
         self.config(limit_param_default=1, api_limit_max=3)
@@ -241,7 +227,7 @@ class TestTasksController(test_utils.BaseTestCase):
         output = self.controller.index(request, sort_dir='asc', limit=3)
         actual = [task.task_id for task in output['tasks']]
         self.assertEqual(3, len(actual))
-        self.assertEqual([UUID1, UUID2, UUID3], actual)
+        self.assertEqual(actual, [UUID1, UUID2, UUID3])
 
     def test_index_with_sort_key(self):
         path = '/tasks'
@@ -281,8 +267,11 @@ class TestTasksController(test_utils.BaseTestCase):
 
     def test_get(self):
         request = unit_test_utils.get_fake_request()
-        task = self.controller.get(request, task_id=UUID1)
+        output = self.controller.get(request, task_id=UUID1)
+        task = output['task']
+        task_details = output['task_details']
         self.assertEqual(UUID1, task.task_id)
+        self.assertEqual(UUID1, task_details.task_id)
         self.assertEqual('import', task.type)
 
     def test_get_non_existent(self):
@@ -293,176 +282,29 @@ class TestTasksController(test_utils.BaseTestCase):
 
     def test_get_not_allowed(self):
         request = unit_test_utils.get_fake_request()
-        self.assertEqual(TENANT1, request.context.project_id)
+        self.assertEqual(request.context.tenant, TENANT1)
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.controller.get, request, UUID4)
 
-    @mock.patch('glance.api.common.get_thread_pool')
-    @mock.patch.object(glance.gateway.Gateway, 'get_task_factory')
-    @mock.patch.object(glance.gateway.Gateway, 'get_task_executor_factory')
-    @mock.patch.object(glance.gateway.Gateway, 'get_task_repo')
-    def test_create(self, mock_get_task_repo, mock_get_task_executor_factory,
-                    mock_get_task_factory, mock_get_thread_pool):
-        # setup
+    def test_create(self):
         request = unit_test_utils.get_fake_request()
-        task = {
-            "type": "import",
-            "input": {
-                "import_from": "swift://cloud.foo/myaccount/mycontainer/path",
-                "import_from_format": "qcow2",
-                "image_properties": {}
-            }
-        }
-        get_task_factory = mock.Mock()
-        mock_get_task_factory.return_value = get_task_factory
-
-        new_task = mock.Mock()
-        get_task_factory.new_task.return_value = new_task
-
-        new_task.run.return_value = mock.ANY
-
-        get_task_executor_factory = mock.Mock()
-        mock_get_task_executor_factory.return_value = get_task_executor_factory
-        get_task_executor_factory.new_task_executor.return_value = mock.Mock()
-
-        get_task_repo = mock.Mock()
-        mock_get_task_repo.return_value = get_task_repo
-        get_task_repo.add.return_value = mock.Mock()
-
-        # call
-        self.controller.create(request, task=task)
-
-        # assert
-        self.assertEqual(1, get_task_factory.new_task.call_count)
-        self.assertEqual(1, get_task_repo.add.call_count)
-        self.assertEqual(
-            1, get_task_executor_factory.new_task_executor.call_count)
-
-        # Make sure that we spawned the task's run method
-        mock_get_thread_pool.assert_called_once_with('tasks_pool')
-        mock_get_thread_pool.return_value.spawn.assert_called_once_with(
-            new_task.run,
-            get_task_executor_factory.new_task_executor.return_value)
-
-    @mock.patch('glance.common.scripts.utils.get_image_data_iter')
-    @mock.patch('glance.common.scripts.utils.validate_location_uri')
-    def test_create_with_live_time(self, mock_validate_location_uri,
-                                   mock_get_image_data_iter):
-        self.skipTest("Something wrong, this test touches registry")
-        request = unit_test_utils.get_fake_request()
-        task = {
-            "type": "import",
-            "input": {
-                "import_from": "http://download.cirros-cloud.net/0.3.4/"
-                               "cirros-0.3.4-x86_64-disk.img",
-                "import_from_format": "qcow2",
-                "image_properties": {
-                    "disk_format": "qcow2",
-                    "container_format": "bare",
-                    "name": "test-task"
-                }
-            }
-        }
-
-        new_task = self.controller.create(request, task=task)
-        executor_factory = self.gateway.get_task_executor_factory(
-            request.context)
-        task_executor = executor_factory.new_task_executor(request.context)
-        task_executor.begin_processing(new_task.task_id)
-        success_task = self.controller.get(request, new_task.task_id)
-
-        # ignore second and microsecond to avoid flaky runs
-        task_live_time = (success_task.expires_at.replace(second=0,
-                                                          microsecond=0) -
-                          success_task.updated_at.replace(second=0,
-                                                          microsecond=0))
-        task_live_time_hour = (task_live_time.days * 24 +
-                               task_live_time.seconds / 3600)
-        self.assertEqual(CONF.task.task_time_to_live, task_live_time_hour)
-
-    def test_create_with_wrong_import_form(self):
-        request = unit_test_utils.get_fake_request()
-        wrong_import_from = [
-            "swift://cloud.foo/myaccount/mycontainer/path",
-            "file:///path",
-            "cinder://volume-id"
-        ]
-        executor_factory = self.gateway.get_task_executor_factory(
-            request.context)
-        task_repo = self.gateway.get_task_repo(request.context)
-
-        for import_from in wrong_import_from:
-            task = {
-                "type": "import",
-                "input": {
-                    "import_from": import_from,
-                    "import_from_format": "qcow2",
-                    "image_properties": {
-                        "disk_format": "qcow2",
-                        "container_format": "bare",
-                        "name": "test-task"
-                    }
-                }
-            }
-            new_task = self.controller.create(request, task=task)
-            task_executor = executor_factory.new_task_executor(request.context)
-            task_executor.begin_processing(new_task.task_id)
-            final_task = task_repo.get(new_task.task_id)
-
-            self.assertEqual('failure', final_task.status)
-            if import_from.startswith("file:///"):
-                msg = ("File based imports are not allowed. Please use a "
-                       "non-local source of image data.")
-            else:
-                supported = ['http', ]
-                msg = ("The given uri is not valid. Please specify a "
-                       "valid uri from the following list of supported uri "
-                       "%(supported)s") % {'supported': supported}
-            self.assertEqual(msg, final_task.message)
-
-    def test_create_with_properties_missed(self):
-        request = unit_test_utils.get_fake_request()
-        executor_factory = self.gateway.get_task_executor_factory(
-            request.context)
-        task_repo = self.gateway.get_task_repo(request.context)
-
-        task = {
-            "type": "import",
-            "input": {
-                "import_from": "swift://cloud.foo/myaccount/mycontainer/path",
-                "import_from_format": "qcow2",
-            }
-        }
-        new_task = self.controller.create(request, task=task)
-        task_executor = executor_factory.new_task_executor(request.context)
-        task_executor.begin_processing(new_task.task_id)
-        final_task = task_repo.get(new_task.task_id)
-
-        self.assertEqual('failure', final_task.status)
-        msg = "Input does not contain 'image_properties' field"
-        self.assertEqual(msg, final_task.message)
-
-    @mock.patch.object(glance.gateway.Gateway, 'get_task_factory')
-    def test_notifications_on_create(self, mock_get_task_factory):
-        request = unit_test_utils.get_fake_request()
-
-        new_task = mock.MagicMock(type='import')
-        mock_get_task_factory.new_task.return_value = new_task
-        new_task.run.return_value = mock.ANY
-
         task = {"type": "import", "input": {
-            "import_from": "http://cloud.foo/myaccount/mycontainer/path",
-            "import_from_format": "qcow2",
-            "image_properties": {}
+            "import_from": "swift://cloud.foo/myaccount/mycontainer/path",
+            "image_from_format": "qcow2"}
         }
-        }
-        task = self.controller.create(request, task=task)
+        output = self.controller.create(request, task=task)
+        task = output['task']
+        task_details = output['task_details']
+        self.assertEqual('import', task.type)
+        self.assertEqual({
+            "import_from": "swift://cloud.foo/myaccount/mycontainer/path",
+            "image_from_format": "qcow2"}, task_details.input)
         output_logs = [nlog for nlog in self.notifier.get_logs()
                        if nlog['event_type'] == 'task.create']
-        self.assertEqual(1, len(output_logs))
+        self.assertEqual(len(output_logs), 1)
         output_log = output_logs[0]
-        self.assertEqual('INFO', output_log['notification_type'])
-        self.assertEqual('task.create', output_log['event_type'])
+        self.assertEqual(output_log['notification_type'], 'INFO')
+        self.assertEqual(output_log['event_type'], 'task.create')
 
 
 class TestTasksControllerPolicies(base.IsolatedUnitTest):
@@ -474,14 +316,28 @@ class TestTasksControllerPolicies(base.IsolatedUnitTest):
         self.controller = glance.api.v2.tasks.TasksController(self.db,
                                                               self.policy)
 
-    def test_access_get_unauthorized(self):
-        rules = {"tasks_api_access": False,
-                 "get_task": True}
+    def test_index_unauthorized(self):
+        rules = {"get_tasks": False}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request()
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.index,
+                          request)
+
+    def test_get_unauthorized(self):
+        rules = {"get_task": False}
         self.policy.set_rules(rules)
         request = unit_test_utils.get_fake_request()
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.get,
                           request, task_id=UUID2)
 
+    def test_create_task_unauthorized(self):
+        rules = {"add_task": False}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request()
+        task = {'type': 'import', 'input': {"import_from": "fake"}}
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.create,
+                          request, task)
+
     def test_delete(self):
         request = unit_test_utils.get_fake_request()
         self.assertRaises(webob.exc.HTTPMethodNotAllowed,
@@ -489,80 +345,12 @@ class TestTasksControllerPolicies(base.IsolatedUnitTest):
                           request,
                           'fake_id')
 
-    def test_access_delete_unauthorized(self):
-        rules = {"tasks_api_access": False}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request()
-        self.assertRaises(webob.exc.HTTPForbidden,
-                          self.controller.delete,
-                          request,
-                          'fake_id')
-
-
-class TestTasksDeserializerPolicies(test_utils.BaseTestCase):
-
-    # NOTE(rosmaita): this is a bit weird, but we check the access
-    # policy in the RequestDeserializer for calls that take bodies
-    # or query strings because we want to make sure the failure is
-    # a 403, not a 400 due to bad request format
-    def setUp(self):
-        super(TestTasksDeserializerPolicies, self).setUp()
-        self.policy = unit_test_utils.FakePolicyEnforcer()
-        self.deserializer = glance.api.v2.tasks.RequestDeserializer(
-            schema=None, policy_engine=self.policy)
-
-    bad_path = '/tasks?limit=NaN'
-
-    def test_access_index_authorized_bad_query_string(self):
-        """Allow access, fail with 400"""
-        rules = {"tasks_api_access": True,
-                 "get_tasks": True}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request(self.bad_path)
-        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.index,
-                          request)
-
-    def test_access_index_unauthorized(self):
-        """Disallow access with bad request, fail with 403"""
-        rules = {"tasks_api_access": False,
-                 "get_tasks": True}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request(self.bad_path)
-        self.assertRaises(webob.exc.HTTPForbidden, self.deserializer.index,
-                          request)
-
-    bad_task = {'typo': 'import', 'input': {"import_from": "fake"}}
-
-    def test_access_create_authorized_bad_format(self):
-        """Allow access, fail with 400"""
-        rules = {"tasks_api_access": True,
-                 "add_task": True}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes(self.bad_task)
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.deserializer.create,
-                          request)
-
-    def test_access_create_unauthorized(self):
-        """Disallow access with bad request, fail with 403"""
-        rules = {"tasks_api_access": False,
-                 "add_task": True}
-        self.policy.set_rules(rules)
-        request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes(self.bad_task)
-        self.assertRaises(webob.exc.HTTPForbidden,
-                          self.deserializer.create,
-                          request)
-
 
 class TestTasksDeserializer(test_utils.BaseTestCase):
 
     def setUp(self):
         super(TestTasksDeserializer, self).setUp()
-        self.policy = unit_test_utils.FakePolicyEnforcer()
-        self.deserializer = glance.api.v2.tasks.RequestDeserializer(
-            policy_engine=self.policy)
+        self.deserializer = glance.api.v2.tasks.RequestDeserializer()
 
     def test_create_no_body(self):
         request = unit_test_utils.get_fake_request()
@@ -571,7 +359,7 @@ class TestTasksDeserializer(test_utils.BaseTestCase):
 
     def test_create(self):
         request = unit_test_utils.get_fake_request()
-        request.body = jsonutils.dump_as_bytes({
+        request.body = jsonutils.dumps({
             'type': 'import',
             'input': {'import_from':
                       'swift://cloud.foo/myaccount/mycontainer/path',
@@ -600,14 +388,14 @@ class TestTasksDeserializer(test_utils.BaseTestCase):
                     'sort_dir': 'desc',
                     'filters': {}}
         output = self.deserializer.index(request)
-        self.assertEqual(expected, output)
+        self.assertEqual(output, expected)
 
     def test_index_strip_params_from_filters(self):
         type = 'import'
         path = '/tasks?type=%s' % type
         request = unit_test_utils.get_fake_request(path)
         output = self.deserializer.index(request)
-        self.assertEqual(type, output['filters']['type'])
+        self.assertEqual(output['filters']['type'], type)
 
     def test_index_with_many_filter(self):
         status = 'success'
@@ -616,16 +404,16 @@ class TestTasksDeserializer(test_utils.BaseTestCase):
                                                            'type': type}
         request = unit_test_utils.get_fake_request(path)
         output = self.deserializer.index(request)
-        self.assertEqual(status, output['filters']['status'])
-        self.assertEqual(type, output['filters']['type'])
+        self.assertEqual(output['filters']['status'], status)
+        self.assertEqual(output['filters']['type'], type)
 
     def test_index_with_filter_and_limit(self):
         status = 'success'
         path = '/tasks?status=%s&limit=1' % status
         request = unit_test_utils.get_fake_request(path)
         output = self.deserializer.index(request)
-        self.assertEqual(status, output['filters']['status'])
-        self.assertEqual(1, output['limit'])
+        self.assertEqual(output['filters']['status'], status)
+        self.assertEqual(output['limit'], 1)
 
     def test_index_non_integer_limit(self):
         request = unit_test_utils.get_fake_request('/tasks?limit=blah')
@@ -663,17 +451,17 @@ class TestTasksDeserializer(test_utils.BaseTestCase):
         path = '/tasks?marker=%s' % marker
         request = unit_test_utils.get_fake_request(path)
         output = self.deserializer.index(request)
-        self.assertEqual(marker, output.get('marker'))
+        self.assertEqual(output.get('marker'), marker)
 
     def test_index_marker_not_specified(self):
         request = unit_test_utils.get_fake_request('/tasks')
         output = self.deserializer.index(request)
-        self.assertNotIn('marker', output)
+        self.assertFalse('marker' in output)
 
     def test_index_limit_not_specified(self):
         request = unit_test_utils.get_fake_request('/tasks')
         output = self.deserializer.index(request)
-        self.assertNotIn('limit', output)
+        self.assertFalse('limit' in output)
 
     def test_index_sort_key_id(self):
         request = unit_test_utils.get_fake_request('/tasks?sort_key=id')
@@ -683,7 +471,7 @@ class TestTasksDeserializer(test_utils.BaseTestCase):
             'sort_dir': 'desc',
             'filters': {}
         }
-        self.assertEqual(expected, output)
+        self.assertEqual(output, expected)
 
     def test_index_sort_dir_asc(self):
         request = unit_test_utils.get_fake_request('/tasks?sort_dir=asc')
@@ -692,7 +480,7 @@ class TestTasksDeserializer(test_utils.BaseTestCase):
             'sort_key': 'created_at',
             'sort_dir': 'asc',
             'filters': {}}
-        self.assertEqual(expected, output)
+        self.assertEqual(output, expected)
 
     def test_index_sort_dir_bad_value(self):
         request = unit_test_utils.get_fake_request('/tasks?sort_dir=invalid')
@@ -707,35 +495,21 @@ class TestTasksSerializer(test_utils.BaseTestCase):
         self.serializer = glance.api.v2.tasks.ResponseSerializer()
         self.fixtures = [
             _domain_fixture(UUID1, type='import', status='pending',
-                            task_input={'loc': 'fake'}, result={},
-                            owner=TENANT1, image_id='fake_image_id',
-                            user_id='fake_user',
-                            request_id='fake_request_id',
+                            input={'loc': 'fake'}, result={}, owner=TENANT1,
                             message='', created_at=DATETIME,
                             updated_at=DATETIME),
             _domain_fixture(UUID2, type='import', status='processing',
-                            task_input={'loc': 'bake'}, owner=TENANT2,
-                            image_id='fake_image_id',
-                            user_id='fake_user',
-                            request_id='fake_request_id',
-                            message='', created_at=DATETIME,
-                            updated_at=DATETIME, result={}),
+                            input={'loc': 'bake'}, owner=TENANT2, message='',
+                            created_at=DATETIME, updated_at=DATETIME,
+                            result={}),
             _domain_fixture(UUID3, type='import', status='success',
-                            task_input={'loc': 'foo'}, owner=TENANT3,
-                            image_id='fake_image_id',
-                            user_id='fake_user',
-                            request_id='fake_request_id',
-                            message='', created_at=DATETIME,
-                            updated_at=DATETIME, result={},
-                            expires_at=DATETIME),
+                            input={'loc': 'foo'}, owner=TENANT3, message='',
+                            created_at=DATETIME, updated_at=DATETIME,
+                            result={}, expires_at=DATETIME),
             _domain_fixture(UUID4, type='import', status='failure',
-                            task_input={'loc': 'boo'}, owner=TENANT4,
-                            image_id='fake_image_id',
-                            user_id='fake_user',
-                            request_id='fake_request_id',
-                            message='', created_at=DATETIME,
-                            updated_at=DATETIME, result={},
-                            expires_at=DATETIME),
+                            input={'loc': 'boo'}, owner=TENANT4, message='',
+                            created_at=DATETIME, updated_at=DATETIME,
+                            result={}, expires_at=DATETIME),
         ]
 
     def test_index(self):
@@ -789,7 +563,7 @@ class TestTasksSerializer(test_utils.BaseTestCase):
         }
         request = webob.Request.blank('/v2/tasks')
         response = webob.Response(request=request)
-        task_fixtures = [f for f in self.fixtures]
+        task_fixtures = [f['task'] for f in self.fixtures]
         result = {'tasks': task_fixtures}
         self.serializer.index(response, result)
         actual = jsonutils.loads(response.body)
@@ -799,7 +573,7 @@ class TestTasksSerializer(test_utils.BaseTestCase):
     def test_index_next_marker(self):
         request = webob.Request.blank('/v2/tasks')
         response = webob.Response(request=request)
-        task_fixtures = [f for f in self.fixtures]
+        task_fixtures = [f['task'] for f in self.fixtures]
         result = {'tasks': task_fixtures, 'next_marker': UUID2}
         self.serializer.index(response, result)
         output = jsonutils.loads(response.body)
@@ -809,19 +583,14 @@ class TestTasksSerializer(test_utils.BaseTestCase):
         url = '/v2/tasks?limit=10&sort_key=id&sort_dir=asc'
         request = webob.Request.blank(url)
         response = webob.Response(request=request)
-        task_fixtures = [f for f in self.fixtures]
+        task_fixtures = [f['task'] for f in self.fixtures]
         result = {'tasks': task_fixtures, 'next_marker': UUID2}
         self.serializer.index(response, result)
         output = jsonutils.loads(response.body)
-
-        expected_url = '/v2/tasks?limit=10&sort_dir=asc&sort_key=id'
-        self.assertEqual(unit_test_utils.sort_url_by_qs_keys(expected_url),
-                         unit_test_utils.sort_url_by_qs_keys(output['first']))
-
-        expect_next = '/v2/tasks?limit=10&marker=%s&sort_dir=asc&sort_key=id'
-        self.assertEqual(unit_test_utils.sort_url_by_qs_keys(
-                         expect_next % UUID2),
-                         unit_test_utils.sort_url_by_qs_keys(output['next']))
+        self.assertEqual('/v2/tasks?sort_key=id&sort_dir=asc&limit=10',
+                         output['first'])
+        expect_next = '/v2/tasks?sort_key=id&sort_dir=asc&limit=10&marker=%s'
+        self.assertEqual(expect_next % UUID2, output['next'])
 
     def test_get(self):
         expected = {
@@ -837,9 +606,6 @@ class TestTasksSerializer(test_utils.BaseTestCase):
             'expires_at': ISOTIME,
             'self': '/v2/tasks/%s' % UUID4,
             'schema': '/v2/schemas/task',
-            'image_id': 'fake_image_id',
-            'user_id': 'fake_user',
-            'request_id': 'fake_request_id',
         }
         response = webob.Response()
         self.serializer.get(response, self.fixtures[3])
@@ -860,9 +626,6 @@ class TestTasksSerializer(test_utils.BaseTestCase):
             'updated_at': ISOTIME,
             'self': '/v2/tasks/%s' % UUID1,
             'schema': '/v2/schemas/task',
-            'image_id': 'fake_image_id',
-            'user_id': 'fake_user',
-            'request_id': 'fake_request_id',
         }
         response = webob.Response()
         self.serializer.get(response, self.fixtures[0])
@@ -882,9 +645,6 @@ class TestTasksSerializer(test_utils.BaseTestCase):
             'updated_at': ISOTIME,
             'self': '/v2/tasks/%s' % UUID2,
             'schema': '/v2/schemas/task',
-            'image_id': 'fake_image_id',
-            'user_id': 'fake_user',
-            'request_id': 'fake_request_id',
         }
         response = webob.Response()
 
@@ -900,12 +660,14 @@ class TestTasksSerializer(test_utils.BaseTestCase):
         self.serializer.create(response, self.fixtures[3])
 
         serialized_task = jsonutils.loads(response.body)
-        self.assertEqual(http.CREATED, response.status_int)
-        self.assertEqual(self.fixtures[3].task_id,
+        self.assertEqual(response.status_int, 201)
+        self.assertEqual(self.fixtures[3]['task'].task_id,
                          serialized_task['id'])
-        self.assertEqual(self.fixtures[3].task_input,
+        self.assertEqual(self.fixtures[3]['task_details'].task_id,
+                         serialized_task['id'])
+        self.assertEqual(self.fixtures[3]['task_details'].input,
                          serialized_task['input'])
-        self.assertIn('expires_at', serialized_task)
+        self.assertTrue('expires_at' in serialized_task)
         self.assertEqual('application/json', response.content_type)
 
     def test_create_ensure_expires_at_is_not_returned(self):
@@ -914,12 +676,14 @@ class TestTasksSerializer(test_utils.BaseTestCase):
         self.serializer.create(response, self.fixtures[0])
 
         serialized_task = jsonutils.loads(response.body)
-        self.assertEqual(http.CREATED, response.status_int)
-        self.assertEqual(self.fixtures[0].task_id,
+        self.assertEqual(response.status_int, 201)
+        self.assertEqual(self.fixtures[0]['task'].task_id,
                          serialized_task['id'])
-        self.assertEqual(self.fixtures[0].task_input,
+        self.assertEqual(self.fixtures[0]['task_details'].task_id,
+                         serialized_task['id'])
+        self.assertEqual(self.fixtures[0]['task_details'].input,
                          serialized_task['input'])
-        self.assertNotIn('expires_at', serialized_task)
+        self.assertFalse('expires_at' in serialized_task)
         self.assertEqual('application/json', response.content_type)
 
         response = webob.Response()
@@ -927,10 +691,12 @@ class TestTasksSerializer(test_utils.BaseTestCase):
         self.serializer.create(response, self.fixtures[1])
 
         serialized_task = jsonutils.loads(response.body)
-        self.assertEqual(http.CREATED, response.status_int)
-        self.assertEqual(self.fixtures[1].task_id,
+        self.assertEqual(response.status_int, 201)
+        self.assertEqual(self.fixtures[1]['task'].task_id,
                          serialized_task['id'])
-        self.assertEqual(self.fixtures[1].task_input,
+        self.assertEqual(self.fixtures[1]['task_details'].task_id,
+                         serialized_task['id'])
+        self.assertEqual(self.fixtures[1]['task_details'].input,
                          serialized_task['input'])
-        self.assertNotIn('expires_at', serialized_task)
+        self.assertFalse('expires_at' in serialized_task)
         self.assertEqual('application/json', response.content_type)
